/**
 * FastIm v1.0.2
 * FastAdmin企业IM客服系统
 * https://www.fastadmin.net/store/fastim.html
 *
 * Copyright 2021 FastAdmin:thinkphp
 *
 * FastAdmin企业IM客服系统不是开源产品，所有文字、图片、样式、风格等版权归企业IM客服系统作者所有，如有复制、仿冒、抄袭、盗用，FastAdmin和企业IM客服系统作者将追究法律责任
 *
 * Released on: November 8, 2021
 */
// 音频播放初始化
window.AudioContext = window.AudioContext || window.webkitAudioContext || window.mozAudioContext || window.msAudioContext;

var FastIm = {
    ws: {
        socketTask: null,
        timer: null,
        errorMsg: [],
        maxReconnectCount: 3, // 最大重连次数
        currentReconnectCount: 0,
        needReconnect: true,
        ready: false
    },
    url: null, // 网站域名
    windowState: {
        fastimWindow: false,
        messageWindow: true
    },
    sessionId: 0,
    escape: new Set(), // ESC响应
    audio: {
        context: new window.AudioContext(),
        source: null,
        buffer: null
    },
    fullScreen: false,
    tempImages: [],
    initialize: function (url, initSuccess = null) {
        FastIm.url = url;

        // 发送初始化请求
        $.ajax({
            url: FastIm.build.url('initialize'),
            success: function (data) {
                FastIm.config = data.data.config;
                FastIm.tokens = data.data.tokens;
                FastIm.userinfo = data.data.userinfo;
                FastIm.onLoad(data.data.window);
                FastIm.config.initializeTime = data.time

                if (data.data.unreadMessage) {
                    FastIm.common.newMessageNotice(data.data.unreadMessage.sessionInfo.sessionUser.nickname, data.data.unreadMessage.messageData, '', data.data.unreadMessage.sessionInfo.type, false);
                } else if (!localStorage.getItem('fastim_non_first_visit')) {
                    // 新用户一句话欢迎提示
                    FastIm.common.newMessageNotice('', FastIm.config.new_user_tip, '', 'group', false);
                }

                if (FastIm.tokens.im_tourists_token) {
                    localStorage.setItem('im_tourists_token', FastIm.tokens.im_tourists_token);
                }

                if (FastIm.userinfo.identity == 'admin') {
                    // 立即连接 Websocket
                    FastIm.socket.connect()
                } else {
                    // 若用户 10 秒后任在此页面，连接Socket
                    setTimeout(function () {
                        FastIm.socket.connect()
                    }, 10000);
                    $('.fastim-orbit').addClass('link-success');
                }

                typeof initSuccess == 'function' && initSuccess()
                initSuccess = null
            },
            complete: function (e) {
                if (e.status != 200) {
                    FastIm.pop.msg('FastIm 初始化请求失败（状态代码:' + e.status + '）', 'error', false);
                }
            }
        });

        FastIm.eventReg();
    },
    onLoad: function (window) {
        $("body").append(window);
        if (FastIm.common.browserType() == "FF") {
            $('#fastim-message').attr('contenteditable', true);
        }
        if (parseInt(FastIm.config.user_config.send_message_key) == 0) {
            $('.fastim-send-button-tis').text('按下Enter发送，Ctrl+Enter换行')
        } else {
            $('.fastim-send-button-tis').text('按下Ctrl+Enter发送消息')
        }

        // 读取按钮位置
        var fastimButtonCoordinate = localStorage.getItem("fastim_button_coordinate");
        if (fastimButtonCoordinate) {
            fastimButtonCoordinate = fastimButtonCoordinate.split(',');
            if (fastimButtonCoordinate[0] &&
                fastimButtonCoordinate[1] &&
                $('body').width() > Number(fastimButtonCoordinate[1]) &&
                $('body').height() > Number(fastimButtonCoordinate[0])
            ) {
                $(".fastim-button-container").css({
                    "top": Number(fastimButtonCoordinate[0]),
                    "left": Number(fastimButtonCoordinate[1]),
                    "right": 'unset'
                });
            }
        }

        // 更新云存储multipart
        if (FastIm.config.upload.storage != 'local') {
            $.getScript(FastIm.build.url('spark'), function () {
                $(document).on('click', '.fastim-upload', function () {
                    var message = {
                        c: 'ImBase',
                        a: 'getUploadMultipart'
                    };
                    FastIm.socket.send(message);
                });

                FastIm.getFileMd5 = function (file, cb) {
                    var blobSlice = File.prototype.slice || File.prototype.mozSlice || File.prototype.webkitSlice,
                        chunkSize = 2097152,
                        chunks = Math.ceil(file.size / chunkSize),
                        currentChunk = 0,
                        spark = new $.fn.SparkMD5.ArrayBuffer(),
                        fileReader = new FileReader();

                    fileReader.onload = function (e) {
                        spark.append(e.target.result);
                        currentChunk++;
                        if (currentChunk < chunks) {
                            loadNext();
                        } else {
                            cb && cb(spark.end());
                        }
                    };

                    fileReader.onerror = function () {
                        FastIm.pop.msg('文件读取错误，请重试！')
                    };

                    function loadNext() {
                        var start = currentChunk * chunkSize,
                            end = ((start + chunkSize) >= file.size) ? file.size : start + chunkSize;

                        fileReader.readAsArrayBuffer(blobSlice.call(file, start, end));
                    }

                    loadNext();
                }
            });
        }

        // 初始化图片粘贴和拖拽上传(外站调用保证功能,不使用require)
        $.getScript(FastIm.build.url('pasteupload.js'), function () {
            $.fn.pasteText = function (text) {
                FastIm.pop.msg('正在上传...', 'tips', false)
            };

            $.fn.pasteUploadImage.defaults = $.extend(true, $.fn.pasteUploadImage.defaults, {
                fileName: "file",
                appendMimetype: false,
                uploadingText: '上传中...',
                ajaxOptions: {
                    url: FastIm.build.url('upload'),
                    beforeSend: function (jqXHR, settings) {
                        $.each(FastIm.config.upload.multipart, function (i, j) {
                            settings.data.append(i, j);
                        });
                        return true;
                    }
                },
                success: function (data, filename, file) {
                    var ret = FastIm.common.onUploadResponse(data);
                    if (ret.code == 0) {
                        FastIm.socket.send({
                            c: 'ImBase',
                            a: 'getUploadMultipart'
                        });
                        FastIm.pop.msg(ret.msg + ' 请重试！', 'error');
                        return false;
                    }

                    var imgNode = document.createElement('img');
                    imgNode.src = ret.data.fullurl;
                    imgNode.className = 'record-img';
                    imgNode.setAttribute('draggable', 'false')
                    FastIm.common.cursorInsert(imgNode)
                    FastIm.pop.hideMsg();

                    // 发送notify请求
                    if (FastIm.config.upload.storage != 'local') {
                        FastIm.getFileMd5(file, function (md5) {
                            var ajaxData = {
                                size: file.size,
                                name: file.name,
                                md5: md5,
                                type: file.type,
                                url: ret.data.url
                            };

                            for (let i in FastIm.config.upload.multipart) {
                                ajaxData[i] = FastIm.config.upload.multipart[i];
                            }

                            $.ajax({
                                url: FastIm.build.url('storage-notify'),
                                type: 'POST',
                                data: ajaxData
                            });
                        })
                    }
                    return false;
                },
                error: function (data, filename, file) {
                    console.log(data, filename, file);
                    FastIm.pop.msg('图片上传失败，请重试~', 'error');
                }
            });

            $('#fastim-message').pasteUploadImage();
        });
    },
    /**
     * ws消息处理
     */
    onMessage: function (msg) {
        var msgFun = new Map([
            ['open', () => {
                FastIm.socket.send({
                    c: 'ImBase',
                    a: 'login',
                    data: {
                        'tokens': FastIm.tokens,
                        'identity': FastIm.userinfo.identity,
                        'platform': 'web',
                        'initialize_time': FastIm.config.initializeTime
                    }
                });
            }],
            ['login_success', () => {
                // 标记连接成功
                FastIm.ws.ready = true;
                FastIm.pop.hideMsg()
                $('.fastim-orbit').addClass('link-success')
                $('.fastim-sidebar-user-status').attr('class', 'fastim-sidebar-user-status fastim-click user-status-' + msg.data.status);

                if (msg.data.unreadMessage) {
                    FastIm.common.newMessageNotice(msg.data.unreadMessage.sessionInfo.sessionUser.nickname, msg.data.unreadMessage.messageData, '', msg.data.unreadMessage.sessionInfo.type, false);
                }

                // 获取会话列表
                let message = {c: 'User', a: 'loadSessionList'};
                FastIm.socket.send(message);
            }],
            ['show_msg', () => {
                if (msg.data) {
                    if ($.isPlainObject(msg.data)) {
                        FastIm.pop.msg(msg.data.msg, msg.data.type);
                    } else {
                        FastIm.pop.msg(msg.data);
                    }
                }
                if (msg.close) {
                    console.log('收服务端要求禁止自动重新连接ws！');
                    FastIm.ws.needReconnect = false;
                }
            }],
            ['pong', () => {
            }],
            ['change_status', () => {
                $('.fastim-sidebar-user-status').attr('class', 'fastim-sidebar-user-status fastim-click user-status-' + msg.data.status);
                if (msg.data.status == 0) {
                    FastIm.ws.ready = false;
                    FastIm.userinfo.status = -1;
                }
            }],
            ['update_status', () => {
                // 更新好友的状态
                var sessionElement = $('.fastim-session-li[data-data="' + msg.data.session_id + '"]'),
                    status = parseInt(msg.data.status)
                if (status == 0 || status == 2) {
                    var statusText = '',
                        statusEL = sessionElement.find('.session-user-status')
                    if (status == 0) {
                        statusText = '[离线]'
                        sessionElement.find('.session-avatar').addClass('fastim-img-gray')
                    } else {
                        statusText = '[忙碌]'
                        sessionElement.find('.session-avatar').removeClass('fastim-img-gray')
                    }
                    if (statusEL.length) {
                        statusEL.eq(0).text(statusText)
                    } else {
                        sessionElement.find('.session-last-message').eq(0).prepend('<span class="session-user-status">' + statusText + '</span>')
                    }
                } else if (status == 1) {
                    sessionElement.find('.session-user-status').eq(0).remove()
                    sessionElement.find('.session-avatar').removeClass('fastim-img-gray')
                }
            }],
            ['input_status', () => {
                if (FastIm.sessionId == msg.data.session_id) {
                    (msg.data.status == 'start') ? $('.fastim-input-status').fadeIn() : $('.fastim-input-status').fadeOut()
                }
            }],
            ['load_to_do', () => {
                FastIm.loadTODO = msg.data.data.nextpage ? msg.data.data : false;
                if (parseInt(msg.data.data.page) == 1) {
                    $('.fastim-to-do .to-do-list .to-do-incomplete').empty()
                    $('.fastim-to-do .to-do-list .to-do-completed').empty()
                    $('.fastim-to-do .fastim-popup-title').html('待办 • ' + msg.data.data.incomplete)
                    $('.fastim-to-do .to-do-more').data('completed', msg.data.data.completed).html('显示已完成待办 • ' + msg.data.data.completed)

                    if (msg.data.incomplete.length) {
                        for (var i = 0; i < msg.data.incomplete.length; i++) {
                            $('.fastim-to-do .to-do-list .to-do-incomplete').append(FastIm.build.widget('to-do-item', msg.data.incomplete[i]))
                        }
                    }
                }
                if (msg.data.completed.length) {
                    for (var i = 0; i < msg.data.completed.length; i++) {
                        msg.data.completed[i].checked = 'checked';
                        $('.fastim-to-do .to-do-list .to-do-completed').append(FastIm.build.widget('to-do-item', msg.data.completed[i]))
                    }
                }
            }],
            ['fast_reply', () => {
                if (msg.data.data.method == 'edit') {
                    FastIm.showWindow('edit-fast-reply')
                    $('.edit-fast-reply input[name="id"]').val(msg.data.info.id)
                    $('.edit-fast-reply input[name="title"]').val(msg.data.info.title)
                    $('.edit-fast-reply textarea[name="content"]').val(msg.data.info.content)
                    $('.edit-fast-reply select[name="status"] option[value="' + msg.data.info.status + '"]').prop('selected', 'selected')
                    return;
                }
                FastIm.loadFastReply = msg.data.data.nextpage ? msg.data.data : false;
                FastIm.loadFastReply.method = 'get';
                var fastimReplyEl = $('.fast-reply-panel .fast-reply-tbody');
                if (parseInt(msg.data.data.page) == 1) {
                    fastimReplyEl.empty()
                }
                if (msg.data.fast_reply_list.length) {
                    for (var i = 0; i < msg.data.fast_reply_list.length; i++) {
                        fastimReplyEl.append(FastIm.build.widget('fast-reply-item', msg.data.fast_reply_list[i]))
                    }

                    if (!msg.data.data.nextpage) {
                        fastimReplyEl.append('<div class="fastim-data-none">没有更多快捷回复了~</div>');
                    }
                } else {
                    var tis = (parseInt(msg.data.data.page) > 1) ? '没有更多快捷回复了~' : '还没有快捷回复~'
                    if ($('.fast-reply-panel .fast-reply-tbody .fastim-data-none').length) {
                        $('.fast-reply-panel .fast-reply-tbody .fastim-data-none').html(tis)
                    } else {
                        fastimReplyEl.append('<div class="fastim-data-none">' + tis + '</div>');
                    }
                }
            }],
            ['config', () => {
                var blockContacts = $('select[name="fastim-block-contacts"]')
                blockContacts.empty()
                for (var i = 0; i < msg.data.shield_users.length; i++) {
                    blockContacts.append('<option value="' + msg.data.shield_users[i].session_id + '">' + msg.data.shield_users[i].nickname + '</option>');
                }
            }],
            ['shield_session', () => {
                if (msg.data.method == 'shield') {
                    $('.fastim-session-li[data-data="' + msg.data.session_id + '"]').remove()
                    if ($('.fastim-session-li').length) {
                        $('.fastim-session-li').eq(0).trigger('click')
                    } else {
                        $('.fastim-session-ul').append('<li class="fastim-data-none">暂时没有会话~</li>')
                        $('.fastim-window-right .chat-window-title .chat-nickname').text('无会话')
                        $('.chat-records').empty()
                        FastIm.loadRecordsData = false;
                        FastIm.hideWindow('message-window')
                    }
                } else if (msg.data.method == 'relieve') {
                    if (msg.data.session_id == FastIm.sessionId) {
                        $('.fastim-window-right .message-window .shield').text('屏蔽会话').removeClass('fastim-color-red')
                    }

                    $('select[name="fastim-block-contacts"] option[value="' + msg.data.session_id + '"]').remove()
                    $('select[name="fastim-block-contacts"] option:first').prop('selected', 'selected')
                    FastIm.pop.msg('解除屏蔽成功~', 'success')
                }
            }],
            ['search_new_contact', () => {
                FastIm.newContactData = msg.data.nextpage ? msg.data.data : false;
                if (parseInt(msg.data.data.page) <= 1) {
                    $('.new-' + msg.data.data.type + '-panel .new-contact-list-flex').html('');
                }
                $('.new-contact-list').hide();
                $('.new-' + msg.data.data.type + '-panel.new-contact-list').fadeIn(100);
                if (msg.data.res.length) {
                    for (var i = 0; i < msg.data.res.length; i++) {
                        msg.data.res[i].avatar = FastIm.common.imgUrl(msg.data.res[i].avatar);
                        if (msg.data.data.type == 'group') {
                            msg.data.res[i].type = 'group'
                            msg.data.res[i].gender_icon = FastIm.common.imgUrl('/assets/addons/fastim/icon/group.png');
                            msg.data.res[i].status = msg.data.res[i].user_count + '/' + msg.data.res[i].max_user_count

                            if (msg.data.res[i].is_group_user) {
                                msg.data.res[i].button = '<button class="user-card-create fastim-click" data-action="open-session" data-type="group" data-data="' + msg.data.res[i].id + '">发消息</button>';
                            } else {
                                msg.data.res[i].button = '<button class="user-card-create fastim-click" data-action="userinfo-opt" data-data="' + msg.data.res[i].id + '" data-opt="join" data-type="group">加入</button>';
                            }
                        } else {
                            msg.data.res[i].type = 'user'
                            msg.data.res[i].gender_icon = FastIm.common.imgUrl('/assets/addons/fastim/icon/' + msg.data.res[i].gender.value + '.png');
                            msg.data.res[i].gender_title = '性别:' + msg.data.res[i].gender.chinese
                            msg.data.res[i].status = msg.data.res[i].status.chinese

                            if (msg.data.res[i].is_friend) {
                                // 发送消息
                                msg.data.res[i].button = '<button class="user-card-create fastim-click" data-action="open-session" data-type="user" data-data="' + msg.data.res[i].id + '">发消息</button>';
                            } else {
                                // 加为好友
                                msg.data.res[i].button = '<button class="user-card-create fastim-click" data-action="add-friends" data-data="' + msg.data.res[i].id + '">加好友</button>';
                            }
                        }

                        var newHTML = FastIm.build.widget('user-card', msg.data.res[i])
                        $('.new-' + msg.data.data.type + '-panel .new-contact-list-flex').append(newHTML);
                    }

                    if (!msg.data.nextpage) {
                        $('.new-' + msg.data.data.type + '-panel .new-contact-list-flex').append('<div class="new-' + msg.data.data.type + '-no-data fastim-data-none">没有更多结果了~</div>');
                    }
                } else {
                    var tis = (parseInt(msg.data.data.page) > 1) ? '没有更多结果了~' : '无搜索结果~'
                    if ($('.new-contact-list-flex .new-' + msg.data.data.type + '-no-data').length) {
                        $('.new-contact-list-flex .new-' + msg.data.data.type + '-no-data').html(tis);
                    } else {
                        $('.new-' + msg.data.data.type + '-panel .new-contact-list-flex').append('<div class="new-' + msg.data.data.type + '-no-data fastim-data-none">' + tis + '</div>');
                    }
                }
            }],
            ['send_message', () => {
                if (msg.data.sendToYourself) {
                    /**
                     * sendToYourself:是否是发送给自己的
                     * 发送给自己的任然需推送给自己的其他在线设备
                     * 此处做标记以在前端删除可能存在的原消息:`message_id`
                     */
                    $('.fastim-window-right .chat-record-item[data-id="' + msg.data.message_id + '"]').remove()
                    return;
                }
                FastIm.common.messageStatus(msg.data.status, msg.data.message_id, true)
                $('.fastim-window-right .chat-record-item[data-id="' + msg.data.message_id + '"]').data('id', msg.data.new_message_id).attr('data-id', msg.data.new_message_id);
            }],
            ['load_session_list', () => {
                var session = msg.data.session
                if (msg.data.data.method == 'load') {
                    var sessionTemplate = FastIm.template.session();
                    if (session.length) {
                        $('.fastim-session-ul').html('')
                        for (var i = 0; i < session.length; i++) {

                            if (session[i].sessionInfo.pushUser.status) {
                                if (parseInt(session[i].sessionInfo.pushUser.status.value) == 0) {
                                    session[i].sessionInfo.pushUser.avatar_gray = 'fastim-img-gray'
                                    session[i].sessionInfo.pushUser.user_status = '<span class="session-user-status">[离线]</span>'
                                } else if (parseInt(session[i].sessionInfo.pushUser.status.value) == 2) {
                                    session[i].sessionInfo.pushUser.user_status = '<span class="session-user-status">[忙碌]</span>'
                                }
                            }

                            session[i].shield = session[i].shield ? 'session-shield' : ''
                            session[i].unreadMessagesNumber = session[i].unreadMessagesNumber > 0 ? '<span class="session-unread-count ' + session[i].shield + '">' + session[i].unreadMessagesNumber + '</span>' : ''
                            session[i].sessionInfo.pushUser.avatar = FastIm.common.imgUrl(session[i].sessionInfo.pushUser.avatar)
                            session[i].sessionInfo.top = session[i].sessionInfo.top ? 'session-top' : ''
                            if (session[i].lastMessage.unread_fixed_msg) {
                                session[i].lastMessage.unread_fixed_msg.message = '[' + session[i].lastMessage.unread_fixed_msg.message + ']'
                            }
                            var newHTML = sessionTemplate(session[i])
                            $('.fastim-session-ul').append(newHTML)
                        }

                        // 注册一个在打开窗口时执行的回调
                        FastIm.showFastimWindowBack = function () {
                            if (FastIm.sessionTempId) {
                                let sessionTempEl = $('.fastim-session-li[data-data="' + FastIm.sessionTempId + '"]')
                                if (sessionTempEl.length) {
                                    sessionTempEl.trigger('click')
                                } else {
                                    $('.fastim-session-li').eq(0).trigger('click')
                                }
                                FastIm.sessionTempId = null
                            } else {
                                $('.fastim-session-li').eq(0).trigger('click')
                            }
                        }
                        if (FastIm.windowState.fastimWindow) {
                            typeof FastIm.showFastimWindowBack == 'function' && FastIm.showFastimWindowBack()
                            FastIm.showFastimWindowBack = null
                        }
                    }
                } else if (msg.data.data.method == 'pickuser') {
                    $('.fastim-pickuser .pick-middle .pick-widget').empty()
                    if (session.length) {
                        for (var i = 0; i < session.length; i++) {
                            session[i].groupId = 'lately'
                            $.extend(session[i], session[i].sessionInfo.pushUser)
                            $('.fastim-pickuser .pick-middle .pick-widget').append(FastIm.build.widget('pickuser', session[i]))
                        }
                    }
                }
            }],
            ['reload_session_list', () => {
                // 重新加载会话列表
                FastIm.sessionTempId = $('.fastim-session-li.active').data('data')
                let message = {c: 'User', a: 'loadSessionList'};
                FastIm.socket.send(message);
            }],
            ['load_session', () => {

                // 保障选择的是加载到的会话
                let liEl = $('.fastim-session-li[data-data="' + msg.data.info.id + '"]');
                $('.fastim-session-li.active').removeClass('active')
                liEl.addClass('active');

                if (msg.data.info.windowType == 'fake_tourist') {
                    FastIm.showWindow('fake-tourist')
                    if (parseInt(msg.data.admin) > 0) {
                        // 管理员离线了,登录链接为刷新当前页
                        $('.fake-tourist-login-a').attr('href', 'javascript:location.reload();')
                    }
                } else if (msg.data.info.windowType == 'message') {
                    var messageTemplate = FastIm.template.message()
                    FastIm.showWindow('message-window')
                    var recordsEL = $('.chat-records'),
                        page = parseInt(msg.data.data.page);

                    let chatNicknameEl = $('.message-window .chat-window-title .chat-nickname')
                    chatNicknameEl.text(msg.data.info.sessionUser.nickname + (msg.data.info.sessionUser.user_count ? '(' + msg.data.info.sessionUser.user_count + ')' : ''))
                    chatNicknameEl.data('session-user', msg.data.info.sessionUser.id).attr('data-session-user', msg.data.info.sessionUser.id)
                    let userType = (msg.data.info.type == 'single') ? 'user' : 'group';
                    chatNicknameEl.data('type', userType).attr('data-type', userType)
                    $('.message-window .report').show()

                    if (msg.data.info.type == 'single') {
                        msg.data.info.sessionUser.friend ? $('.message-window .add-friends').hide() : $('.message-window .add-friends').data('data', msg.data.info.sessionUser.id).show()
                        msg.data.info.shield ? $('.message-window .shield').text('解除屏蔽').addClass('fastim-color-red').show() : $('.message-window .shield').text('屏蔽会话').removeClass('fastim-color-red').show()
                    } else {
                        $('.message-window .shield').hide()
                        $('.message-window .add-friends').hide()

                        // 按需弹窗群公告
                        FastIm.common.showNewGroupChatNotice(msg.data.unreadFixedMsg);
                    }

                    FastIm.loadRecordsData = msg.data.nextpage ? msg.data.data : false
                    if (page == 1) {
                        $('.chat-records').html('')
                    }

                    if (msg.data.message.length) {

                        for (var i = 0; i < msg.data.message.length; i++) {
                            if (page == 1) {
                                recordsEL.append('<li class="chat-record-item system"><span>' + msg.data.message[i].datetime + '</span></li>');
                            }
                            for (var y = 0; y < msg.data.message[i].data.length; y++) {

                                var pushUser = {} // 消息上需要的用户数据

                                if (msg.data.message[i].data[y].type == 'system') {
                                    if (msg.data.message[i].data[y].message.display_user == 'all' || msg.data.message[i].data[y].message.display_user == FastIm.userinfo.id) {
                                        var newHTML = '<li class="chat-record-item system"><span>' + msg.data.message[i].data[y].message.message + '</span></li>';
                                    } else {
                                        continue;
                                    }
                                } else {
                                    msg.data.message[i].data[y].message = FastIm.build.message(msg.data.message[i].data[y])
                                    if (msg.data.info.type == 'single') {
                                        // 单聊消息
                                        if (msg.data.message[i].data[y].sender == 'me') {
                                            pushUser.id = msg.data.info.user.id
                                            pushUser.avatar = FastIm.common.imgUrl(msg.data.info.user.avatar)

                                            msg.data.message[i].data[y].status = FastIm.common.messageStatus(msg.data.message[i].data[y].status, msg.data.message[i].data[y].id)
                                        } else {
                                            pushUser.id = msg.data.info.sessionUser.id
                                            pushUser.avatar = FastIm.common.imgUrl(msg.data.info.sessionUser.avatar)

                                            msg.data.message[i].data[y].status = FastIm.common.messageStatus(3, msg.data.message[i].data[y].id)
                                        }
                                        pushUser.hidden_nickname = 'fastim-hidden'
                                        msg.data.message[i].data[y].record_content_before = 'record_content_before'
                                    } else {
                                        // 群聊消息
                                        if (msg.data.message[i].data[y].sender_id == FastIm.userinfo.id) {
                                            pushUser.hidden_nickname = 'fastim-hidden'
                                            msg.data.message[i].data[y].record_content_before = 'record_content_before'

                                            if (msg.data.message[i].data[y].read_number >= (msg.data.info.sessionUser.user_count - 1)) {
                                                msg.data.message[i].data[y].status = FastIm.common.messageStatus(6, msg.data.message[i].data[y].id);
                                            } else if (msg.data.message[i].data[y].read_number > 0) {
                                                msg.data.message[i].data[y].status = FastIm.common.messageStatus(5, msg.data.message[i].data[y].id, false, msg.data.message[i].data[y].read_number)
                                            } else {
                                                msg.data.message[i].data[y].status = FastIm.common.messageStatus(msg.data.message[i].data[y].status, msg.data.message[i].data[y].id)
                                            }
                                        }

                                        pushUser.id = msg.data.message[i].data[y].userInfo.id
                                        pushUser.nickname = msg.data.message[i].data[y].userInfo.nickname
                                        pushUser.avatar = FastIm.common.imgUrl(msg.data.message[i].data[y].userInfo.avatar)
                                    }
                                    var newHTML = messageTemplate({
                                        messageData: msg.data.message[i].data[y],
                                        sessionInfo: {
                                            pushUser: pushUser
                                        }
                                    });
                                }
                                (page == 1) ? recordsEL.append(newHTML) : recordsEL.prepend(newHTML)
                            }
                            if (page > 1) {
                                recordsEL.prepend('<li class="chat-record-item system"><span>' + msg.data.message[i].datetime + '</span></li>');
                            }
                        }

                        setTimeout(function () {
                            (page == 1) ? recordsEL.scrollTop(recordsEL[0].scrollHeight) : recordsEL.scrollTop(recordsEL[0].scrollHeight - FastIm.recordScrollHeight)
                        }, 100)
                    } else if (page <= 1) {
                        recordsEL.append('<div class="fastim-data-none">暂时没有聊天记录~</div>');
                    }

                    var messageEl = $('#fastim-message')
                    if (msg.data.info.sessionUser.speak && parseInt(msg.data.info.sessionUser.speak) == 1) {
                        // 禁止发言
                        messageEl.attr('contenteditable', false).addClass('no-speak').append('管理员已禁止发言');
                        FastIm.fastImMessageContenteditable = true;
                    } else {
                        // 消息输入框聚焦
                        FastIm.fastImMessageContenteditable = false;
                        FastIm.common.browserType() == "FF" ? messageEl.attr('contenteditable', true) : messageEl.attr('contenteditable', 'plaintext-only')
                        if (page == 1) {
                            messageEl.removeClass('no-speak').empty();
                        } else {
                            messageEl.removeClass('no-speak');
                        }
                        setTimeout(function () {
                            messageEl.trigger('focus')
                        }, 200)
                    }
                } else if (msg.data.info.windowType == 'new_friends_message' || msg.data.info.windowType == 'group_notice') {
                    // 修改会话最后消息
                    var sessionElement = $('.fastim-session-li[data-data="' + msg.data.info.id + '"]')
                    if (sessionElement.length) {
                        sessionElement.find(".session-info-time").eq(0).html(msg.data.lastMessage.last_time)
                        sessionElement.find(".last-message-text .last-message").eq(0).html(msg.data.lastMessage.last_message)
                    }

                    var serviceMessageTemplate = FastIm.template.serviceMessage(),
                        serviceMessageEl = $('.fastim-window-right .service-message'),
                        page = parseInt(msg.data.data.page);

                    FastIm.showWindow('service-window')
                    let chatNicknameEl = $('.service-window .chat-window-header .chat-nickname');
                    chatNicknameEl.text(msg.data.info.sessionUser.nickname).data('type', 'service').attr('data-type', 'service')
                    chatNicknameEl.data('session-user', msg.data.info.sessionUser.id).attr('data-session-user', msg.data.info.sessionUser.id)

                    FastIm.serviceMessageData = msg.data.nextpage ? msg.data.data : false
                    if (page <= 1) {
                        serviceMessageEl.empty()
                    }

                    if (msg.data.message.length) {
                        for (var i = 0; i < msg.data.message.length; i++) {
                            serviceMessageEl.append('<div class="service-message-datatime">' + msg.data.message[i].datetime + '</div>');

                            for (var y = 0; y < msg.data.message[i].data.length; y++) {
                                if (msg.data.info.windowType == 'group_notice') {
                                    // 群通知
                                    var newHTML = serviceMessageTemplate(FastIm.build.serviceMessageInfo({
                                        messageData: msg.data.message[i].data[y]
                                    }, 'group_notice', {
                                        userInfo: msg.data.message[i].data[y].userInfo,
                                        groupChatInfo: msg.data.message[i].data[y].groupChatInfo
                                    }));
                                } else {
                                    var newHTML = serviceMessageTemplate(FastIm.build.serviceMessageInfo({
                                        messageData: msg.data.message[i].data[y]
                                    }, 'new_friends_message', {
                                        userInfo: msg.data.message[i].data[y].userInfo
                                    }));
                                }
                                serviceMessageEl.append(newHTML);
                            }
                        }

                        if (!msg.data.nextpage) {
                            serviceMessageEl.append('<div class="fastim-data-none">没有更多消息了~</div>');
                        }
                    } else {
                        var tis = (page > 1) ? '没有更多消息了~' : '暂时没有消息~'
                        if ($('.service-message .fastim-data-none').length) {
                            $('.service-message .fastim-data-none').html(tis)
                        } else {
                            serviceMessageEl.append('<div class="fastim-data-none">' + tis + '</div>');
                        }
                    }
                } else if (msg.data.info.windowType == 'csr_message') {
                    var messageTemplate = FastIm.template.message()
                    FastIm.showWindow('message-window')
                    var recordsEL = $('.chat-records'),
                        page = parseInt(msg.data.data.page);

                    let chatNicknameEl = $('.message-window .chat-window-title .chat-nickname')
                    chatNicknameEl.text(msg.data.info.sessionUser.nickname).data('type', 'service').attr('data-type', 'service')
                    chatNicknameEl.data('session-user', msg.data.info.sessionUser.id).attr('data-session-user', msg.data.info.sessionUser.id)

                    $('.message-window .shield, .message-window .add-friends, .message-window .report').hide()

                    FastIm.loadRecordsData = msg.data.nextpage ? msg.data.data : false
                    if (page == 1) {
                        $('.chat-records').html('')
                    }

                    if (msg.data.message.length) {
                        for (var i = 0; i < msg.data.message.length; i++) {
                            if (page == 1) {
                                recordsEL.append('<li class="chat-record-item system"><span>' + msg.data.message[i].datetime + '</span></li>');
                            }
                            for (var y = 0; y < msg.data.message[i].data.length; y++) {
                                var pushUser = {} // 消息上需要的用户数据

                                if (msg.data.message[i].data[y].type == 'system') {
                                    if (msg.data.message[i].data[y].message.display_user == 'all' || msg.data.message[i].data[y].message.display_user == FastIm.userinfo.id) {
                                        var newHTML = '<li class="chat-record-item system"><span>' + msg.data.message[i].data[y].message.message + '</span></li>';
                                    } else {
                                        continue;
                                    }
                                } else {
                                    msg.data.message[i].data[y].message = FastIm.build.message(msg.data.message[i].data[y])
                                    if (msg.data.message[i].data[y].sender == 'me') {
                                        pushUser.id = msg.data.info.user.id
                                        pushUser.hidden_nickname = 'fastim-hidden'
                                        pushUser.avatar = FastIm.common.imgUrl(msg.data.info.user.avatar)

                                        msg.data.message[i].data[y].record_content_before = 'record_content_before'
                                        msg.data.message[i].data[y].status = FastIm.common.messageStatus(msg.data.message[i].data[y].status, msg.data.message[i].data[y].id)
                                    } else {
                                        if (msg.data.message[i].data[y].csr) {
                                            pushUser = msg.data.message[i].data[y].csr
                                            pushUser.avatar = FastIm.common.imgUrl(msg.data.message[i].data[y].csr.avatar)
                                        } else {
                                            pushUser.type = 'service'
                                            pushUser.id = msg.data.info.sessionUser.id
                                            pushUser.avatar = FastIm.common.imgUrl(msg.data.info.sessionUser.avatar)
                                            pushUser.nickname = msg.data.info.sessionUser.nickname
                                        }

                                        msg.data.message[i].data[y].status = FastIm.common.messageStatus(3, msg.data.message[i].data[y].id)
                                    }
                                    var newHTML = messageTemplate({
                                        messageData: msg.data.message[i].data[y],
                                        sessionInfo: {
                                            pushUser: pushUser
                                        }
                                    });
                                }
                                (page == 1) ? recordsEL.append(newHTML) : recordsEL.prepend(newHTML)
                            }
                            if (page > 1) {
                                recordsEL.prepend('<li class="chat-record-item system"><span>' + msg.data.message[i].datetime + '</span></li>');
                            }
                        }

                        setTimeout(function () {
                            (page == 1) ? recordsEL.scrollTop(recordsEL[0].scrollHeight) : recordsEL.scrollTop(recordsEL[0].scrollHeight - FastIm.recordScrollHeight)
                        }, 100)
                    } else if (page <= 1) {
                        recordsEL.append('<div class="fastim-data-none">暂时没有聊天记录~</div>');
                    }

                    // 消息输入框聚焦
                    var messageEl = $('#fastim-message')
                    FastIm.fastImMessageContenteditable = false;
                    FastIm.common.browserType() == "FF" ? messageEl.attr('contenteditable', true) : messageEl.attr('contenteditable', 'plaintext-only')
                    if (page == 1) {
                        messageEl.removeClass('no-speak').empty()
                    } else {
                        messageEl.removeClass('no-speak');
                    }
                    setTimeout(function () {
                        messageEl.trigger('focus')
                    }, 200)
                }
            }],
            ['get_groups', () => {
                if (msg.data.data.type == 'pick-contacts') {
                    for (var i = 0; i < msg.data.res.friends.length; i++) {
                        msg.data.res.friends[i].groupId = 'pickgroup'
                        $('.fastim-pickuser .pick-middle .pick-widget').append(FastIm.build.widget('pickgroup', msg.data.res.friends[i]))
                    }
                } else if (msg.data.data.type == 'contacts') {
                    let activeID = $('.fastim-address-list .address-group-box.active').data('id')
                    $('.fastim-address-list .address-friends-panel, .address-group-panel').html('')
                    var {friends, group} = msg.data.res
                    for (let i = 0; i < friends.length; i++) {
                        $('.fastim-address-list .address-friends-panel').append('<div data-action="load-group-data" data-id="' + friends[i].id + '" class="fastim-group-box group-id-' + friends[i].id + ' address-group-box fastim-click">' + friends[i].name + '</div>');
                    }
                    for (let i = 0; i < group.length; i++) {
                        $('.fastim-address-list .address-group-panel').append('<div data-action="load-group-data" data-id="' + group[i].id + '" class="fastim-group-box group-id-' + group[i].id + ' address-group-box fastim-click">' + group[i].name + '</div>');
                    }
                    if (activeID) {
                        $('.fastim-address-list .fastim-group-box.group-id-' + activeID).addClass('active');
                    }
                } else if (msg.data.data.type == 'collection') {
                    let activeID = $('.fastim-groups .collection-group-box.active').data('id')
                    var collection = msg.data.res.collection
                    $('.fastim-groups .collection-group').html('')
                    for (let i = 0; i < collection.length; i++) {
                        $('.collection-group').append('<div data-action="load-group-data" data-id="' + collection[i].id + '" class="fastim-group-box group-id-' + collection[i].id + ' collection-group-box fastim-click">' + collection[i].name + '</div>');
                    }
                    if (activeID) {
                        $('.fastim-groups .fastim-group-box.group-id-' + activeID).addClass('active');
                    }
                }
            }],
            ['load_group', () => {
                // 分组内的好友和群组/收藏
                if (msg.data.data.method == 'pick-group-user') {
                    // 选择好友或群组
                    $('.fastim-pickuser .pick-middle .pick-widget .pick-item').hide()
                    var widget = $('.fastim-pickuser .pick-middle .pick-widget div[data-data="' + msg.data.data.id + '"]')

                    for (var i = 0; i < msg.data.res.length; i++) {
                        msg.data.res[i].id = msg.data.res[i].friend_id
                        msg.data.res[i].groupId = msg.data.data.id
                        widget.after(FastIm.build.widget('pickuser', msg.data.res[i]))
                    }
                } else if (msg.data.data.method == 'group-user') {
                    $('.fastim-contacts-window,.contacts-window-box').fadeIn()
                    $('.fastim-contacts-window .contacts-title').text(msg.data.group.name)
                    FastIm.loadGroup = msg.data.nextpage ? msg.data.data : false
                    var contactsListEl = $('.fastim-contacts-window .contacts-list'),
                        typeName = (msg.data.data.type == 'user') ? '好友' : '群聊';
                    if (parseInt(msg.data.data.page) <= 1) {
                        contactsListEl.empty()
                    }

                    if (msg.data.res.length) {
                        for (var i = 0; i < msg.data.res.length; i++) {
                            msg.data.res[i].avatar = FastIm.common.imgUrl(msg.data.res[i].avatar)
                            if (msg.data.data.type == 'user') {
                                msg.data.res[i].type = 'user'
                                msg.data.res[i].id = msg.data.res[i].friend_id
                                msg.data.res[i].gender_icon = FastIm.common.imgUrl('/assets/addons/fastim/icon/' + msg.data.res[i].gender.value + '.png');
                                msg.data.res[i].gender_title = '性别:' + msg.data.res[i].gender.chinese
                                msg.data.res[i].status = msg.data.res[i].status.chinese

                                // 发送消息
                                msg.data.res[i].button = '<button class="user-card-create fastim-click" data-action="open-session" data-type="user" data-data="' + msg.data.res[i].id + '">发消息</button>';
                            } else {
                                msg.data.res[i].type = 'group'
                                msg.data.res[i].gender_icon = FastIm.common.imgUrl('/assets/addons/fastim/icon/group.png');
                                msg.data.res[i].status = msg.data.res[i].user_count + '/' + msg.data.res[i].max_user_count
                                msg.data.res[i].button = '<button class="user-card-create fastim-click" data-action="open-session" data-type="group" data-data="' + msg.data.res[i].id + '">发消息</button>';
                            }

                            msg.data.res[i].idtype = msg.data.res[i].id + msg.data.res[i].type
                            var newHTML = FastIm.build.widget('user-card', msg.data.res[i]);
                            contactsListEl.append(newHTML);
                        }

                        if (!msg.data.nextpage) {
                            contactsListEl.append('<div class="fastim-data-none">此分组内没有更多' + typeName + '了~</div>');
                        }
                    } else {
                        var tis = (parseInt(msg.data.data.page) > 1) ? '此分组内没有更多' + typeName + '了~' : '分组内还没有' + typeName + '~'
                        if ($('.fastim-contacts-window .contacts-list .fastim-data-none').length) {
                            $('.fastim-contacts-window .contacts-list .fastim-data-none').html(tis)
                        } else {
                            contactsListEl.append('<div class="fastim-data-none">' + tis + '</div>');
                        }
                    }
                } else if (msg.data.data.method == 'load-collection') {
                    var collectionTbodyEl = $('.fastim-collection-window .collection-data-tbody')
                    FastIm.loadGroup = msg.data.nextpage ? msg.data.data : false
                    if (parseInt(msg.data.data.page) <= 1) {
                        $('.fastim-collection-window').fadeIn()
                        collectionTbodyEl.empty()
                    }

                    $('.fastim-collection-window .collection-type').text(msg.data.group.name)

                    var dataNone = '没有更多收藏内容了~';
                    if (msg.data.res.length) {
                        for (var i = 0; i < msg.data.res.length; i++) {
                            msg.data.res[i].buildType = 'collection';
                            collectionTbodyEl.append('<div data-data="' + msg.data.res[i].id + '" class="fastim-titem">\
                                    <div class="fastim-titem-flex7 ' + (msg.data.res[i].type == 'file' ? 'collection-file' : '') + '">' + FastIm.build.message(msg.data.res[i]) + '</div>\
                                    <div class="fastim-titem-flex3">' + msg.data.res[i].createtime + '</div>\
                                    <div class="fastim-titem-flex2">' + msg.data.res[i].from + '</div>\
                                </div>');
                        }

                        if (!msg.data.nextpage) {
                            if ($('.fastim-collection-window .collection-data-tbody .fastim-data-none').length) {
                                $('.fastim-collection-window .collection-data-tbody .fastim-data-none').html(dataNone)
                            } else {
                                collectionTbodyEl.append('<div class="fastim-data-none">' + dataNone + '</div>');
                            }
                        }
                    } else {
                        if ($('.fastim-collection-window .collection-data-tbody .fastim-data-none').length) {
                            $('.fastim-collection-window .collection-data-tbody .fastim-data-none').html(dataNone)
                        } else {
                            collectionTbodyEl.append('<div class="fastim-data-none">' + dataNone + '</div>');
                        }
                    }
                }
            }],
            ['user_search', () => {
                if (msg.data.data.type == 'pick') {
                    $('.fastim-pickuser .pick-widget .pick-item,.fastim-pickuser .pick-widget .pick-group').hide()
                    $('.fastim-pickuser .pick-widget .pick-group .pick-group-status').removeClass('rotate-90')
                    $('.fastim-pickuser .pick-widget .group-search-result,.fastim-pickuser .pick-widget .search-result-group').remove()
                    if (msg.data.friends.length) {
                        $('.fastim-pickuser .pick-widget').append('<div class="search-result-group">好友</div>')
                        for (var i = 0; i < msg.data.friends.length; i++) {
                            msg.data.friends[i].groupId = 'search-result'
                            $('.fastim-pickuser .pick-widget').append(FastIm.build.widget('pickuser', msg.data.friends[i]))
                        }
                    }
                    if (msg.data.lately_session_user.length) {
                        $('.fastim-pickuser .pick-widget').append('<div class="search-result-group">最近会话</div>')
                        for (var i = 0; i < msg.data.lately_session_user.length; i++) {
                            $.extend(msg.data.lately_session_user[i], msg.data.lately_session_user[i].sessionInfo.pushUser)
                            msg.data.lately_session_user[i].groupId = 'search-lately-result'
                            $('.fastim-pickuser .pick-widget').append(FastIm.build.widget('pickuser', msg.data.lately_session_user[i]))
                        }
                    }
                    if (!msg.data.lately_session_user.length && !msg.data.friends.length) {
                        $('.fastim-pickuser .pick-widget').append('<div class="search-result-group">没有更多数据~</div>')
                    }
                } else if (msg.data.data.type == 'all') {
                    FastIm.showWindow('fastim-search-result')
                    FastIm.searchResult = msg.data
                    $('.fastim-search-result').empty()
                    // 好友,每组只渲染5个
                    if (msg.data.friends.length) {
                        $('.fastim-search-result').append('<div class="search-result-group">好友</div>')
                        for (var i = 0; i < msg.data.friends.length && i < 5; i++) {
                            $('.fastim-search-result').append(FastIm.build.widget('search-all-item', msg.data.friends[i], 'user'))
                        }
                        if (msg.data.friends.length > 5) {
                            $('.fastim-search-result').append(FastIm.build.widget('load-more', 'friends', 'search-result'))
                        }
                    }
                    // 群聊
                    if (msg.data.group_chat.length) {
                        $('.fastim-search-result').append('<div class="search-result-group">群聊</div>')
                        for (var i = 0; i < msg.data.group_chat.length && i < 5; i++) {
                            $('.fastim-search-result').append(FastIm.build.widget('search-all-item', msg.data.group_chat[i], 'group'))
                        }
                        if (msg.data.group_chat.length > 5) {
                            $('.fastim-search-result').append(FastIm.build.widget('load-more', 'group_chat', 'search-result'))
                        }
                    }
                    // 最近会话
                    if (msg.data.lately_session.length) {
                        $('.fastim-search-result').append('<div class="search-result-group">最近会话</div>')
                        for (var i = 0; i < msg.data.lately_session.length && i < 5; i++) {
                            msg.data.lately_session[i].id = msg.data.lately_session[i].sessionInfo.pushUser.id
                            msg.data.lately_session[i].avatar = msg.data.lately_session[i].sessionInfo.pushUser.avatar
                            msg.data.lately_session[i].nickname = msg.data.lately_session[i].sessionInfo.pushUser.nickname
                            msg.data.lately_session[i].last_time = msg.data.lately_session[i].lastMessage.last_time
                            $('.fastim-search-result').append(FastIm.build.widget('search-all-item', msg.data.lately_session[i], (msg.data.lately_session[i].sessionInfo.type == 'single' ? 'user' : msg.data.lately_session[i].sessionInfo.type)))
                        }
                        if (msg.data.lately_session.length > 5) {
                            $('.fastim-search-result').append(FastIm.build.widget('load-more', 'lately_session', 'search-result'))
                        }
                    }
                    if (!msg.data.friends.length && !msg.data.group_chat.length && !msg.data.lately_session.length) {
                        $('.fastim-search-result').append('<div class="fastim-data-none">无搜索结果~</div>');
                    }
                    $('.fastim-search-result .search-res-item:first').addClass('active')
                    $('.fastim-search-result').scrollTop(0)
                    FastIm.searchSessionSelect = 1;
                }
            }],
            ['create_group', () => {
                FastIm.hideWindow('fastim-userinfo')
            }],
            ['receipt_group_chat_notice', () => {
                FastIm.pop.msg(msg.data.msg, 'success');
                if (msg.data.data.type && msg.data.data.type == 'list') {
                    $('#receipted-count-' + msg.data.data.id).text(msg.data.receiptedCount + '人已确认').fadeIn();
                    $('#group-notice-receipt-' + msg.data.data.id).hide();
                } else {
                    FastIm.hideWindow('new-group-notice-popup')
                }
            }],
            ['group_chat_notice', () => {
                if (msg.data.data.method == 'list') {
                    FastIm.loadGroupChatNotice = msg.data.nextpage ? msg.data.data : false;
                    if (msg.data.data.page == 1) {
                        $('.groupinfo-notice-panel .groupinfo-notice-list').html('');
                    }
                    if (msg.data.isLeader) {
                        $('.fastim-userinfo').css({
                            width: '460px',
                            left: 'calc(50% - 230px)'
                        });
                        FastIm.groupNoticeWiden = true
                    }
                    if (msg.data.list.length) {
                        var listHtml = ''
                        for (let i = 0; i < msg.data.list.length; i++) {
                            listHtml += FastIm.build.widget('group-chat-notice-item', msg.data.list[i], msg.data.isLeader)
                        }
                        $('.groupinfo-notice-panel .groupinfo-notice-list').append(listHtml)
                        if (!msg.data.nextpage) {
                            $('.groupinfo-notice-panel .fastim-data-none').text('没有更多公告了...')
                        } else {
                            $('.groupinfo-notice-panel .fastim-data-none').text('')
                        }
                    } else {
                        $('.groupinfo-notice-panel .fastim-data-none').text('没有更多公告了...')
                    }
                }
            }],
            ['group_chat_notice_info', () => {
                if (msg.data.data.type == 'get') {
                    FastIm.common.showNewGroupChatNotice(msg.data.info);
                } else if (msg.data.data.type == 'edit') {
                    $('#notice_group_id').val(msg.data.info.id)
                    $('#group_notice_group_id').val(msg.data.info.group_id)
                    $('.im-add-group-notice-popup textarea[name="content"]').val(msg.data.info.content)
                    FastIm.tempImages = msg.data.info.images
                    var imagesEl = $('.im-add-group-notice-popup .im-group-notice-imgs')
                    for (let i = 0; i < msg.data.info.images.length; i++) {
                        imagesEl.append(FastIm.build.widget('upload-box-image', {
                            'fullurl': msg.data.info.images[i]
                        }));
                    }
                    $("#notice-popup").prop("checked", parseInt(msg.data.info.popup) == 1 ? true:false);
                    $("#notice-receipt").prop("checked", parseInt(msg.data.info.receipt) == 1 ? true:false);
                    $("#notice-top").prop("checked", parseInt(msg.data.info.top) == 1 ? true:false);
                    FastIm.showWindow('im-add-group-notice-popup')
                }
            }],
            ['group-notice-users', () => {
                FastIm.loadGroupNoticeUsers = msg.data.nextpage ? msg.data.data : false;
                var el = $('.im-user-list-items')
                if (msg.data.data.page == 1) {
                    $('#im-user-list-popup-title').text(msg.data.data.method == 'read-number' ? '已阅读群员':'已确认群员')
                    $('#im-user-list-popup-createtime').text(msg.data.data.method == 'read-number' ? '阅读时间':'确认时间')
                    el.html('')
                }

                for (let i = 0; i < msg.data.users.length; i++) {
                    let itemHtml = '<div class="groupchat-member-item">'
                    itemHtml += '<div class="groupchat-member-basic">'
                    itemHtml += '<img src="' + msg.data.users[i].avatar + '" />'
                    itemHtml += '<div class="groupchat-member-nickname_origin" title="' + msg.data.users[i].nickname + '">' + msg.data.users[i].nickname + '</div>'
                    itemHtml += '</div>'
                    itemHtml += '<div class="groupchat-member-speaktime">' + msg.data.users[i].createtime + '</div>'
                    itemHtml += '</div>'
                    el.append(itemHtml)
                }
            }],
            ['group_chat_at', () => {
                var atSpan = $('.' + msg.data.data.at_class_name), atUsersEl = $('.fastim-at-users'), atUsersElBox = $('.fastim-at-users-box')
                atUsersElBox.empty();
                if (atSpan && msg.data.group_member.length) {
                    var atUserItem = FastIm.template.atUserItem()
                    for (let i = 0; i < msg.data.group_member.length; i++) {
                        if (msg.data.group_member[i].remark) {
                            msg.data.group_member[i].nickname += '(' + msg.data.group_member[i].remark + ')'
                        }
                        atUsersElBox.append(atUserItem(msg.data.group_member[i]))
                    }
                    atUsersElBox.find('.fastim-at-user-item:first-child').addClass('active')
                    atUsersElBox.scrollTop(0)
                    FastIm.atUserSelect = 1;
                    $('#at_users_search').trigger('focus')
                } else {
                    atUsersElBox.append('<div class="fastim-data-none">没有更多了...</div>')
                }
            }],
            ['new_message', () => {
                var sessionType = msg.data.sessionInfo.type

                if (sessionType == 'group') {
                    // 群聊-不理会发送给特定用户的系统消息
                    if (msg.data.messageData.type == 'system' && (msg.data.messageData.message.display_user != FastIm.userinfo.id && msg.data.messageData.message.display_user != 'all')) {
                        return;
                    }

                    // 在会话列表渲染群聊资料,在会话面板渲染发信用户资料
                    let pushUser = msg.data.sessionInfo.pushUser,
                        sessionUser = msg.data.sessionInfo.sessionUser
                    msg.data.sessionInfo.pushUser = sessionUser
                    msg.data.sessionInfo.sessionUser = pushUser
                    FastIm.imSession(msg.data); // 会话列表会话保障
                    msg.data.sessionInfo.pushUser = pushUser
                    msg.data.sessionInfo.sessionUser = sessionUser
                } else {
                    FastIm.imSession(msg.data); // 会话列表会话保障
                }

                if (!msg.data.shield) {
                    let noticeAvatar = FastIm.common.imgUrl(msg.data.sessionInfo.pushUser.avatar)
                    FastIm.common.newMessageNotice(msg.data.sessionInfo.pushUser.nickname, msg.data.lastMessage.last_message, noticeAvatar, sessionType, true);
                }

                if (FastIm.sessionId == msg.data.sessionInfo.id) {
                    if (sessionType == 'single' || sessionType == 'group') {
                        var recordsEL = $('.chat-records');
                        if (sessionType == 'single') {
                            // 单聊
                            msg.data.messageData.record_content_before = 'record_content_before';
                            msg.data.sessionInfo.pushUser.hidden_nickname = 'fastim-hidden';
                        }

                        if (msg.data.messageData.type == 'system') {
                            if (msg.data.messageData.message.display_user == 'all' || msg.data.messageData.message.display_user == FastIm.userinfo.id) {
                                var newHTML = '<li class="chat-record-item system"><span>' + msg.data.messageData.message.message + '</span></li>';
                            }
                        } else {
                            var messageTemplate = FastIm.template.message()
                            msg.data.sessionInfo.pushUser.avatar = FastIm.common.imgUrl(msg.data.sessionInfo.pushUser.avatar)
                            if (msg.data.messageData.type == 'group_chat_notice') {
                                FastIm.common.showNewGroupChatNotice(msg.data.messageData.message)
                                var groupChatNotice = {
                                    c: 'Message',
                                    a: 'readMessage',
                                    data: {
                                        'session_id': msg.data.sessionInfo.id,
                                        'type': 'group_chat_notice',
                                        'id': msg.data.messageData.message.id,
                                        'action': ''
                                    }
                                };
                            }
                            msg.data.messageData.sender = (msg.data.sessionInfo.pushUser.id == FastIm.userinfo.id) ? 'me' : 'you'
                            msg.data.messageData.message = FastIm.build.message(msg.data.messageData)
                            msg.data.messageData.status = FastIm.common.messageStatus(3, msg.data.messageData.id)
                            var newHTML = messageTemplate(msg.data)
                        }
                        recordsEL.append(newHTML)

                        FastIm.showMessageWindowBack = function () {
                            let message = {
                                c: 'Message',
                                a: 'readMessage',
                                data: {
                                    'session_id': msg.data.sessionInfo.id
                                }
                            }
                            FastIm.socket.send(message);
                            recordsEL.scrollTop(recordsEL[0].scrollHeight)
                            FastIm.common.unreadMessagesDisplay(msg.data.sessionInfo.id, 0)

                            if (msg.data.messageData.type == 'group_chat_notice') {
                                FastIm.socket.send(groupChatNotice);
                            }
                        }
                    } else if (sessionType == 'service' && msg.data.sessionInfo.chat_id == 1) {
                        // 好友申请
                        var serviceMessageTemplate = FastIm.template.serviceMessage();
                        var newHTML = serviceMessageTemplate(FastIm.build.serviceMessageInfo(msg.data, 'new_friends_message', {
                            userInfo: msg.data.sessionInfo.userInfo
                        }));
                        var recordsEL = $('.fastim-window-right .service-message')
                        recordsEL.prepend(newHTML)

                        FastIm.showMessageWindowBack = function () {
                            let message = {
                                c: 'Message',
                                a: 'readMessage',
                                data: {
                                    'session_id': FastIm.sessionId
                                }
                            }
                            FastIm.socket.send(message);
                            recordsEL.scrollTop(0)
                            FastIm.common.unreadMessagesDisplay(FastIm.sessionId, 0)
                        }
                    } else if (sessionType == 'service' && msg.data.sessionInfo.chat_id == 2) {
                        // 群通知
                        var serviceMessageTemplate = FastIm.template.serviceMessage();
                        var newHTML = serviceMessageTemplate(FastIm.build.serviceMessageInfo(msg.data, 'group_notice', {
                            userInfo: msg.data.sessionInfo.userInfo,
                            groupChatInfo: msg.data.sessionInfo.groupChatInfo
                        }));
                        var recordsEL = $('.fastim-window-right .service-message')
                        recordsEL.prepend(newHTML)

                        if (msg.data.sessionInfo.reloadSessionList) {
                            FastIm.onMessage({event: 'reload_session_list'}); // 重载会话列表
                        }

                        FastIm.showMessageWindowBack = function () {
                            let message = {
                                c: 'Message',
                                a: 'readMessage',
                                data: {
                                    'session_id': FastIm.sessionId
                                }
                            }
                            FastIm.socket.send(message);
                            recordsEL.scrollTop(0)
                            FastIm.common.unreadMessagesDisplay(FastIm.sessionId, 0)
                        }
                    } else if (sessionType == 'service' && msg.data.sessionInfo.chat_id == 3) {
                        // 智能客服消息
                        var recordsEL = $('.chat-records');

                        if (msg.data.messageData.type == 'system') {
                            if (msg.data.messageData.message.display_user == 'all' || msg.data.messageData.message.display_user == FastIm.userinfo.id) {
                                var newHTML = '<li class="chat-record-item system"><span>' + msg.data.messageData.message.message + '</span></li>';
                            }
                        } else {
                            var messageTemplate = FastIm.template.message()
                            msg.data.sessionInfo.pushUser.type = 'service'
                            msg.data.sessionInfo.pushUser.avatar = FastIm.common.imgUrl(msg.data.sessionInfo.pushUser.avatar)
                            msg.data.messageData.message = FastIm.build.message(msg.data.messageData)
                            msg.data.messageData.status = FastIm.common.messageStatus(3, msg.data.messageData.id)
                            msg.data.messageData.sender = (msg.data.sessionInfo.recipientUser.id == FastIm.userinfo.id) ? 'you' : 'me'
                            var newHTML = messageTemplate(msg.data)
                        }
                        recordsEL.append(newHTML)

                        FastIm.showMessageWindowBack = function () {
                            let message = {
                                c: 'Message',
                                a: 'readMessage',
                                data: {
                                    'session_id': msg.data.sessionInfo.id
                                }
                            }
                            FastIm.socket.send(message);
                            recordsEL.scrollTop(recordsEL[0].scrollHeight)
                            FastIm.common.unreadMessagesDisplay(msg.data.sessionInfo.id, 0)
                        }
                    } else {
                        // 其他
                        let message = {
                            c: 'User',
                            a: 'loadSession',
                            data: {
                                'session_id': FastIm.sessionId,
                                'page': 1
                            }
                        }
                        FastIm.socket.send(message);

                        FastIm.showMessageWindowBack = function () {
                            FastIm.common.unreadMessagesDisplay(FastIm.sessionId, 0)
                        }
                    }

                    // 清理未读标记
                    if (FastIm.windowState.fastimWindow && FastIm.windowState.messageWindow) {
                        typeof FastIm.showMessageWindowBack == "function" && FastIm.showMessageWindowBack()
                        FastIm.showMessageWindowBack = null
                    }
                }

                if (!FastIm.windowState.messageWindow) {
                    $('.tab-new-message').fadeIn(200)
                }
            }],
            ['info-detail', () => {
                msg.data.info.avatar = FastIm.common.imgUrl(msg.data.info.avatar)
                msg.data.info.uploadicon = FastIm.common.imgUrl('/assets/addons/fastim/icon/upload-img.png')
                FastIm.showWindow('fastim-userinfo')

                if (msg.data.data.method == 'user-edit') {
                    // 编辑用户资料
                    $('.fastim-userinfo .fastim-popup-title').text('编辑资料')
                    var info = '<div class="fastim-userinfo-detail fastim-userinfo-form">\
                                    <form name="userinfo" novalidate>';
                    info += FastIm.build.widget('im-input', {
                        title: '昵称',
                        name: 'nickname',
                        value: msg.data.info.nickname
                    });
                    info += FastIm.build.widget('im-textarea', {title: '签名', name: 'bio', value: msg.data.info.bio});
                    info += FastIm.build.widget('im-select-obj', {
                        title: '性别',
                        name: 'gender',
                        data: [
                            {id: 'secrecy', name: '保密'},
                            {id: 'male', name: '男'},
                            {id: 'female', name: '女'}
                        ],
                        value: msg.data.info.gender.value,
                        imchange: false
                    });

                    info += FastIm.build.widget('date-three-linkage', {title: '生日', class: 'fastim-userinfo-ymd'});
                    info += FastIm.build.widget('im-input', {title: '邮箱', name: 'email', value: msg.data.info.email});
                    info += FastIm.build.widget('im-input', {title: '手机', name: 'mobile', value: msg.data.info.mobile});
                    info += FastIm.build.widget('im-input', {
                        title: '公司',
                        name: 'company',
                        value: msg.data.info.company
                    });
                    info += FastIm.build.widget('im-select', {
                        title: '职业',
                        name: 'occupation',
                        data: msg.data.info.occupationData,
                        value: msg.data.info.occupation
                    });
                    info += '<div class="fastim-center-buttons">';
                    info += '<button type="button" data-action="close-window" data-data="fastim-userinfo" class="im-button im-max-button fastim-click im-button-grey">取消</button>';
                    info += '<button type="button" data-action="save-userinfo" class="im-button im-max-button fastim-click">保存</button>';
                    info += '</div>';
                    info += '</form>';
                    info += '</div>';
                    $('.fastim-userinfo').append(info)

                    // 初始化 select
                    FastIm.common.YMDselect('fastim-userinfo-ymd')
                    if (msg.data.info.birthday) {
                        $('.fastim-userinfo-detail select[name="year"]').val(msg.data.info.birthday[0])
                        $('.fastim-userinfo-detail select[name="month"]').val(msg.data.info.birthday[1])
                        $('.fastim-userinfo-detail select[name="day"]').val(msg.data.info.birthday[2])
                    }
                } else if (msg.data.data.method == 'group-edit') {
                    // 编辑群组资料
                    $('.fastim-userinfo').append(FastIm.build.groupInfo(msg.data.info, 'edit'))
                    $('.fastim-userinfo .fastim-popup-title').text('编辑' + msg.data.info.nickname)
                } else {
                    // 查看资料
                    var appendCallBack = null

                    if (msg.data.data.method == 'get-new-info') {
                        /*
                         * 修改用户昵称需要重载会话列表
                         * 单次禁用hidePopup的功能
                         */
                        FastIm.doNotHidePopup = true
                        FastIm.onMessage({event: 'reload_session_list'}); // 重载会话列表
                    }

                    $('.fastim-userinfo .fastim-popup-title').text('')

                    if (msg.data.data.type == 'user') {
                        // 用户资料
                        msg.data.info.other = msg.data.info.status.chinese
                        if (msg.data.info.remark) {
                            msg.data.info.nickname = msg.data.info.nickname_origin
                        }

                        var fastimUserinfoBox = FastIm.build.widget('fastim-userinfo-box', msg.data.info);

                        var fastimUserinfoDetail = '<div class="fastim-userinfo-detail">';
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {title: '账号', value: msg.data.info.id});
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {
                            title: '个人',
                            value: (parseInt(msg.data.info.age) ? msg.data.info.age + '岁 ' : '') + msg.data.info.gender.chinese
                        });
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {
                            title: '生日',
                            value: msg.data.info.birthday
                        });
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {
                            title: '邮箱',
                            value: msg.data.info.email
                        });
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {
                            title: '手机',
                            value: msg.data.info.mobile
                        });
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {
                            title: '公司',
                            value: msg.data.info.company
                        });
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {
                            title: '职业',
                            value: msg.data.info.occupation
                        });
                        if (msg.data.info.friend && msg.data.info.id != FastIm.userinfo.id) {
                            // 修改分组功能
                            fastimUserinfoDetail += FastIm.build.widget('im-select-obj', {
                                title: '分组',
                                name: 'fastim-friend-group',
                                data: msg.data.info.groups,
                                value: msg.data.info.group,
                                id: msg.data.info.id,
                                imchange: true
                            });
                        }
                        fastimUserinfoDetail += '<div class="fastim-center-buttons">';
                        if (msg.data.info.id == FastIm.userinfo.id) {
                            // 编辑资料
                            fastimUserinfoDetail += '<button data-action="userinfo-opt" data-opt="edit" data-type="user" data-data="' + msg.data.info.id + '" class="im-button im-max-button fastim-click im-button-grey">编辑资料</button>';
                            appendCallBack = function () {
                                $('.fastim-userinfo-box #uploadavatar,.fastim-userinfo-box .upload-avatar').show()
                                $('.fastim-userinfo-box #uploadavatar').data('type', 'user').attr('data-type', 'user')
                                $('.fastim-userinfo-box #uploadavatar').data('id', msg.data.info.id).attr('data-id', msg.data.info.id)
                            }
                        } else if (msg.data.info.friend) {
                            // 发送消息
                            fastimUserinfoDetail += '<button data-action="open-session" data-type="user" data-data="' + msg.data.info.id + '" class="im-button im-max-button fastim-click">发送消息</button>';
                            fastimUserinfoDetail += '<button data-action="del-contact" data-type="user" data-data="' + msg.data.info.id + '" class="im-button im-max-button fastim-bg-red fastim-click">删除好友</button>';
                            appendCallBack = function () {
                                $('.fastim-userinfo-box #uploadavatar,.fastim-userinfo-box .upload-avatar').hide()
                            }
                        } else {
                            // 加为好友
                            fastimUserinfoDetail += '<button data-action="add-friends" data-data="' + msg.data.info.id + '" class="im-button im-max-button fastim-click">加为好友</button>';
                            appendCallBack = function () {
                                $('.fastim-userinfo-box #uploadavatar,.fastim-userinfo-box .upload-avatar').hide()
                            }
                        }
                        fastimUserinfoDetail += '</div>';
                        fastimUserinfoDetail += '</div>';
                        $('.fastim-userinfo').append(fastimUserinfoBox)
                        $('.fastim-userinfo').append(fastimUserinfoDetail)
                    } else if (msg.data.data.type == 'group') {
                        // 群资料
                        msg.data.info.other = '群账号:' + msg.data.info.id
                        msg.data.info.type = 'group'
                        var fastimUserinfoBox = FastIm.build.widget('fastim-userinfo-box', msg.data.info)
                        var fastimUserinfoDetail = '<div class="fastim-userinfo-detail groupinfo-menu-tab-panel groupinfo-basic-panel">'
                        if (msg.data.info.leader) {
                            fastimUserinfoDetail += FastIm.build.widget('im-text', {
                                title: '群主',
                                value: '<img data-action="get-info" data-data="user" data-session-user="' + msg.data.info.leader.id + '" class="leader-avatar fastim-click" src="' + FastIm.common.imgUrl(msg.data.info.leader.avatar) + '" />'
                            });
                        }
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {
                            title: '群人数',
                            value: msg.data.info.user_count + '/' + msg.data.info.max_user_count
                        });
                        if (typeof msg.data.info.add_mode != 'undefined') {
                            fastimUserinfoDetail += FastIm.build.widget('im-text', {
                                title: '加群模式',
                                value: (parseInt(msg.data.info.add_mode) == 0) ? '需管理员审核' : '无需审核'
                            });
                        }
                        if (typeof msg.data.info.invite_join_group != 'undefined') {
                            fastimUserinfoDetail += FastIm.build.widget('im-text', {
                                title: '邀请模式',
                                value: (parseInt(msg.data.info.invite_join_group) == 0) ? '成员邀请好友需要审核' : '成员邀请好友无需审核'
                            });
                        }
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {
                            title: '成员发言',
                            value: (parseInt(msg.data.info.speak) == 0) ? '允许发言' : '禁止发言'
                        });
                        if (typeof msg.data.info.history_message != 'undefined') {
                            fastimUserinfoDetail += FastIm.build.widget('im-text', {
                                title: '历史消息',
                                value: (parseInt(msg.data.info.history_message) == 0) ? '不允许新入群用户查看' : '允许新入群用户查看'
                            });
                        }
                        if (typeof msg.data.info.retrieval_settings != 'undefined') {
                            fastimUserinfoDetail += FastIm.build.widget('im-text', {
                                title: '检索设置',
                                value: (parseInt(msg.data.info.retrieval_settings) == 0) ? '禁用搜素加群' : '允许被搜素到'
                            });
                        }
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {
                            title: '创建时间',
                            value: msg.data.info.createtime
                        });

                        if (msg.data.info.groupMember) {
                            fastimUserinfoDetail += FastIm.build.widget('im-select-obj', {
                                title: '分组',
                                name: 'fastim-groupchat-group',
                                data: msg.data.info.groups,
                                value: msg.data.info.group,
                                id: msg.data.info.id,
                                imchange: true
                            });
                        }

                        fastimUserinfoDetail += '<div class="fastim-center-buttons">';
                        if (msg.data.info.deletetime) {
                            fastimUserinfoDetail += '<div class="fastim-data-none">群聊已解散</div>'
                            appendCallBack = function () {
                                $('.fastim-userinfo-box #uploadavatar,.fastim-userinfo-box .upload-avatar').hide()
                            }
                        } else if (msg.data.info.isLeader) {
                            // 编辑资料
                            fastimUserinfoDetail += '<button data-action="userinfo-opt" data-opt="edit" data-type="group" data-data="' + msg.data.info.id + '" class="im-button im-max-button fastim-click im-button-grey">编辑资料</button>';
                            fastimUserinfoDetail += '<button data-action="del-contact" data-type="dissolution-group" data-data="' + msg.data.info.id + '" class="im-button im-max-button fastim-bg-red fastim-click">解散群聊</button>';
                            appendCallBack = function () {
                                $('.fastim-userinfo-box #uploadavatar,.fastim-userinfo-box .upload-avatar').show()
                                $('.fastim-userinfo-box #uploadavatar').data('type', 'group').attr('data-type', 'group')
                                $('.fastim-userinfo-box #uploadavatar').data('id', msg.data.info.id).attr('data-id', msg.data.info.id)
                            }
                        } else if (!msg.data.info.groupMember) {
                            // 加入群聊
                            fastimUserinfoDetail += '<button data-action="userinfo-opt" data-opt="join" data-type="group" data-data="' + msg.data.info.id + '" class="im-button im-max-button fastim-click im-button-grey">加入群聊</button>';
                            appendCallBack = function () {
                                $('.fastim-userinfo-box #uploadavatar,.fastim-userinfo-box .upload-avatar').hide()
                            }
                        } else {
                            // 发送消息
                            fastimUserinfoDetail += '<button data-action="open-session" data-type="group" data-data="' + msg.data.info.id + '" class="im-button im-max-button fastim-click">发送消息</button>';
                            fastimUserinfoDetail += '<button data-action="del-contact" data-type="group" data-data="' + msg.data.info.id + '" class="im-button im-max-button fastim-bg-red fastim-click">退出群聊</button>';
                            appendCallBack = function () {
                                $('.fastim-userinfo-box #uploadavatar,.fastim-userinfo-box .upload-avatar').hide()
                            }
                        }
                        fastimUserinfoDetail += '</div>';
                        fastimUserinfoDetail += '</div>';
                        fastimUserinfoDetail += '<div class="fastim-userinfo-detail groupinfo-menu-tab-panel groupinfo-member-panel fastim-hidden"></div>';
                        fastimUserinfoDetail += '<div class="fastim-userinfo-detail groupinfo-menu-tab-panel groupinfo-notice-panel fastim-hidden">';
                        fastimUserinfoDetail += msg.data.info.isLeader ? '<div data-action="im-add-group-notice-popup" data-data="' + msg.data.info.id + '" class="fastim-click im-button add-group-chat-notice">发布公告</div>':'';
                        fastimUserinfoDetail += '<div class="groupinfo-notice-list fastim-scrollbar"></div>';
                        fastimUserinfoDetail += '<div class="fastim-data-none">正在加载...</div></div>';
                        $('.fastim-userinfo').append(fastimUserinfoBox)
                        $('.fastim-userinfo').append(fastimUserinfoDetail)
                    } else if (msg.data.data.type == 'service') {
                        // 服务号资料
                        msg.data.info.other = '服务账号'
                        msg.data.info.type = 'service'
                        var fastimUserinfoBox = FastIm.build.widget('fastim-userinfo-box', msg.data.info)

                        var fastimUserinfoDetail = '<div class="fastim-userinfo-detail">';
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {title: '个人', value: '-'});
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {title: '邮箱', value: '-'});
                        fastimUserinfoDetail += FastIm.build.widget('im-text', {title: '手机', value: '-'});
                        fastimUserinfoDetail += '<div class="fastim-center-buttons">';
                        fastimUserinfoDetail += '<button data-action="close-window" data-data="fastim-userinfo" class="im-button im-max-button fastim-click">关闭</button>';
                        appendCallBack = function () {
                            $('.fastim-userinfo-box #uploadavatar,.fastim-userinfo-box .upload-avatar').hide()
                        }
                        fastimUserinfoDetail += '</div>';
                        fastimUserinfoDetail += '</div>';
                        $('.fastim-userinfo').append(fastimUserinfoBox)
                        $('.fastim-userinfo').append(fastimUserinfoDetail)
                    }

                    typeof appendCallBack == 'function' && appendCallBack()
                }
            }],
            ['report', () => {
                FastIm.showWindow('fastim-report')
                $('.fastim-report .fastim-popup-title').text('举报' + msg.data.sessionUser.nickname)
                $('.fastim-report #fastim-report-form-type').val((msg.data.type == 'single') ? 'user' : msg.data.type)
                $('.fastim-report label.describe').text('举报详情')
                $('.fastim-report input[name="describe"]').attr('placeholder', '请详细描述被举报人的恶意行为')
            }],
            ['open_session', () => {
                FastIm.imSession(msg.data)
                FastIm.showWindow('message')
                FastIm.showWindow('message-window')
                $('.fastim-sidebar-tab img[data-data="message"]').trigger('click')
                $('.fastim-session-li[data-data="' + msg.data.sessionInfo.id + '"]').trigger('click')
                $('.fastim-session-ul').scrollTop(0)
            }],
            ['create_new_contact', () => {
                FastIm.hideWindow('new-contact-apply')

                var showEl = {group: true, remark: true, note: true, tis: false},
                    buttons = [],
                    newHTML = '';
                msg.data.info.avatar = FastIm.common.imgUrl(msg.data.info.avatar);
                msg.data.info.type = msg.data.data.type;
                msg.data.info.action = msg.data.data.action;

                // 添加好友
                if (msg.data.data.type == 'friends') {
                    msg.data.info.icon = FastIm.common.imgUrl('/assets/addons/fastim/icon/' + msg.data.info.gender.value + '.png');
                    msg.data.info.other = msg.data.info.status.chinese

                    if (parseInt(msg.data.info.temp_session) == 1) {
                        // 添加直接聊天的按钮
                        buttons.push({
                            attr: 'data-type="user" data-data="' + msg.data.info.id + '"',
                            action: 'open-session',
                            class: 'fastim-bg-red',
                            title: '仅聊天'
                        })
                    }

                    // 预同意
                    if (msg.data.data.action && msg.data.data.action == 'pre-agree') {
                        msg.data.info.action = 'agree-friends'
                        msg.data.info.id = msg.data.data.message_id
                        showEl.note = false
                        buttons.push({
                            attr: '',
                            class: '',
                            action: 'submit-new-contact-form',
                            title: '同意申请'
                        })
                    } else if (parseInt(msg.data.info.verify_method) == 1) {
                        msg.data.info.action = 'apply'
                        showEl.note = false
                        buttons.push({
                            attr: '',
                            class: '',
                            action: 'submit-new-contact-form',
                            title: '加为好友'
                        })
                    } else if (parseInt(msg.data.info.verify_method) == 0) {
                        msg.data.info.action = 'apply'
                        buttons.push({
                            attr: '',
                            class: '',
                            action: 'submit-new-contact-form',
                            title: '发送申请'
                        })
                    }
                } else if (msg.data.data.type == 'group') {
                    // 加入群聊
                    msg.data.info.icon = FastIm.common.imgUrl('/assets/addons/fastim/icon/group.png')
                    msg.data.info.other = msg.data.info.user_count + '/' + msg.data.info.max_user_count
                    showEl.remark = false

                    if (parseInt(msg.data.info.add_mode) == 0) {
                        // 需要验证
                        msg.data.info.action = 'apply'
                        buttons.push({
                            attr: '',
                            class: '',
                            action: 'submit-new-contact-form',
                            title: '申请加群'
                        })
                    } else if (parseInt(msg.data.info.add_mode) == 1) {
                        // 无需验证
                        msg.data.info.action = 'apply'
                        showEl.note = false
                        showEl.tis = '加入该群无需管理员验证'
                        buttons.push({
                            attr: '',
                            class: '',
                            action: 'submit-new-contact-form',
                            title: '加入群聊'
                        })
                    }
                }

                buttons.push({
                    attr: 'data-data="new-contact-apply"',
                    class: 'im-button-grey',
                    action: 'close-window',
                    title: '取消'
                });
                newHTML = FastIm.build.widget('create-new-contact', msg.data.info, {
                    showEl: showEl,
                    buttons: buttons
                });
                $('.fastim-modal .fastim-window').prepend(newHTML);

                // 渲染用户的分组
                var userGroup = $('.fastim-popup.new-contact-apply-box select[name="new_contact_user_group"]')
                userGroup.empty()
                for (var i = 0; i < msg.data.group.length; i++) {
                    userGroup.append('<option value="' + msg.data.group[i].id + '">' + msg.data.group[i].name + '</option>');
                }
                FastIm.common.editEscape('new-contact-apply', 'add')
            }],
            ['new_friends_option', () => {
                FastIm.pop.msg(msg.data.msg, msg.data.type);
                let message = {
                    c: 'User',
                    a: 'loadSession',
                    data: {
                        'session_id': msg.data.session_id,
                        'page': 1
                    }
                }
                FastIm.socket.send(message);
            }],
            ['del-contact', () => {
                // 删除联系人
                FastIm.hideWindow('fastim-userinfo')
                FastIm.pop.msg(msg.data.msg);
                FastIm.onMessage({event: 'reload_session_list'}); // 重载会话列表

                if (FastIm.getGroupsContacts) {
                    $('.contacts-list .contact-' + msg.data.data.id + msg.data.data.type).remove()
                }
            }],
            ['del-collection', () => {
                // 删除收藏
                FastIm.pop.msg(msg.data.msg);
                $('.fastim-tbody.collection-data-tbody .fastim-titem[data-data="' + msg.data.data.id + '"]').remove()
            }],
            ['group_chat_member', () => {
                FastIm.groupChatInfo = msg.data.info
                if (msg.data.group_member.length) {
                    var memberPanelEl = $('.fastim-userinfo .groupinfo-member-panel')
                    memberPanelEl.empty().append('\
                            <div class="groupchat-members">\
                                <div class="groupchat-member-item groupchat-members-title">\
                                    <div class="groupchat-member-basic">成员</div>\
                                    <div class="groupchat-member-nickname">群内昵称</div>\
                                    <div class="groupchat-member-speaktime">入群时间</div>\
                                </div>\
                            </div>\
                            <div class="groupchat-items fastim-scrollbar"></div>');

                    var membersEl = $('.fastim-userinfo .groupchat-items')
                    for (var i = 0; i < msg.data.group_member.length; i++) {

                        if (msg.data.group_member[i].id == FastIm.userinfo.id) {
                            var editGroupchatNickname = '<span class="fastim-click fastim-color-blue" data-action="edit-groupmember-nickname" data-value="' + msg.data.group_member[i].nickname + '" data-data="' + msg.data.data.id + '">' + (msg.data.group_member[i].nickname != '' ? msg.data.group_member[i].nickname : '点击编辑') + '</span>';
                        } else {
                            var editGroupchatNickname = msg.data.group_member[i].nickname
                        }

                        membersEl.append('\
                                <div class="groupchat-member-item" data-id="' + msg.data.group_member[i].id + '">\
                                    <div class="groupchat-member-basic">\
                                        <img src="' + msg.data.group_member[i].avatar + '" />\
                                        <div class="groupchat-member-nickname_origin" title="' + msg.data.group_member[i].nickname_origin + '">' + msg.data.group_member[i].nickname_origin + '</div>\
                                    </div>\
                                    <div class="groupchat-member-nickname">' + editGroupchatNickname + '</div>\
                                    <div class="groupchat-member-speaktime">' + msg.data.group_member[i].speaktime + '</div>\
                                </div>');
                    }

                    let newEl = '\
                        <div class="fastim-center-buttons">';
                    newEl += '<button type="button" data-action="invitation-groupmember" data-groupname="' + msg.data.info.nickname + '" data-data="' + msg.data.data.id + '" class="im-button im-max-button fastim-click">邀请好友</button>';
                    newEl += '</div>';

                    memberPanelEl.append(newEl);
                }
            }],
            ['read_message', () => {
                // 消息被读
                if (msg.data.sessionType == 'group') {
                    FastIm.showMessageWindowBack = function () {
                        FastIm.common.messageStatus(msg.data.status, msg.data.message_id, true, msg.data.read_number);
                    }
                } else if (FastIm.sessionId == msg.data.session_id) {
                    FastIm.showMessageWindowBack = function () {
                        let newEl = FastIm.common.messageStatus(1, 'all');
                        $('.chat-record-item.me .fastim-message-status').not('.send-message-fail').replaceWith(newEl);
                    }
                }

                if (FastIm.windowState.fastimWindow && FastIm.windowState.messageWindow) {
                    typeof FastIm.showMessageWindowBack == "function" && FastIm.showMessageWindowBack()
                    FastIm.showMessageWindowBack = null
                }
            }],
            ['group_chat_shield', () => {
                // 添加菜单项
                $('.fastim-ins .fastim-contextmenu').append('<div class="fastim-contextmenu-item" data-action="session-block-groupmessage" data-id="' + msg.data.session_id + '">' + (msg.data.shield == 'shield' ? '解除消息免打扰' : '消息免打扰') + '</div>');
            }],
            ['session_info', () => {
                let message = {
                    c: 'User',
                    a: 'infoDetail',
                    data: {
                        id: msg.data.id,
                        type: msg.data.type,
                        method: 'get'
                    }
                }
                FastIm.socket.send(message);
            }],
            ['message_operation', () => {
                if (msg.data.data.action == 'message-copy') {
                    // 复制
                    var flag = FastIm.common.copyText(msg.data.messageInfo.messageText);
                    if (flag) {
                        FastIm.pop.msg('复制成功~', 'success')
                    } else {
                        FastIm.pop.msg('复制失败，请点击引用按钮后在消息框内复制~', 'info')
                    }
                } else if (msg.data.data.action == 'message-quote') {
                    // 引用
                    var messageEl = $('#fastim-message')
                    messageEl.empty()
                    var message = messageEl.html();
                    messageEl.append(msg.data.messageInfo.messageText)
                    if (FastIm.common.browserType() == "IE" || FastIm.common.browserType() == "Edge") {
                        messageEl.append(message + "<div></div>");
                    } else if (FastIm.common.browserType() == "FF") {
                        // FF浏览器插入换行时,会同时插入两个
                        messageEl.append(message + "<br />");
                    } else {
                        messageEl.append(message + "<div></div>");
                    }
                    messageEl.append('—————————')
                    FastIm.common.messageInputLineFeed()
                } else if (msg.data.data.action == 'message-delete') {
                    // 删除消息-修改最后消息
                    $('.chat-record-item[data-id="' + msg.data.data.id + '"]').remove()
                    var sessionElement = $('.fastim-session-li[data-data="' + msg.data.messageInfo.session_id + '"]')
                    if (sessionElement.length) {
                        sessionElement.find(".session-info-time").eq(0).html(msg.data.messageInfo.last_time)
                        sessionElement.find(".last-message-text .last-message").eq(0).html(msg.data.messageInfo.last_message)
                    }
                } else if (msg.data.data.action == 'message-new-window') {
                    // 新窗口打开
                    window.open(msg.data.messageInfo.messageUrl);
                }
            }],
            ['forward_message', () => {
                // 转发消息
                var sessionIdOld = FastIm.sessionId, reloadRecord = false

                for (var i = 0; i < msg.data.session_ids.length; i++) {
                    reloadRecord = (reloadRecord || msg.data.session_ids[i] == sessionIdOld) ? true : false
                    FastIm.sessionId = msg.data.session_ids[i]
                    for (var y = 0; y < msg.data.message.length; y++) {
                        FastIm.sendMessage(msg.data.message[y].message, msg.data.message[y].type)
                    }
                }
                FastIm.sessionId = sessionIdOld

                FastIm.pop.msg((msg.data.data.type == 'collection' ? '收藏' : '消息') + '已转发~', 'success');

                if (reloadRecord) {
                    FastIm.socket.send({
                        c: 'User',
                        a: 'loadSession',
                        data: {
                            'session_id': FastIm.sessionId,
                            'page': 1
                        }
                    });
                    FastIm.common.unreadMessagesDisplay(FastIm.sessionId, 0)
                }

                if (msg.data.data.type == 'message') {
                    FastIm.hideWindow('fastim-pickuser')
                }
            }],
            ['del-group-member', () => {
                FastIm.pop.msg('群成员已删除~', 'success');
                let message = {
                    c: 'Message',
                    a: 'groupChatMember',
                    data: {
                        'id': FastIm.groupChatInfo.id,
                        'method': 'get'
                    }
                }
                FastIm.socket.send(message);
            }],
            ['upload_multipart', () => {
                if (msg.data.multipart) {
                    FastIm.config.upload.multipart = msg.data.multipart
                }
            }],
            ['default', () => {
                console.log('收到新的ws消息')
            }]
        ]);

        let action = msgFun.get(msg.event) || msgFun.get('default')
        return action.call(this);
    },
    showWindow: function (name) {
        let showFun = new Map([
            ['fastim', () => {

                if (!FastIm.common.isPc()) {
                    // 跳转到 H5
                    window.location.href = FastIm.config.h5_url;
                    return;
                }

                FastIm.windowState.fastimWindow = true;
                if (!FastIm.ws.ready) {
                    FastIm.pop.absolutePrompt(FastIm.config.im_name + ' 网络连接中...', 'tips', false)
                    if (FastIm.userinfo.identity != 'admin') {
                        FastIm.socket.connect(); // 用户打开窗口，直接连接 websocket
                    }
                }
                $('.fastim-modal').show()
                FastIm.hideWindow('light-prompt')
                $('.fastim-button-container').fadeOut(100)
                setTimeout(function () {
                    $('.fastim-modal').addClass('in')
                }, 20);

                if (FastIm.windowState.messageWindow) {
                    FastIm.common.messageCursorToEnd();
                    FastIm.messageRange = window.getSelection().getRangeAt(0)
                }

                typeof FastIm.showMessageWindowBack == "function" && FastIm.showMessageWindowBack()
                FastIm.showMessageWindowBack = null

                typeof FastIm.showFastimWindowBack == "function" && FastIm.showFastimWindowBack()
                FastIm.showFastimWindowBack = null

                FastIm.escape = new Set(['fastim', ...FastIm.escape]);

                if (!localStorage.getItem('fastim_non_first_visit')) {
                    localStorage.setItem('fastim_non_first_visit', true);
                }
            }],
            ['message', () => {
                FastIm.windowState.messageWindow = true
                $('.fastim-main-window,.tab-new-message').hide()
                $('.fastim-session-list,.session-window').show()
                typeof FastIm.showMessageWindowBack == "function" && FastIm.showMessageWindowBack()
                FastIm.showMessageWindowBack = null
            }],
            ['contacts', () => {
                FastIm.windowState.messageWindow = false
                $('.fastim-main-window').hide()
                $('.fastim-address-list').fadeIn(100)
                if (!FastIm.getGroupsContacts) {
                    FastIm.getGroups('contacts');
                    FastIm.getGroupsContacts = true;
                }

                // 重装已选择分组的数据
                $('.fastim-groups .address-group-box.active').trigger('click')
            }],
            ['collection', () => {
                FastIm.windowState.messageWindow = false
                $('.fastim-main-window').hide()
                $('.collection-group').fadeIn(100)
                if (!FastIm.getGroupsCollection) {
                    FastIm.getGroups('collection');
                    FastIm.getGroupsCollection = true;
                }

                // 重装已选择分组的数据
                $('.fastim-groups .collection-group-box.active').trigger('click')
            }],
            ['to-do', () => {
                $('.fastim-to-do').fadeIn(100)
                FastIm.common.editEscape('to-do', 'add')

                // 加载待办
                let message = {
                    c: 'ImBase',
                    a: 'loadTODO',
                    data: {
                        'method': 'get'
                    }
                }
                FastIm.socket.send(message);
            }],
            ['add-to-do', () => {
                $('.fastim-to-do .add-to-do-panel').fadeIn(100)
                FastIm.common.editEscape('add-to-do', 'add')
            }],
            ['fastim-config', () => {
                FastIm.hideWindow('all-popup')
                $('.fastim-config').fadeIn()
                FastIm.common.editEscape('fastim-config', 'add')
            }],
            ['edit-fast-reply', () => {
                $('.edit-fast-reply').fadeIn()
                document.getElementById("fast_reply_form").reset()
                $('.edit-fast-reply input[name="id"]').val('0')
                FastIm.common.editEscape('edit-fast-reply', 'add')
            }],
            ['fastim-report', () => {
                FastIm.hideWindow('all-popup')
                $('.fastim-report').fadeIn()
                FastIm.common.editEscape('fastim-report', 'add')
            }],
            ['create-new-contact', () => {
                FastIm.hideWindow('all-popup')
                $('.create-new-contact').fadeIn(200);
                $('.search-new-contact-input').trigger('focus')
                FastIm.common.editEscape('create-new-contact', 'add')
            }],
            ['fastim-pickuser', () => {
                FastIm.hideWindow('all-popup')
                $('.fastim-pickuser').fadeIn()
                FastIm.common.editEscape('fastim-pickuser', 'add')
                let message = {
                    c: 'User',
                    a: 'loadSessionList',
                    data: {
                        method: 'pickuser'
                    }
                };
                FastIm.socket.send(message);

                FastIm.pickUserGroup = [], FastIm.pickChecked = new Set();
                $('.fastim-pickuser .pick-selected').empty()
                $('.fastim-pickuser .pick-left .pick-tab.active').removeClass('active')
                $('.fastim-pickuser .pick-left .pick-tab[data-tab="pick-lately"]').addClass('active')
            }],
            ['fastim-search-result', () => {
                $('.fastim-search-result').show()
                FastIm.common.editEscape('fastim-search-result', 'add')
            }],
            ['fake-tourist', () => {
                $('.service-window,.message-window').hide()
                $('.fake-tourist').show()
            }],
            ['message-window', () => {
                $('.service-window,.fake-tourist').hide()
                $('.message-window').show()
                $('#fastim-message').trigger('focus')
            }],
            ['service-window', () => {
                $('.message-window,.fake-tourist').hide()
                $('.service-window').show()
                FastIm.hideWindow('all-minor')
            }],
            ['smiley', () => {
                FastIm.hideWindow('all-minor')
                FastIm.hideWindow('all-popup')
                $('.fastim-emoji').fadeIn(100);
                $('.toolbar-group .smiley').parent().addClass('fastim-bg-gray')
                FastIm.common.editEscape('smiley', 'add')
            }],
            ['link-input', () => {
                FastIm.hideWindow('all-minor')
                FastIm.hideWindow('all-popup')
                $('.link-input').fadeIn(100);
                $('.toolbar-group .link').parent().addClass('fastim-bg-gray')
                $('.link-input .link_url').trigger('focus').val('https://').select()
                FastIm.common.editEscape('link-input', 'add')
            }],
            ['fast-reply-select', () => {
                FastIm.hideWindow('all-minor')
                FastIm.hideWindow('all-popup')
                $('.fast-reply-select').fadeIn()
                FastIm.common.editEscape('fast-reply-select', 'add')
            }],
            ['preview-img', () => {
                FastIm.hideWindow('all-minor')
                FastIm.hideWindow('all-popup')
                $('.fastim-shade').fadeIn()
                $('.fastim-preview-img').show()
                FastIm.common.editEscape('preview-img', 'add')
            }],
            ['fastim-userinfo', () => {
                // 不关闭创建新联系窗口
                if (FastIm.escape.has('create-new-contact')) {
                    $('.fastim-popup').not('.create-new-contact').hide()
                    FastIm.common.editEscape('create-new-contact', 'demand')
                } else {
                    FastIm.hideWindow('all-popup')
                }

                $('.fastim-userinfo-box,.fastim-userinfo-detail').remove()

                $('.fastim-userinfo').fadeIn()
                FastIm.common.editEscape('fastim-userinfo', 'add')
            }],
            ['fastim-link-popup', () => {
                $('.fastim-link-popup').fadeIn()
                FastIm.common.editEscape('fastim-link-popup', 'add')
            }],
            ['im-add-group-notice-popup', () => {
                $('.im-add-group-notice-popup').fadeIn()
                FastIm.common.editEscape('im-add-group-notice-popup', 'add')
            }],
            ['new-group-notice-popup', () => {
                $('.new-group-notice-popup').fadeIn()
                FastIm.common.editEscape('new-group-notice-popup', 'add')
            }],
            ['im-user-list-popup', () => {
                $('.im-user-list-popup').fadeIn()
                FastIm.common.editEscape('im-user-list-popup', 'add')
            }],
            ['fastim-at-users', () => {
                $('.fastim-at-users').fadeIn()
                FastIm.common.editEscape('fastim-at-users', 'add')
            }]
        ]);

        let action = showFun.get(name) || showFun.get('default')
        return action.call(this);
    },
    hideWindow: function (name) {
        let hideFun = new Map([
            ['fastim', () => {
                FastIm.windowState.fastimWindow = false;
                $('.fastim-modal').removeClass('in');
                $('.fastim-modal').fadeOut(100);
                $('.fastim-button-container').fadeIn(300)
                FastIm.hideWindow('all-popup')
                FastIm.hideWindow('all-minor');
                FastIm.common.editEscape('fastim')
            }],
            ['message-window', () => {
                console.log('隐藏message窗口')
            }],
            ['all-popup', () => {
                // 隐藏各类弹窗
                if (FastIm.doNotHidePopup) {
                    FastIm.doNotHidePopup = false
                    return;
                }
                $('.fastim-popup').hide()
                $('.fastim-popup-menu').fadeOut()
                $('.fastim-shade').fadeOut()
                $('.fastim-preview-img').hide()
                FastIm.hideWindow('fastim-confirm')
                FastIm.hideWindow('fastim-search-result')
                FastIm.common.editEscape('', 'empty')
            }],
            ['all-minor', () => {
                // 隐藏杂项小弹窗
                $('.fastim-popup-menu').fadeOut()
                $('.message-window-popup').fadeOut()
                $('.toolbar-group.fastim-bg-gray').removeClass('fastim-bg-gray')
                $('.fastim-contextmenu').remove()
                FastIm.hideWindow('fastim-at-users')
            }],
            ['to-do', () => {
                $('.fastim-to-do').fadeOut(100)
                FastIm.common.editEscape('to-do')
            }],
            ['add-to-do', () => {
                $('.fastim-to-do .add-to-do-panel').hide()
                FastIm.common.editEscape('add-to-do')
            }],
            ['fastim-config', () => {
                $('.fastim-config').fadeOut()
                FastIm.common.editEscape('fastim-config')
            }],
            ['edit-fast-reply', () => {
                $('.edit-fast-reply').fadeOut()
                FastIm.common.editEscape('edit-fast-reply')
            }],
            ['fastim-confirm', () => {
                $('.fastim-confirm').remove()
                FastIm.common.editEscape('fastim-confirm')
            }],
            ['fastim-report', () => {
                $('.fastim-report').fadeOut()
                FastIm.tempImages = []
                $('.fastim-report-form .upload-box-image').not('.fastim-report-form .upload-box-image:first').remove()
                FastIm.common.editEscape('fastim-report')
            }],
            ['create-new-contact', () => {
                $('.search-new-contact-input').val('')
                $('.new-contact-list').hide()
                $('.create-new-contact').hide()
                FastIm.common.editEscape('create-new-contact')
            }],
            ['fastim-pickuser', () => {
                $('.fastim-pickuser').fadeOut()
                FastIm.common.editEscape('fastim-pickuser')
            }],
            ['fastim-search-result', () => {
                $('.fastim-search-result').fadeOut()
                FastIm.common.editEscape('fastim-search-result')
                FastIm.searchSessionSelect = false
                FastIm.searchResult = []
            }],
            ['message-window', () => {
                $('.message-window').hide()
            }],
            ['smiley', () => {
                $('.fastim-emoji').fadeOut(100);
                $('.toolbar-group .smiley').parent().removeClass('fastim-bg-gray')
                FastIm.common.editEscape('smiley')
            }],
            ['link-input', () => {
                $('.link-input').fadeOut(100);
                $('.toolbar-group .link').parent().removeClass('fastim-bg-gray')
                FastIm.common.editEscape('link-input')
            }],
            ['fast-reply-select', () => {
                $('.fast-reply-select').fadeOut()
                FastIm.common.editEscape('fast-reply-select')
            }],
            ['preview-img', () => {
                $('.fastim-shade').fadeOut()
                $('.fastim-preview-img').hide()
                FastIm.common.editEscape('preview-img')
            }],
            ['fastim-userinfo', () => {
                if (FastIm.groupNoticeWiden) {
                    $('.fastim-userinfo').css({
                        left: 'calc(50% - 190px)',
                        width: '380px'
                    })
                    FastIm.groupNoticeWiden = false
                }
                $('.fastim-userinfo').hide()
                FastIm.groupChatInfo = false
                FastIm.common.editEscape('fastim-userinfo')
            }],
            ['edit-groupmember-nickname', () => {
                // 关闭群昵称修改
                FastIm.saveGroupmemberNicknameBool = true
                var editNicknameEl = $('input[data-action="save-groupmember-nickname"]');
                var data = editNicknameEl.data('data')
                var value = editNicknameEl.data('value')
                var editGroupchatNickname = '<span class="fastim-click fastim-color-blue" data-action="edit-groupmember-nickname" data-value="' + value + '" data-data="' + data + '">' + (value != '' ? value : '点击编辑') + '</span>';
                FastIm.common.editEscape('edit-groupmember-nickname')
                try {
                    editNicknameEl.replaceWith(editGroupchatNickname)
                } catch (err) {
                    // 按下ecs时,可能元素已被blur事件删除掉了
                }
            }],
            ['save-userinfo-remark', () => {
                // 关闭好友备注修改
                FastIm.saveUserRemarkBool = true
                var el = $('.fastim-userinfo-box .imedit_userinfo_remark_input'),
                    remark = el.data('remark'),
                    id = el.data('id')
                var remarkEl = '<div data-action="edit_remark" data-id="' + id + '" data-data="' + remark + '" class="fastim-click imedit_userinfo_remark">修改备注</div>';
                FastIm.common.editEscape('save-userinfo-remark')
                try {
                    el.replaceWith(remarkEl)
                } catch (err) {
                    // 按下ecs时,可能元素已被blur事件删除掉了
                }
            }],
            ['update-group-name-input', () => {
                // 关闭分组重命名输入框
                FastIm.saveUpdateGroupNameBool = true
                var el = $('.fastim-group-box .fastim-group-input'),
                    id = el.data('id'),
                    value = el.data('value');

                var groupEl = '<div data-action="load-group-data" data-id="' + id + '" class="fastim-group-box group-id-' + id + ' address-group-box fastim-click">' + value + '</div>';
                FastIm.common.editEscape('update-group-name-input')
                try {
                    el.parent().replaceWith(groupEl)
                    $('.fastim-group-box.group-id-' + id).trigger('click')
                } catch (err) {
                    // 按下ecs时,可能元素已被blur事件删除掉了
                }
            }],
            ['add-group-name-input', () => {
                // 关闭分组添加输入框
                $('.fastim-new-group').remove()
                FastIm.common.editEscape('add-group-name-input')
            }],
            ['new-contact-apply', () => {
                $('.fastim-popup.new-contact-apply-box').remove()
                FastIm.common.editEscape('new-contact-apply')
            }],
            ['fastim-link-popup', () => {
                $('.fastim-link-url').empty()
                $('.fastim-link-url-a').attr('href', 'javascript:;')
                $('.fastim-link-popup').hide()
                FastIm.common.editEscape('fastim-link-popup')
            }],
            ['light-prompt', () => {
                $('.fastim-light-prompt').fadeOut(200)
            }],
            ['im-add-group-notice-popup', () => {
                $('.im-add-group-notice-popup').fadeOut()
                FastIm.tempImages = []
                $('.group-notice .upload-box-image').not('.group-notice .upload-box-image:first').remove()
                $('#group_notice_group_id').val('')
                FastIm.common.editEscape('im-add-group-notice-popup')
            }],
            ['new-group-notice-popup', () => {
                $('.new-group-notice-popup').fadeOut()
                FastIm.common.editEscape('new-group-notice-popup')
            }],
            ['im-user-list-popup', () => {
                $('.im-user-list-popup').fadeOut()
                FastIm.common.editEscape('im-user-list-popup')
            }],
            ['fastim-at-users', () => {
                if ($('.fastim-at-users').css('display') == 'block') {
                    var atSpan = $('.' + FastIm.atInputClassName)
                    if (atSpan) {
                        atSpan.remove();
                    }
                    var selection = window.getSelection()
                    selection.removeAllRanges();
                    selection.addRange(FastIm.messageRange);

                    $('#at_users_search').val('');
                    $('.fastim-at-users').fadeOut(100)
                    $('#fastim-message').trigger('focus')
                    FastIm.common.editEscape('fastim-at-users')
                }
            }]
        ]);
        let action = hideFun.get(name) || hideFun.get('default')
        return action.call(this);
    },
    searchNewContact: function (type) {
        var keyword = $("input[name='search-new-" + type + "']").val()
        if (!keyword) {
            FastIm.pop.msg('请输入搜索关键词~', 'tips');
            return;
        }

        let message = {
            c: 'User',
            a: 'searchNewContact',
            data: {
                'keyword': keyword,
                'type': type,
                'page': 1
            }
        }
        FastIm.socket.send(message);
    },
    getGroups: function (type) {
        let message = {
            c: 'ImBase',
            a: 'getGroups',
            data: {
                'type': type
            }
        }
        FastIm.socket.send(message);
    },
    /**
     * 会话列表的会话保障
     * 新会话生成
     * 最后消息更新
     * 消息阅读数更新等
     * @param  {obj} data 会话数据
     */
    imSession: function (data) {
        $('.fastim-session-ul .fastim-data-none').remove()
        var sessionElement = $('.fastim-session-li[data-data="' + data.sessionInfo.id + '"]')
        if (sessionElement.length) {
            sessionElement.find(".session-info-time").eq(0).html(data.lastMessage.last_time)
            sessionElement.find(".last-message-text .last-message").eq(0).html(data.lastMessage.last_message)
            if (data.lastMessage.unread_fixed_msg) {
                sessionElement.find(".last-message-fixed").eq(0).html('[' + data.lastMessage.unread_fixed_msg.message + ']')
            }
            FastIm.common.unreadMessagesDisplay(data.sessionInfo.id, data.unreadMessagesNumber, data.shield)

            let firstSession = $('.fastim-session-ul li.session-top');

            // 将会话移动到第一位
            if (data.sessionInfo.top || firstSession.length == 0) {
                $('.fastim-session-ul li').eq(0).before(sessionElement);
            } else if (firstSession.length > 0) {
                // 将会话移动到非置顶会话的第一位
                firstSession.eq(firstSession.length - 1).after(sessionElement);
            }

            // 收到消息时，确认可能更新的用户备注
            if (sessionElement.data('type') == 'single' && sessionElement.find(".session-info-name").text() != data.sessionInfo.pushUser.nickname && data.sessionInfo.pushUser.nickname != FastIm.userinfo.nickname) {
                sessionElement.find(".session-info-name").text(data.sessionInfo.pushUser.nickname)
                sessionElement.find(".session-avatar").attr('src', FastIm.common.imgUrl(data.sessionInfo.pushUser.avatar))
                $('.message-window .chat-window-title .chat-nickname').text(data.sessionInfo.pushUser.nickname)
            }
        } else {
            if (data.sessionInfo.type == 'single') {
                if (data.sessionInfo.pushUser.status) {
                    if (parseInt(data.sessionInfo.pushUser.status.value) == 0 || parseInt(data.sessionInfo.pushUser.status.value) == 3) {
                        data.sessionInfo.pushUser.avatar_gray = 'fastim-img-gray'
                        data.sessionInfo.pushUser.user_status = '<span class="session-user-status">[离线]</span>'
                    } else if (parseInt(data.sessionInfo.pushUser.status.value) == 2) {
                        data.sessionInfo.pushUser.user_status = '<span class="session-user-status">[忙碌]</span>'
                    }
                }
            } else {
                data.sessionInfo.pushUser.user_status = ''
            }

            data.shield = data.shield ? 'session-shield' : ''
            data.unreadMessagesNumber = data.unreadMessagesNumber > 0 ? '<span class="session-unread-count ' + data.shield + '">' + data.unreadMessagesNumber + '</span>' : ''
            data.sessionInfo.pushUser.avatar = FastIm.common.imgUrl(data.sessionInfo.pushUser.avatar);
            if (data.lastMessage.unread_fixed_msg) {
                data.lastMessage.unread_fixed_msg.message = '[' + data.lastMessage.unread_fixed_msg.message + ']'
            }
            var sessionTemplate = FastIm.template.session();
            var newHTML = sessionTemplate(data)
            let firstSession = $('.fastim-session-ul li.session-top');
            if (data.sessionInfo.top || firstSession.length == 0) {
                $('.fastim-session-ul').prepend(newHTML)
            } else if (firstSession.length > 0) {
                firstSession.eq(firstSession.length - 1).after(newHTML);
            }
        }
    },
    /**
     * 发送IM消息
     * @param  {obj} message 消息内容
     * @param  {int} type    消息类型
     */
    sendMessage: function (message, type) {

        if (!message) {
            FastIm.pop.msg('请输入消息内容~')
            return;
        }

        if (FastIm.fastImMessageContenteditable) {
            FastIm.pop.msg('管理员已禁止发言~')
            return;
        }

        FastIm.hideWindow('all-minor')

        // 检查 websocket 是否连接
        if (!FastIm.ws.socketTask || FastIm.ws.socketTask.readyState != 1) {
            FastIm.pop.msg(FastIm.config.im_name + ' 网络链接异常，请刷新重试~')
            return;
        }

        if (!FastIm.sessionId) {
            FastIm.pop.msg('请选择一个会话~')
            return;
        }

        // 当前会话
        var sessionEl = $('.fastim-session-li[data-data="' + FastIm.sessionId + '"]')

        var messageId = new Date().getTime() + FastIm.sessionId + Math.floor(Math.random() * 10000); // 临时消息ID
        var messageData = {
            c: 'Message',
            a: 'sendMessage',
            data: {
                message: message,
                type: type,
                session_id: FastIm.sessionId,
                tokens: FastIm.tokens ? FastIm.tokens : false, // 发消息时检测用户登录态是否过期
                message_id: messageId,
                identity: FastIm.userinfo.identity,
                at_users: sessionEl.data('type') == 'group' ? FastIm.atUsers:[]
            }
        };
        FastIm.socket.send(messageData);
        FastIm.atUsers = []

        var data = {
            messageData: {
                id: messageId,
                type: type,
                record_content_before: 'record_content_before',
                status: FastIm.common.messageStatus(3, messageId),
                message: FastIm.build.message({message: message, type: type}),
                sender: 'me',
            },
            sessionInfo: {
                id: FastIm.sessionId,
                top: sessionEl.hasClass('session-top'),
                pushUser: {
                    id: FastIm.userinfo.id,
                    avatar: FastIm.userinfo.avatar,
                    nickname: FastIm.userinfo.nickname,
                    hidden_nickname: 'fastim-hidden'
                }
            },
            lastMessage: {
                last_time: FastIm.common.getHoursMinutes(),
                last_message: FastIm.common.formatMessage({message: message, type: type}),
            },
            shield: false,
            unreadMessagesNumber: 0,
        };

        var messageTemplate = FastIm.template.message()
        var newHTML = messageTemplate(data)
        var recordsEL = $('.chat-records')
        recordsEL.append(newHTML)
        FastIm.imSession(data)
        $('#fastim-message').html('')
        $('.chat-records .fastim-data-none').remove()
        recordsEL.scrollTop(recordsEL[0].scrollHeight)
    },
    eventReg: function () {
        $(document).ready(function () {

            // 过滤FF浏览器消息输入框中的html标签
            if (FastIm.common.browserType() == "FF") {
                $(document).on('paste', '#fastim-message', function (e) {
                    setTimeout(function () {
                        var message = $(e.target).html();
                        $(e.target).html(message.replace("/<(?!img|div).*?>/g", ''));
                        FastIm.common.messageCursorToEnd();
                    }, 100);
                });
            }

            // 阻止所有表单提交
            $(document).on('submit', '.fastim-ins form', function () {
                return false;
            });

            // 禁止回车键换行
            $(document).on('keypress', '#fastim-message', function (event) {
                if (!event.ctrlKey && event.keyCode == "13" && parseInt(FastIm.config.user_config.send_message_key) == 0) {
                    event.preventDefault()
                }
            });

            $(window).on('resize', function () {
                FastIm.hideWindow('light-prompt')
            });

            // 鼠标移出某元素
            $(document).on('mouseleave', '.fastim-popup-menu,.fastim-contextmenu', function (e) {
                var el = $(e.currentTarget)
                if (el.hasClass('fastim-popup-menu')) {
                    // 隐藏popup菜单
                    $('.fastim-popup-menu').fadeOut(200)
                } else if (el.hasClass('fastim-contextmenu')) {
                    // 隐藏右击菜单
                    $('.fastim-contextmenu').remove()
                }
            });

            // 隐藏各类工具面板
            $(document).on('click', '.fastim-window-sidebar,.fastim-window-middle,.fastim-window-right,.address-menu-tab', function () {
                FastIm.hideWindow('all-minor')
                FastIm.hideWindow('all-popup')
            });

            // 聚焦
            $(document).on('focus', '.im-on-focus', function (e) {
                var el = $(e.currentTarget)
                if (el.hasClass('add-blue-border')) {
                    el.parent().addClass('im-input-focus-on')
                } else if (el[0].id == 'fastim-message') {
                    FastIm.messageRange = window.getSelection().getRangeAt(0)
                }
            });

            // 脱焦
            $(document).on('blur', '.im-on-focus', function (e) {
                var el = $(e.currentTarget)
                if (el.hasClass('add-blue-border')) {
                    el.parent().removeClass('im-input-focus-on')
                } else if (el[0].id == 'fastim-message' && FastIm.inputStatusNotice) {
                    let message = {
                        c: 'Message',
                        a: 'inputStatusNotice',
                        data: {
                            'end_input_session_id': FastIm.inputStatusNotice
                        }
                    }
                    FastIm.socket.send(message);
                    FastIm.inputStatusNotice = 0
                } else if (el.hasClass('imedit_userinfo_remark_input')) {
                    // 保存编辑好友备注
                    FastIm.common.saveUserRemark();
                } else if (el.hasClass('save-groupmember-nickname-input')) {
                    // 保存群昵称修改
                    FastIm.common.saveGroupmemberNickname()
                } else if (el.hasClass('fastim-group-input')) {
                    // 修改分组名称、添加分组
                    var action = el.data('action');
                    if (action == 'update-group-name') {
                        FastIm.common.saveUpdateGroupName(el)
                    } else if (action == 'add-new-group') {
                        var newname = el.val(),
                            type = el.data('type');

                        if (!newname) {
                            FastIm.pop.msg('分组名称不能为空！');
                            FastIm.hideWindow('add-group-name-input')
                            return;
                        }

                        let message = {
                            c: 'ImBase',
                            a: 'updateGroup',
                            data: {
                                type: type,
                                name: newname,
                                action: 'add-new-group'
                            }
                        }
                        FastIm.socket.send(message);
                    }
                }
            });

            // 通用的输入监听、输入时改变 button 状态
            $(document).on('input propertychange', '#fastim-message,.active-button,.fastim-search-input,#at_users_search', function (event) {
                var e = $(event.currentTarget);
                var sessionType = $('.fastim-session-li[data-data="' + FastIm.sessionId + '"]').data('type')

                if (e.hasClass('fastim-search-input')) {
                    FastIm.searchSessionSelect = 0;
                    return false;
                } else if (e[0].id == 'at_users_search') {
                    FastIm.common.atInput('search', e.val());
                    return false;
                } else if (e[0].id == 'fastim-message') {

                    // 若在按键事件中监听可能出现中文输入法无法检测到@输入
                    if (event.originalEvent.data == '@') {
                        FastIm.common.atInput('start');
                    }

                    FastIm.messageRange = window.getSelection().getRangeAt(0)
                    if (FastIm.sessionId != FastIm.inputStatusNotice && parseInt(FastIm.config.user_config.input_status) == 1 && sessionType == 'single') {
                        let message = {
                            c: 'Message',
                            a: 'inputStatusNotice',
                            data: {
                                'session_id': FastIm.sessionId
                            }
                        }
                        FastIm.socket.send(message);
                        FastIm.inputStatusNotice = FastIm.sessionId
                    }
                }

                var focus = e.data('focus');
                if (focus && (e.html() || e.val())) {
                    $('.' + focus).addClass('fastim-bg-blue')
                } else {
                    $('.' + focus).removeClass('fastim-bg-blue')
                }
            });

            // change事件
            $(document).on('change', '.pick-checkbox,.im-change', function (e) {
                var e = $(e.currentTarget),
                    val = e.val()
                if (e.hasClass('pick-checkbox')) {
                    if (e[0].checked) {
                        FastIm.pickChecked.add(val)

                        var el = e.parent().clone(),
                            forid = el.find('.pick-checkbox').val()
                        el.removeAttr('for');
                        el.addClass('pick-checkbox-' + forid)
                        el.find('.pick-checkbox').remove()
                        $('.pick-selected').append(el)

                        // 选择所有此值
                        $('.impick-value-' + val).prop('checked', true)
                    } else if (FastIm.pickChecked.has(val)) {
                        FastIm.pickChecked.delete(val)
                        $('.pick-selected .pick-checkbox-' + val).remove()

                        // 取消选择所有此值
                        $('.impick-value-' + val).prop('checked', false)
                    }
                } else if (e.hasClass('fastim-friend-group')) {
                    let message = {
                        c: 'User',
                        a: 'updateFriendInfo',
                        data: {
                            id: e.data('id'),
                            new_group: val,
                            method: 'update_group'
                        }
                    }
                    FastIm.socket.send(message);
                } else if (e.hasClass('fastim-groupchat-group')) {
                    let message = {
                        c: 'User',
                        a: 'updateFriendInfo',
                        data: {
                            id: e.data('id'),
                            new_group: val,
                            method: 'update_groupchat_group'
                        }
                    }
                    FastIm.socket.send(message);
                }
            });

            // 文件上传
            $(document).on('change', '.fastim-upload', function (e) {
                var elID = e.target.id;
                if ((elID == 'reportimage' && FastIm.tempImages.length >= 4) || (elID == 'groupnoticeimage' && FastIm.tempImages.length >= 10)) {
                    FastIm.pop.msg('图片数量已达上限!');
                    $('#' + elID).val('')
                    return false;
                }

                if ((elID == 'fastimimage' || elID == 'fastimfile') && FastIm.fastImMessageContenteditable) {
                    FastIm.pop.msg('管理员已禁止发言!');
                    return false;
                }

                var file = $('#' + elID)[0].files[0];
                if (!file) {
                    return;
                }

                FastIm.hideWindow('all-minor');

                // 上传文件
                var formData = new FormData();
                formData.append("file", file);
                if (FastIm.config.upload.multipart) {
                    for (let i in FastIm.config.upload.multipart) {
                        formData.append(i, FastIm.config.upload.multipart[i]);
                    }
                }

                $.ajax({
                    url: FastIm.build.url('upload'),
                    type: 'post',
                    data: formData,
                    contentType: false,
                    processData: false,
                    success: function (res) {
                        if (res.code == 1) {
                            // 发送notify请求
                            if (FastIm.config.upload.storage != 'local') {
                                FastIm.getFileMd5(file, function (md5) {
                                    var ajaxData = {
                                        size: file.size,
                                        name: file.name,
                                        md5: md5,
                                        type: file.type,
                                        url: res.data.url
                                    };

                                    for (let i in FastIm.config.upload.multipart) {
                                        ajaxData[i] = FastIm.config.upload.multipart[i];
                                    }

                                    $.ajax({
                                        url: FastIm.build.url('storage-notify'),
                                        type: 'POST',
                                        data: ajaxData
                                    });
                                })
                            }

                            if (elID == 'reportimage' || elID == 'groupnoticeimage') {
                                FastIm.tempImages.push(res.data.fullurl)
                                let imagesEl = elID == 'reportimage' ? $('.fastim-reportimage'):$('.im-add-group-notice-popup .im-group-notice-imgs')
                                imagesEl.append(FastIm.build.widget('upload-box-image', {
                                    'fullurl': res.data.fullurl
                                }));
                            } else if (elID == 'uploadavatar') {
                                // 用户头像上传
                                var el = $(e.target),
                                    type = el.data('type'),
                                    id = el.data('id')
                                let message = {
                                    c: 'User',
                                    a: 'infoDetail',
                                    data: {
                                        url: res.data.fullurl,
                                        type: type,
                                        id: id,
                                        method: 'post-avatar'
                                    }
                                }
                                FastIm.socket.send(message);
                            } else {
                                let fileName = res.data.url.split('.');
                                let fileSuffix = fileName[fileName.length - 1];
                                let imgSuffix = ['png', 'jpg', 'gif', 'jpeg', 'bmp'];
                                let audioSuffix = ['mp3', 'mpeg', 'wav', 'acc'];
                                let videoSuffix = ['mp4', 'ogg', 'webm'];

                                if (imgSuffix.includes(fileSuffix)) {
                                    FastIm.sendMessage(res.data.fullurl, 'image');
                                } else if (audioSuffix.includes(fileSuffix)) {
                                    FastIm.sendMessage(res.data.fullurl, 'audio');
                                } else if (videoSuffix.includes(fileSuffix)) {
                                    FastIm.sendMessage(res.data.fullurl, 'video');
                                } else {
                                    var fileSize = (file.size / 1024).toFixed(2)
                                    FastIm.sendMessage({
                                        url: res.data.fullurl,
                                        size: fileSize > 1024 ? (fileSize / 1024).toFixed(2) + 'Mb' : fileSize + 'Kb',
                                        suffix: fileSuffix
                                    }, 'file');
                                }
                            }
                        } else {
                            FastIm.pop.msg(res.msg)
                        }
                    },
                    error: function (e) {
                        FastIm.pop.msg('文件上传失败,请重试！')
                    },
                    complete: function () {
                        $('#' + elID).val('')
                    }
                });
            });

            // 通用tab切换
            $(document).on('click', '.fastim-tab', function (e) {
                FastIm.common.stopPropagation(e);
                var e = $(e.currentTarget)
                var tab = e.data('tab');
                var toggle = e.data('toggle');

                if (e.hasClass('active')) {
                    return;
                } else {
                    $('.' + toggle + '.active').removeClass('active');
                    e.addClass('active')
                }

                $('.' + toggle + '-panel').hide()
                $('.' + tab + '-panel').fadeIn(100)

                if (tab == 'fast-reply') {
                    let message = {
                        c: 'ImBase',
                        a: 'fastReply',
                        data: {
                            method: 'get',
                            page: 1
                        }
                    }
                    FastIm.socket.send(message);
                } else if (tab == 'pick-friends') {
                    // 加载好友分组
                    $('.fastim-pickuser .pick-middle .pick-widget .group-search-result').remove() // 清理搜索结果
                    $('.fastim-pickuser .pick-middle .pick-widget .search-result-group').remove() // 清理搜索分组
                    $('.fastim-pickuser .pick-middle .pick-widget .pick-item').hide()

                    // 好友-带分组
                    if (FastIm.pickUserGroup.includes('pickgroup')) {
                        $('.fastim-pickuser .pick-middle .pick-widget .pick-group').show()
                    } else {
                        FastIm.getGroups('pick-contacts');
                    }
                } else if (tab == 'pick-lately') {
                    $('.fastim-pickuser .pick-middle .pick-widget .group-search-result').remove()
                    $('.fastim-pickuser .pick-middle .pick-widget .search-result-group').remove()
                    $('.fastim-pickuser .pick-middle .pick-widget .pick-group .pick-group-status').removeClass('rotate-90');

                    // 最近会话
                    if (FastIm.pickUserGroup.includes("lately")) {
                        $('.fastim-pickuser .pick-middle .pick-group').hide()
                        $('.fastim-pickuser .pick-middle .pick-item').hide()
                        $('.fastim-pickuser .group-lately').show()

                        // 勾选已选择的
                        $('.fastim-pickuser .group-lately .pick-checkbox:checkbox:checked').prop('checked', false);
                        for (var item of FastIm.pickChecked) {
                            $('#pick-lately-item-' + item).prop('checked', true)
                        }
                    }
                } else if (tab == 'groupinfo-member') {
                    var id = e.data('data')
                    if (id) {
                        // 加载群资料面板的群成员列表
                        let message = {
                            c: 'Message',
                            a: 'groupChatMember',
                            data: {
                                'id': id,
                                'method': 'get'
                            }
                        }
                        FastIm.socket.send(message);
                    }
                } else if (tab == 'groupinfo-notice') {
                    var id = e.data('data')
                    FastIm.socket.send({
                        c: 'Message',
                        a: 'groupChatNotice',
                        data: {
                            'group_id': id,
                            'method': 'list',
                            'page': 1
                        }
                    });
                }
            });

            // 双击事件
            // 图片预览改至layer,方便外站调用和尽可能兼容,不直接使用layer
            $(document).on('dblclick', '.record-img', function (event) {
                var e = $(event.currentTarget),
                    src = e.attr('src');

                let img = new Image();
                img.src = src;
                img.onload = function () {
                    let imgarea = [img.width, img.height];
                    let winarea = [$(window).width() - 100, $(window).height() - 100];

                    //如果 实际图片的宽或者高比 屏幕大（那么进行缩放）
                    if (imgarea[0] > winarea[0] || imgarea[1] > winarea[1]) {
                        var wh = [imgarea[0] / winarea[0], imgarea[1] / winarea[1]];//取宽度缩放比例、高度缩放比例
                        if (wh[0] > wh[1]) {//取缩放比例最大的进行缩放
                            imgarea[0] = imgarea[0] / wh[0];
                            imgarea[1] = imgarea[1] / wh[0];
                        } else if (wh[0] < wh[1]) {
                            imgarea[0] = imgarea[0] / wh[1];
                            imgarea[1] = imgarea[1] / wh[1];
                        }
                    }

                    FastIm.showWindow('preview-img')
                    $('.fastim-preview-img').css({
                        width: imgarea[0] + 'px',
                        height: imgarea[1] + 'px',
                        left: 'calc(50% - ' + (imgarea[0] / 2) + 'px)',
                        top: 'calc(50% - ' + (imgarea[1] / 2) + 'px)'
                    });
                    $('.fastim-preview-img .fastim-img-fill').attr('src', src)
                };
                img.onerror = function (e) {
                    console.log('图片加载失败', e)
                    FastIm.pop.msg('图片加载失败,请重试~', 'error');
                };
                FastIm.common.stopPropagation(event);
            });

            // 右击
            $(document).on('contextmenu', '.fastim-group-box, .fastim-session-li, .fastim-contextmenu-item, .record-content-box,.collection-data-tbody .fastim-titem,.groupchat-member-item', function (event) {
                var e = $(event.currentTarget)
                if (e.hasClass('fastim-group-box') && !e.hasClass('collection-group-box')) {
                    // 好友分组右击-收藏的分组未提供右击菜单且屏蔽浏览器右击菜单
                    if (id != 'new-group') {
                        // 选中该分组
                        e.trigger('click')
                    }
                    var id = e.data('id')
                    if (parseFloat(id).toString() == "NaN") {
                        var menu = [{'action': 'group-add', 'name': '添加分组'},];
                    } else {
                        var menu = [
                            {'action': 'group-moveup', 'name': '上移'},
                            {'action': 'group-movedown', 'name': '下移'},
                            {'action': 'group-rename', 'name': '重命名'},
                            {'action': 'group-add', 'name': '添加分组'},
                            {'action': 'group-del', 'name': '删除分组'}
                        ];
                    }

                    FastIm.common.contextMenu({top: event.pageY, left: (event.pageX + 14)}, menu, id);
                } else if (e.hasClass('fastim-session-li')) {
                    var type = e.data('type'),
                        data = e.data('data')
                    var menu = [
                        {'action': 'session-top', 'name': e.hasClass('session-top') ? '取消置顶' : '设为置顶'}
                    ];

                    // 只用户和群聊显示查看资料
                    if (type == 'single' || type == 'group') {
                        menu.push({'action': 'session-info', 'name': '查看资料'});
                    }

                    menu.push({'action': 'session-remove', 'name': '从会话列表移除'});

                    // 获取群消息的群聊消息状态
                    if (type == 'group') {
                        let message = {
                            c: 'Message',
                            a: 'getGroupChatShield',
                            data: {
                                'session_id': data
                            }
                        }
                        FastIm.socket.send(message);
                    }
                    FastIm.common.contextMenu({top: event.pageY, left: (event.pageX + 14)}, menu, data);
                } else if (e.hasClass('record-content-box')) {
                    var id = e.parent().data('id'),
                        type = e.parent().data('type');
                    if (type == 'default') {
                        var menu = [
                            {'action': 'message-forward', 'name': '转发'},
                            {'action': 'message-copy', 'name': '复制'},
                            {'action': 'message-quote', 'name': '引用'},
                            {'action': 'message-collection', 'name': '添加到收藏'},
                            {'action': 'message-to-do', 'name': '添加到待办'}
                        ];
                    } else {
                        var menu = [{'action': 'message-forward', 'name': '转发'}];
                        if (type == 'link') {
                            menu.push({'action': 'message-copy', 'name': '复制链接'}, {
                                'action': 'message-quote',
                                'name': '引用链接'
                            })
                        }

                        menu.push({'action': 'message-collection', 'name': '添加到收藏'});

                        if (type == 'image' || type == 'link') {
                            menu.push({'action': 'message-new-window', 'name': '新窗口打开'})
                        }
                    }
                    let sessionType = $('.fastim-session-li[data-data="' + FastIm.sessionId + '"]').data('type')
                    if (sessionType == 'single') {
                        menu.push({'action': 'message-delete', 'name': '删除消息'});
                    }
                    FastIm.common.contextMenu({top: event.pageY, left: (event.pageX + 14)}, menu, id);
                } else if (e.hasClass('fastim-contextmenu-item')) {
                } else if (e.parent().hasClass('collection-data-tbody')) {
                    data = e.data('data')
                    var menu = [
                        {'action': 'collection-forward', 'name': '转发'},
                        {'action': 'collection-del', 'name': '删除收藏'}
                    ];
                    FastIm.common.contextMenu({top: event.pageY, left: (event.pageX + 14)}, menu, data);
                } else if (e.hasClass('groupchat-member-item')) {
                    var id = e.data('id')
                    if (!FastIm.groupChatInfo || FastIm.groupChatInfo.leader != FastIm.userinfo.id || FastIm.groupChatInfo.leader == id) {
                        return false;
                    }

                    var menu = [
                        {'action': 'del-group-member', 'name': '删除成员'}
                    ];
                    FastIm.common.contextMenu({top: event.pageY, left: (event.pageX + 16)}, menu, id);
                }

                return false;
            });

            // 点击右击菜单
            $(document).on('click', '.fastim-contextmenu-item', function (event) {
                $('.fastim-contextmenu').remove();
                var e = $(event.currentTarget),
                    action = e.data('action'),
                    id = e.data('id');

                var sessionOperation = () => {
                    let message = {
                        c: 'User',
                        a: 'sessionOperation',
                        data: {
                            id: id,
                            action: action
                        }
                    };
                    FastIm.socket.send(message);
                }

                var messageOperation = () => {
                    let message = {
                        c: 'Message',
                        a: 'messageOperation',
                        data: {
                            id: id,
                            action: action
                        }
                    };
                    FastIm.socket.send(message);
                }

                var buildFun = new Map([
                    ['group-add', () => {
                        var randomId = Math.ceil(Math.random() * 9999),
                            element = $('.fastim-group-box.group-id-' + id);
                        if (element.parent().hasClass('address-friends-panel')) {
                            // 好友
                            $('.address-friends-panel').append('<div data-id="new-group" class="fastim-group-box address-group-box fastim-new-group"><input data-type="friends" data-id="' + randomId + '" data-action="add-new-group" placeholder="请输入新组名,回车保存" class="fastim-group-input im-on-focus new-group-' + randomId + '" /></div>');
                        } else if (element.parent().hasClass('address-group-panel')) {
                            // 群聊
                            $('.address-group-panel').append('<div data-id="new-group" class="fastim-group-box address-group-box fastim-new-group"><input data-type="group" data-id="' + randomId + '" data-action="add-new-group" placeholder="请输入新组名,回车保存" class="fastim-group-input im-on-focus new-group-' + randomId + '" /></div>');
                        } else if (element.parent().hasClass('collection-group')) {
                            // 收藏
                            $('.collection-group').append('<div data-id="new-group" class="fastim-group-box collection-group-box fastim-new-group"><input data-type="collection" data-id="' + randomId + '" data-action="add-new-group" placeholder="请输入新组名,回车保存" class="fastim-group-input im-on-focus new-group-' + randomId + '" /></div>');
                        }
                        $('.fastim-group-input.new-group-' + randomId).focus()
                        FastIm.common.editEscape('add-group-name-input', 'add')
                    }],
                    ['group-del', () => {
                        var element = $('.fastim-group-box.group-id-' + id);
                        FastIm.pop.confirm({
                            point: {top: (event.pageY - 140), left: (event.pageX - 20)},
                            msg: '确认要删除分组“' + element.text() + '”吗？',
                            success: function () {
                                let message = {
                                    c: 'ImBase',
                                    a: 'updateGroup',
                                    data: {
                                        id: id,
                                        action: 'del'
                                    }
                                }
                                FastIm.socket.send(message);

                                if (element.hasClass('active')) {
                                    element.parent().find('.fastim-group-box').eq(0).click()
                                }

                                FastIm.hideWindow('fastim-confirm')
                            }
                        });
                    }],
                    ['group-moveup', () => {
                        let message = {
                            c: 'ImBase',
                            a: 'updateGroup',
                            data: {
                                id: id,
                                action: 'moveup'
                            }
                        }
                        FastIm.socket.send(message);
                    }],
                    ['group-movedown', () => {
                        let message = {
                            c: 'ImBase',
                            a: 'updateGroup',
                            data: {
                                id: id,
                                action: 'movedown'
                            }
                        }
                        FastIm.socket.send(message);
                    }],
                    ['group-rename', () => {
                        var element = $('.fastim-group-box.group-id-' + id);
                        var groupName = element.text();
                        element.html('<input data-value="' + groupName + '" data-action="update-group-name" data-id="' + id + '" placeholder="请输入新组名,回车保存" class="fastim-group-input im-on-focus group-id-' + id + '" />')
                        $('.fastim-group-input.group-id-' + id).val(groupName).select()
                        FastIm.common.editEscape('update-group-name-input', 'add')
                    }],
                    ['session-top', sessionOperation],
                    ['session-info', sessionOperation],
                    ['session-remove', sessionOperation],
                    ['session-block-groupmessage', sessionOperation],
                    ['message-forward', () => {
                        FastIm.showWindow('fastim-pickuser')
                        $('.fastim-pickuser .fastim-popup-title').text('转发消息')
                        $('.fastim-pickuser button[data-action="pickuser-selected"]').data('data', 'message-forward').attr('data-data', 'message-forward')
                        $('.fastim-pickuser button[data-action="pickuser-selected"]').data('id', id).attr('data-id', id)
                    }],
                    ['message-copy', messageOperation],
                    ['message-quote', messageOperation],
                    ['message-collection', messageOperation],
                    ['message-to-do', messageOperation],
                    ['message-delete', messageOperation],
                    ['message-new-window', messageOperation],
                    ['collection-del', () => {
                        FastIm.pop.confirm({
                            point: {top: (event.pageY - 140), left: (event.pageX - 20)},
                            msg: '确认要删除此收藏吗？',
                            success: function () {
                                let message = {
                                    c: 'User',
                                    a: 'collection',
                                    data: {
                                        id: id,
                                        action: 'del'
                                    }
                                }
                                FastIm.socket.send(message);
                                FastIm.hideWindow('fastim-confirm')
                            }
                        });
                    }],
                    ['collection-forward', () => {
                        FastIm.showWindow('fastim-pickuser')
                        $('.fastim-pickuser .fastim-popup-title').text('转发收藏内容')
                        $('.fastim-pickuser button[data-action="pickuser-selected"]').data('data', 'collection-forward').attr('data-data', 'collection-forward')
                        $('.fastim-pickuser button[data-action="pickuser-selected"]').data('id', id).attr('data-id', id)
                    }],
                    ['del-group-member', () => {
                        FastIm.socket.send({
                            c: 'Message',
                            a: 'delGroupMember',
                            data: {
                                'id': FastIm.groupChatInfo.id,
                                'member_id': id
                            }
                        })
                    }]
                ]);

                let actionFun = buildFun.get(action) || buildFun.get('default')
                return actionFun.call(this);
            });

            // 点击模态窗口空白区域
            $(document).on('click', '.fastim-modal', function (e) {
                if ($(e.target).hasClass('fastim-modal')) {
                    FastIm.hideWindow('fastim')
                }
            });

            // 点击事件
            $(document).on('click', '.fastim-click,.fastim-close', function (event) {
                var e = $(event.currentTarget),
                    action = e.data('action'),
                    data = e.data('data')

                let funMap = new Map([
                    ['show-fastim', () => {
                        if (!FastIm.clickImButton) {
                            FastIm.showWindow('fastim')
                            // 申请通知权限
                            FastIm.common.applyNotificationAuth()
                        }
                    }],
                    ['show-window', () => {
                        FastIm.showWindow(data)
                    }],
                    ['close-window', () => {
                        FastIm.hideWindow(data)
                    }],
                    ['toggle-el-to-class', () => {
                        $('.' + data).fadeToggle();
                        $('.fastim-contextmenu').remove()
                    }],
                    ['toggle-el-to-function', () => {
                        let elClass = e.data('class')
                        $('.' + elClass).css('display') == 'block' ? FastIm.hideWindow(data) : FastIm.showWindow(data)
                    }],
                    ['close-fastim-pop-msg', () => {
                        FastIm.pop.hideMsg()
                    }],
                    ['play-voice', () => {
                        let voiceEl = $('#voice_' + data)[0]
                        if (voiceEl.canPlayType('audio/wav') == '') {
                            FastIm.pop.msg('您的浏览器不支持播放音频！', 'error');
                            return false;
                        }
                        let sender = e.data('sender')

                        if (FastIm.playVoice && FastIm.playVoice != parseInt(data)) {
                            $('#voice_' + FastIm.playVoice)[0].pause()
                        }

                        if (voiceEl.paused) {
                            FastIm.playVoice = data
                            voiceEl.currentTime = 0;
                            voiceEl.play();
                            $('#img_play_' + data).attr('src', FastIm.common.imgUrl('/assets/addons/fastim/img/' + sender + '-voice.gif'));

                            // 标记语音消息已播放
                            if (e.find('.unread-voice').length) {
                                e.find('.unread-voice').remove()
                                FastIm.socket.send({c: 'User', a: 'changeVoiceStatus', data: {'id': data}});
                            }

                            let callback = function (e) {
                                let msgID = $(e.target).data('id'), playImg = $('#img_play_' + msgID);
                                playImg.attr('src', FastIm.common.imgUrl('/assets/addons/fastim/img/' + playImg.parent().data('sender') + '-voice.png'));
                            }
                            voiceEl.addEventListener('pause', callback)
                            voiceEl.addEventListener('ended', callback)
                        } else {
                            voiceEl.pause()
                        }
                    }],
                    ['change-status', () => {
                        $('.fastim-user-status-list').fadeOut(100);
                        if (FastIm.userinfo.status == -1 && data != 0) {
                            FastIm.userinfo.status = 0;
                            FastIm.ws.needReconnect = true;
                            FastIm.pop.msg(FastIm.config.im_name + ' 正在重新连接网络...', 'tips', false)
                            FastIm.socket.connect();
                            return;
                        }

                        let message = {c: 'ImBase', a: 'changeStatus', data: {'status': data}}
                        FastIm.socket.send(message);
                    }],
                    ['change-imtab', () => {
                        FastIm.hideWindow('all-minor')
                        FastIm.hideWindow('all-popup')
                        if (e.hasClass('active')) {
                            return;
                        } else {
                            var active = $('.fastim-sidebar-tab .active');
                            active.removeClass('active').attr('src', FastIm.common.imgUrl('/assets/addons/fastim/icon/' + active.data('data') + '.png'));
                            e.addClass('active').attr('src', FastIm.common.imgUrl('/assets/addons/fastim/icon/' + data + '-select.png'));

                            // 显示对应窗口
                            FastIm.showWindow(data);
                        }
                    }],
                    ['sidebar-tab-menu', () => {
                        let menuFun = new Map([
                            ['exit', () => {
                                $('.fastim-ins').fadeOut(100)
                                let message = {c: 'ImBase', a: 'changeStatus', data: {'status': 0}}
                                FastIm.socket.send(message);
                            }],
                            ['full-screen', () => {
                                window.open(FastIm.build.url('full-screen'), '_blank');
                            }],
                            ['config', () => {
                                let message = {c: 'ImBase', a: 'config'}
                                FastIm.socket.send(message);
                                FastIm.showWindow('fastim-config')
                            }],
                            ['help-document', () => {
                                window.open("https://doc.fastadmin.net/fastim", '_blank');
                            }],
                            ['feedback', () => {
                                FastIm.showWindow('fastim-report')
                                $('.fastim-report .fastim-popup-title').text('问题反馈')
                                $('.fastim-report #fastim-report-form-type').val('feedback')
                                $('.fastim-report label.describe').text('反馈详情')
                                $('.fastim-report input[name="describe"]').attr('placeholder', '请输入反馈内容')
                            }]
                        ]);
                        menu = menuFun.get(data) || menuFun.get('default')
                        return menu.call(this);
                    }],
                    ['save-to-do', () => {
                        var addTo_doEl = $('.fastim-to-do .add-to-do-textarea')
                        var id = addTo_doEl.data('id')
                        var val = addTo_doEl.val()
                        if (!val) {
                            FastIm.pop.msg('请输入待办内容~')
                            return;
                        }

                        let message = {
                            c: 'ImBase',
                            a: 'loadTODO',
                            data: {
                                'method': 'save',
                                'id': id,
                                'add_val': val
                            }
                        }
                        FastIm.socket.send(message);
                        FastIm.hideWindow('add-to-do')
                        addTo_doEl.data('id', '0').val('')
                    }],
                    ['to-do-more', () => {
                        var completed = $('.fastim-to-do .to-do-more').data('completed')
                        if ($('.fastim-to-do .to-do-completed').css('display') == 'block') {
                            $('.fastim-to-do .to-do-completed').fadeOut(100)
                            $('.fastim-to-do .to-do-more').html('显示已完成待办 • ' + completed)
                        } else {
                            $('.fastim-to-do .to-do-completed').fadeIn(100)
                            $('.fastim-to-do .to-do-more').html('隐藏已完成待办 • ' + completed)
                        }
                    }],
                    ['load-more', () => {
                        type = e.data('type');
                        var click_action = false; // 打开会话时会话的标识
                        if (type == 'search-result') {
                            switch (data) {
                                case 'friends':
                                    click_action = 'user';
                                    break;
                                case 'group_chat':
                                    click_action = 'group';
                                    break;
                                default:
                                    click_action = false;
                                    break;
                            }
                            let newElClass = 'fastim-' + data + '-more';
                            $('.fastim-search-result .fastim-load-more[data-data="' + data + '"]').replaceWith('<div class="' + newElClass + '"></div>');
                            var newEl = $('.' + newElClass)
                            for (var i = 5; i < FastIm.searchResult[data].length; i++) {
                                click_action = click_action ? click_action : (FastIm.searchResult[data][i].type == 'single' ? 'user' : FastIm.searchResult[data][i].type);
                                newEl.append(FastIm.build.widget('search-all-item', FastIm.searchResult[data][i], click_action))
                            }
                            $('.fastim-window input[name="search-session"]').focus()
                        }
                    }],
                    ['to-do-checkbox', () => {
                        // 完成待办
                        var complete = e.prop('checked')
                        let message = {
                            c: 'ImBase',
                            a: 'loadTODO',
                            data: {
                                'method': 'update',
                                'status': complete,
                                'id': data
                            }
                        }
                        FastIm.socket.send(message);
                    }],
                    ['edit-to-do', () => {
                        FastIm.showWindow('add-to-do')
                        $('.fastim-to-do .add-to-do-textarea').data('id', data).val(e.html())
                    }],
                    ['save-config', () => {
                        var config = $("form[name='fastim-config-form']").serializeArray()
                        var values = {};

                        var checkboxNames = [];
                        var checkbox = $('form[name=fastim-config-form] input[type=checkbox]');
                        $.each(checkbox, function(){
                            if($.inArray(this.name, checkboxNames) == -1){
                                checkboxNames.push(this.name);
                                values[this.name] = ''
                            }
                        });

                        $.each(config, function () {
                            if($.inArray(this.name, checkboxNames) >= 0){
                                values[this.name] += this.value + ','
                            } else {
                                values[this.name] = this.value || '';//构造 {name : value}
                            }
                        });
                        let message = {c: 'ImBase', a: 'updateUserConfig', data: values}
                        FastIm.socket.send(message);
                    }],
                    ['unblock', () => {
                        var sessionId = $('select[name="fastim-block-contacts"]').val()
                        if (sessionId) {
                            let message = {
                                c: 'ImBase',
                                a: 'shieldSession',
                                data: {
                                    session_id: sessionId,
                                    method: 'relieve'
                                }
                            }
                            FastIm.socket.send(message);
                        } else {
                            FastIm.pop.msg('请选择一个想要解除屏蔽的好友~');
                        }
                    }],
                    ['submit-fast-reply', () => {
                        var fast_reply = $("form[name='fast_reply']").serializeArray()
                        var values = {};
                        $.each(fast_reply, function (i, field) {
                            values[field.name] = field.value;
                        });

                        if (!values.title) {
                            FastIm.pop.msg('请输入标题~');
                            return;
                        }
                        if (!values.content) {
                            FastIm.pop.msg('请输入回复内容~');
                            return;
                        }

                        let message = {
                            c: 'ImBase',
                            a: 'fastReply',
                            data: {
                                method: 'update',
                                page: 1,
                                data: values
                            }
                        }
                        FastIm.socket.send(message);
                        FastIm.hideWindow('edit-fast-reply')
                    }],
                    ['edit-reply-item', () => {
                        let message = {
                            c: 'ImBase',
                            a: 'fastReply',
                            data: {
                                method: 'edit',
                                id: data
                            }
                        }
                        FastIm.socket.send(message);
                    }],
                    ['del-reply-item', () => {
                        FastIm.pop.confirm({
                            point: {top: (event.pageY - 10), left: (event.pageX + 22)},
                            msg: '确认要删除此快捷回复吗？',
                            success: function () {
                                let message = {
                                    c: 'ImBase',
                                    a: 'fastReply',
                                    data: {
                                        method: 'del',
                                        id: data
                                    }
                                }
                                FastIm.socket.send(message);
                                FastIm.hideWindow('fastim-confirm')
                            }
                        });
                    }],
                    ['del-tempimage', () => {
                        for (var i = 0; i < FastIm.tempImages.length; i++) {
                            if (FastIm.tempImages[i] == data) {
                                FastIm.tempImages.splice(i, 1)
                                e.parent().remove()
                                break;
                            }
                        }
                    }],
                    ['im-add-group-notice-popup', () => {
                        $('#group_notice_group_id').val(data)
                        FastIm.showWindow('im-add-group-notice-popup')
                    }],
                    ['save-group-notice', () => {
                        var notice = $(".group-notice form[name='group-notice-form']").serializeArray(),
                            noticeimages = '',values = {};

                        $.each(notice, function (i, field) {
                            values[field.name] = field.value;
                        });

                        if (!values['content']) {
                            FastIm.pop.msg('请输入公告内容~')
                            return;
                        }

                        $(':input', ".group-notice form[name='group-notice-form']").not(':button').not('input[type="checkbox"]').val('');
                        $("#notice-receipt,#notice-top").prop("checked", false);

                        if (FastIm.tempImages) {
                            for (var i = 0; i < FastIm.tempImages.length; i++) {
                                noticeimages += FastIm.tempImages[i] + ','
                            }
                            $('.group-notice .upload-box-image').not('.group-notice .upload-box-image:first').remove()
                        }
                        FastIm.hideWindow('im-add-group-notice-popup')

                        values.images = noticeimages
                        values.method = 'post'

                        if (values.id) {
                            values.type = 'post-edit'
                            FastIm.socket.send({c: 'Message', a: 'groupChatNoticeOpt', data: values});
                        } else {
                            FastIm.socket.send({c: 'Message', a: 'groupChatNotice', data: values});
                        }
                    }],
                    ['opt-group-chat-notice', () => {
                        let type = e.data('type')
                        if (type == 'edit') {
                            FastIm.socket.send({
                                c: 'Message',
                                a: 'groupChatNoticeOpt',
                                data: {
                                    id: data,
                                    type: 'edit'
                                }
                            });
                        } else if (type == 'del') {
                            FastIm.pop.confirm({
                                point: {top: (event.pageY - 10), left: (event.pageX + 22)},
                                msg: '确认要删除这条公告吗？',
                                success: function () {
                                    let message = {
                                        c: 'Message',
                                        a: 'groupChatNoticeOpt',
                                        data: {
                                            'id': data,
                                            'type': type
                                        }
                                    }
                                    FastIm.socket.send(message);
                                    FastIm.hideWindow('fastim-confirm')
                                }
                            });
                        }
                    }],
                    ['group-notice-receipt', () => {
                        let type = e.data('type')
                        let message = {
                            c: 'Message',
                            a: 'receiptGroupChatNotice',
                            data: {
                                'id': data,
                                'session_id': FastIm.sessionId,
                                'type': type
                            }
                        }
                        FastIm.socket.send(message);
                    }],
                    ['save-report', () => {
                        var report = $(".fastim-report form[name='fastim-report-form']").serializeArray(),
                            reportimage = '',values = {};

                        $.each(report, function (i, field) {
                            values[field.name] = field.value;
                        });

                        var title = (values.type == 'feedback') ? '反馈' : '举报';
                        if (!values['describe']) {
                            FastIm.pop.msg('请输入' + title + '详情~')
                            return;
                        }

                        $(':input', ".fastim-report form[name='fastim-report-form']").not(':button').val('');

                        if (FastIm.tempImages) {
                            for (var i = 0; i < FastIm.tempImages.length; i++) {
                                reportimage += FastIm.tempImages[i] + ','
                            }
                            $('.fastim-report-form .upload-box-image').not('.fastim-report-form .upload-box-image:first').remove()
                        }
                        FastIm.hideWindow('fastim-report')
                        values.reportimage = reportimage
                        values.session_id = FastIm.sessionId
                        values.method = 'post'

                        let message = {c: 'ImBase', a: 'report', data: values}
                        FastIm.socket.send(message);
                    }],
                    ['search-right-menu', () => {
                        if (data == 'add') {
                            // 显示创建新联系窗口
                            FastIm.showWindow('create-new-contact')
                        } else if (data == 'create') {
                            FastIm.showWindow('fastim-pickuser')
                            $('.fastim-pickuser .fastim-popup-title').text('选择朋友以创建群聊')
                            $('.fastim-pickuser button[data-action="pickuser-selected"]').data('data', 'create-group').attr('data-data', 'create-group')
                        }
                    }],
                    ['search-new-contact-botton', () => {
                        FastIm.searchNewContact(data)
                    }],
                    ['open-pick-group', () => {
                        // 好友选择器-加载分组内的好友
                        $('.fastim-pickuser .pick-middle .pick-widget .pick-item').hide()
                        if (e.find('.pick-group-status').hasClass('rotate-90')) {
                            e.find('.pick-group-status').removeClass('rotate-90')
                        } else {
                            $('.fastim-pickuser .pick-middle .pick-widget .pick-group .pick-group-status').removeClass('rotate-90');
                            var widget = $('.fastim-pickuser .pick-middle .pick-widget div[data-data="' + data + '"]')
                            widget.find('.pick-group-status').addClass('rotate-90')
                            if (FastIm.pickUserGroup.includes(data.toString())) {
                                $('.fastim-pickuser .pick-middle .pick-widget .group-' + data).show()
                                $('.fastim-pickuser .group-' + data + ' .pick-checkbox:checkbox:checked').prop('checked', false);
                                for (var item of FastIm.pickChecked) {
                                    $('#pick-' + data + '-item-' + item).prop('checked', true)
                                }
                            } else {
                                let message = {
                                    c: 'User',
                                    a: 'loadGroup',
                                    data: {
                                        id: data,
                                        page: 1,
                                        method: 'pick-group-user'
                                    }
                                }
                                FastIm.socket.send(message);
                            }
                        }
                    }],
                    ['change-session', () => {
                        if (e.hasClass('active')) {
                            return false;
                        }
                        FastIm.sessionId = data;
                        $('.fastim-session-li.active').removeClass('active')
                        e.addClass('active')

                        $('.message-window .chat-window-title .chat-nickname').text('加载中...')

                        let message = {
                            c: 'User',
                            a: 'loadSession',
                            data: {
                                'session_id': data,
                                'page': 1
                            }
                        }
                        FastIm.socket.send(message);
                        FastIm.common.unreadMessagesDisplay(data, 0)

                        FastIm.hideWindow('all-minor')
                        FastIm.hideWindow('all-popup')
                    }],
                    ['send-message', () => {
                        if (e[0].id == 'fastim-message') {
                            // 在已聚焦的情况下改变了光标的位置-记录光标位置
                            FastIm.messageRange = window.getSelection().getRangeAt(0)
                            FastIm.hideWindow('all-minor');
                            return false;
                        }
                        FastIm.sendMessage($('#fastim-message').html(), 'default');
                        $('.chat-window-input-bottom .im-button').removeClass('fastim-bg-blue')
                        $('#fastim-message').trigger('focus')
                        FastIm.inputStatusNotice = 0;
                    }],
                    ['select-emoji', () => {
                        FastIm.hideWindow('smiley')
                        if (FastIm.fastImMessageContenteditable) {
                            FastIm.pop.msg('管理员已禁止发言!');
                            return false;
                        }

                        let newEl = $(e.clone()).removeClass('fastim-click').removeAttr('data-action');

                        var imgNode = document.createElement('img');
                        imgNode.src = newEl.attr('src');
                        imgNode.title = newEl.attr('title');
                        imgNode.className = newEl.attr('class');
                        imgNode.setAttribute('draggable', 'false');
                        imgNode.setAttribute('data-code', newEl.data('code'));
                        FastIm.common.cursorInsert(imgNode, FastIm.messageRange)
                        FastIm.messageRange = window.getSelection().getRangeAt(0)
                    }],
                    ['send-fast-reply', () => {
                        FastIm.sendMessage(data, 'default');
                        FastIm.hideWindow('fast-reply-select')
                        $('#fastim-message').trigger('focus')
                    }],
                    ['submit-link', () => {
                        var linkForm = $(".fastim-window-right form[name='link-form']").serializeArray()
                        var values = {};
                        $.each(linkForm, function (i, field) {
                            values[field.name] = field.value;
                        });

                        if (values.link_url == 'https://') {
                            FastIm.pop.msg('请输入正确的链接~')
                            $('.fastim-window-right .link-input input[name="link_url"]').select()
                            return;
                        }

                        $(':input', ".fastim-window-right form[name='link-form']").not(':button').val('');
                        FastIm.sendMessage(values, 'link')
                        FastIm.hideWindow('link-input')
                        $('#fastim-message').trigger('focus')
                    }],
                    ['pickuser-selected', () => {
                        // 用户选择完毕
                        if (!FastIm.pickChecked.size) {
                            FastIm.pop.msg('请选择好友~')
                            return;
                        }

                        var userstr = ''
                        for (var item of FastIm.pickChecked) {
                            userstr += item + ','
                        }

                        if (data == 'create-group') {
                            // 创建群聊
                            FastIm.showWindow('fastim-userinfo')
                            var newHTML = FastIm.build.groupInfo({
                                nickname: '',
                                bio: '',
                                add_mode: 0,
                                invite_join_group: 0,
                                speak: 0,
                                history_message: 0,
                                retrieval_settings: 1,
                                max_user_count: 200
                            }, 'create');
                            $('.fastim-userinfo').append(newHTML)
                            $('.fastim-userinfo .fastim-popup-title').text('创建群聊')
                        } else if (data == 'invitation-groupmember') {
                            // 为群聊邀请好友
                            FastIm.socket.send({
                                c: 'Message',
                                a: 'invitationGroupmember',
                                data: {
                                    pickuser: userstr,
                                    id: e.data('group')
                                }
                            });
                            FastIm.hideWindow('fastim-pickuser')
                        } else if (data == 'collection-forward') {
                            FastIm.socket.send({
                                c: 'Message',
                                a: 'forward',
                                data: {
                                    pickuser: userstr,
                                    message_ids: e.data('id'),
                                    type: 'collection'
                                }
                            });
                            FastIm.hideWindow('fastim-pickuser')
                        } else if (data == 'message-forward') {
                            FastIm.socket.send({
                                c: 'Message',
                                a: 'forward',
                                data: {
                                    pickuser: userstr,
                                    message_ids: e.data('id'),
                                    type: 'message'
                                }
                            })
                        }
                    }],
                    ['save-groupinfo', () => {
                        var groupinfo = $('.fastim-userinfo form[name="groupinfo"]').serializeArray()
                        var values = {};
                        $.each(groupinfo, function (i, field) {
                            values[field.name] = field.value;
                        });

                        if (!values['nickname']) {
                            FastIm.pop.msg('请输入正确的群名称~')
                            return;
                        }

                        if (values.method == 'create') {
                            if (FastIm.pickChecked.size == 0) {
                                FastIm.pop.msg('请选择群成员~')
                                return;
                            }
                            var userstr = ''
                            for (var item of FastIm.pickChecked) {
                                userstr += item + ','
                            }
                            let message = {
                                c: 'Message',
                                a: 'createGroup',
                                data: {
                                    pickuser: userstr,
                                    data: values
                                }
                            }
                            FastIm.socket.send(message);
                        } else {
                            values.method = 'post-group-edit'
                            values.type = 'group'

                            let message = {c: 'User', a: 'infoDetail', data: values}
                            FastIm.socket.send(message);
                            FastIm.hideWindow('fastim-userinfo')
                        }
                    }],
                    ['get-info', () => {
                        var sessionUser = e.data('session-user'),
                            type = e.data('type')
                        if (type == 'none') {
                            return;
                        }

                        // 点击会话窗口的头像
                        if (sessionUser) {
                            data = {id: sessionUser}
                        } else {
                            data = {id: FastIm.userinfo.id}
                        }

                        if (!type || type == 'tourist' || type == 'csr') {
                            type = 'user'
                        }
                        data.type = type
                        data.method = 'get'
                        let message = {c: 'User', a: 'infoDetail', data: data}
                        FastIm.socket.send(message);
                    }],
                    ['edit_remark', () => {
                        id = e.data('id')
                        $('.fastim-userinfo-box .imedit_userinfo_remark').replaceWith('<input data-id="' + id + '" data-remark="' + data + '" class="imedit_userinfo_remark_input im-on-focus" data-action="save-userinfo-remark" placeholder="请输入好友备注" />')
                        $('.fastim-userinfo-box .imedit_userinfo_remark_input').val(data).select()
                        FastIm.common.editEscape('save-userinfo-remark', 'add')
                    }],
                    ['edit-groupmember-nickname', () => {
                        value = e.data('value')
                        e.replaceWith('<input data-data="' + data + '" class="save-groupmember-nickname-input im-on-focus" data-action="save-groupmember-nickname" placeholder="请输入群昵称" data-value="' + value + '" value="' + value + '" />')
                        $('.fastim-userinfo .groupinfo-member-panel .groupchat-member-nickname input').select()
                        FastIm.common.editEscape('edit-groupmember-nickname', 'add')
                    }],
                    ['shield', () => {
                        if (e.hasClass('fastim-color-red')) {
                            // 解除屏蔽
                            let message = {
                                c: 'User',
                                a: 'shieldSession',
                                data: {
                                    session_id: FastIm.sessionId,
                                    method: 'relieve'
                                }
                            }
                            FastIm.socket.send(message);
                        } else {
                            if (!FastIm.sessionId) {
                                FastIm.pop.msg('请选择一个会话~');
                                return;
                            }
                            FastIm.pop.confirm({
                                point: {top: (event.pageY + 5), left: (event.pageX - 20)},
                                msg: '确认要屏蔽此会话吗？（可在设置中解除）',
                                success: function () {
                                    let message = {
                                        c: 'User',
                                        a: 'shieldSession',
                                        data: {
                                            session_id: FastIm.sessionId,
                                            method: 'shield'
                                        }
                                    }
                                    FastIm.socket.send(message);
                                    FastIm.hideWindow('fastim-confirm')
                                }
                            });
                        }
                    }],
                    ['report', () => {
                        if (!FastIm.sessionId) {
                            FastIm.pop.msg('请选择一个会话~');
                            return;
                        }
                        let message = {
                            c: 'ImBase',
                            a: 'report',
                            data: {
                                session_id: FastIm.sessionId,
                                type: 'user',
                                method: 'get'
                            }
                        }
                        FastIm.socket.send(message)
                    }],
                    ['open-session', () => {
                        let message = {c: 'Message', a: 'openSession', data: {'id': data, 'type': e.data('type')}}
                        FastIm.socket.send(message);
                        $('.fastim-window-search input[name="search-session"]').val('')
                        FastIm.hideWindow('all-minor')
                        FastIm.hideWindow('all-popup')
                    }],
                    ['add-friends', () => {
                        let message = {
                            c: 'User',
                            a: 'createNewContact',
                            data: {
                                'id': data,
                                'type': 'friends',
                                'method': 'get',
                                'action': 'getinfo'
                            }
                        }
                        FastIm.socket.send(message);
                    }],
                    ['submit-new-contact-form', () => {
                        // 同意申请、发送申请、加入群聊
                        var newContactApply = $(".fastim-popup.new-contact-apply-box form[name='new-contact-apply']").serializeArray()
                        $(':input', ".fastim-popup.new-contact-apply-box form[name='new-contact-apply']").not(':button').val('').removeAttr('selected');

                        var values = {};
                        $.each(newContactApply, function (i, field) {
                            values[field.name] = field.value;
                        });

                        if (values['action'] == 'agree-friends') {
                            var message = {
                                c: 'User',
                                a: 'newFriendsOption',
                                data: {
                                    'id': values.id,
                                    'action': 'agree',
                                    'data': values
                                }
                            }
                            FastIm.socket.send(message);
                        } else if (values['action'] == 'apply') {
                            values.method = 'post'
                            var message = {c: 'User', a: 'createNewContact', data: values}
                            FastIm.socket.send(message);
                        }
                        FastIm.hideWindow('new-contact-apply')
                    }],
                    ['service-message-option', () => {
                        var id = e.data('id');
                        if (data == 'agree') {
                            var message = {
                                c: 'User',
                                a: 'createNewContact',
                                data: {
                                    'id': e.data('sender'),
                                    'message_id': id,
                                    'type': 'friends',
                                    'action': 'pre-agree',
                                    'method': 'get'
                                }
                            }
                            FastIm.socket.send(message);
                        } else {
                            let message = {
                                c: 'User',
                                a: 'newFriendsOption',
                                data: {
                                    'id': id,
                                    'action': data
                                }
                            }
                            FastIm.socket.send(message);
                        }
                    }],
                    ['hide-userinfo-window', () => {
                        FastIm.hideWindow('fastim-userinfo')
                        FastIm.hideWindow('new-contact-apply')
                    }],
                    ['userinfo-opt', () => {
                        type = e.data('type'), opt = e.data('opt')
                        if (opt == 'edit') {
                            // 编辑
                            let message = {
                                c: 'User',
                                a: 'infoDetail',
                                data: {
                                    id: data,
                                    type: type,
                                    method: type + '-edit'
                                }
                            }
                            FastIm.socket.send(message);
                        } else if (opt == 'join') {
                            // 加群
                            let message = {
                                c: 'User',
                                a: 'createNewContact',
                                data: {
                                    'id': data,
                                    'type': 'group',
                                    'method': 'get',
                                    'action': 'getinfo'
                                }
                            }
                            FastIm.socket.send(message);
                        } else {
                            // 分享
                            console.log('分享')
                        }
                    }],
                    ['save-userinfo', () => {
                        var userinfo = $('.fastim-userinfo-detail form[name="userinfo"]').serializeArray()
                        var values = {};
                        $.each(userinfo, function (i, field) {
                            values[field.name] = field.value;
                        });

                        if (!values['nickname']) {
                            FastIm.pop.msg('请输入正确的昵称~')
                            return;
                        }

                        values.method = 'post-user-edit'
                        values.type = 'user'
                        values.id = FastIm.userinfo.id

                        var dDate = new Date();
                        var dCurYear = dDate.getFullYear();

                        values.birthday = values.year + '-' + values.month + '-' + values.day
                        values.age = dCurYear - values.year;
                        delete (values.year);
                        delete (values.month);
                        delete (values.day);

                        let message = {c: 'User', a: 'infoDetail', data: values}
                        FastIm.socket.send(message);
                        FastIm.hideWindow('fastim-userinfo')
                    }],
                    ['join-group-option', () => {
                        var id = e.data('id');
                        let message = {
                            c: 'User',
                            a: 'joinGroupOption',
                            data: {
                                'id': id,
                                'action': data
                            }
                        }
                        FastIm.socket.send(message);
                    }],
                    ['del-contact', () => {
                        type = e.data('type')

                        switch (type) {
                            case 'user':
                                msg = '你确定要删除此好友吗？';
                                break;
                            case 'group':
                                msg = '你确定要退出群聊吗？';
                                break;
                            case 'dissolution-group':
                                msg = '你确定要解散群聊吗？';
                                type = 'group';
                                break;
                        }

                        FastIm.pop.confirm({
                            point: {top: (event.pageY - 10), left: (event.pageX + 22)},
                            msg: msg,
                            success: function () {
                                let message = {
                                    c: 'User',
                                    a: 'delContact',
                                    data: {
                                        'id': data,
                                        'type': type
                                    }
                                }
                                FastIm.socket.send(message);
                                FastIm.hideWindow('fastim-confirm')
                            }
                        });
                    }],
                    ['invitation-groupmember', () => {
                        groupname = e.data('groupname')
                        FastIm.showWindow('fastim-pickuser')
                        $('.fastim-pickuser .fastim-popup-title').text('邀请朋友加入群聊:' + groupname)
                        $('.fastim-pickuser button[data-action="pickuser-selected"]').data('data', 'invitation-groupmember').attr('data-data', 'invitation-groupmember').data('group', data)
                    }],
                    ['invitation-joingroup-option', () => {
                        var id = e.data('id');
                        let message = {
                            c: 'Message',
                            a: 'invitationjoinGroupOption',
                            data: {
                                'id': id,
                                'action': data
                            }
                        }
                        FastIm.socket.send(message);
                    }],
                    ['load-group-data', () => {
                        var id = e.data('id');
                        if (id == 'new-group') {
                            return;
                        }

                        if (e.hasClass('address-group-box')) {
                            // 好友和群聊分组
                            $('.fastim-address-list .address-group-box').removeClass('active')
                            e.addClass('active')
                            let message = {
                                c: 'User',
                                a: 'loadGroup',
                                data: {
                                    id: id,
                                    page: 1,
                                    method: 'group-user'
                                }
                            }
                            FastIm.socket.send(message);
                        } else if (e.hasClass('collection-group-box')) {
                            // 收藏分组
                            $('.fastim-groups .collection-group-box').removeClass('active')
                            e.addClass('active')
                            let message = {
                                c: 'User',
                                a: 'loadGroup',
                                data: {
                                    id: id,
                                    page: 1,
                                    method: 'load-collection'
                                }
                            }
                            FastIm.socket.send(message);
                        }

                        FastIm.hideWindow('all-minor')
                        FastIm.hideWindow('all-popup')
                    }],
                    ['kbs-click', () => {
                        let message = {
                            c: 'Message',
                            a: 'loadKbs',
                            data: {
                                id: data
                            }
                        }
                        FastIm.socket.send(message);
                    }],
                    ['distribution-group-select', () => {
                        let message = {
                            c: 'Message',
                            a: 'distributionCsr',
                            data: {
                                group_id: data
                            }
                        }
                        FastIm.socket.send(message);
                    }],
                    ['manual-csr', () => {
                        let message = {
                            c: 'Message',
                            a: 'distributionCsr',
                            data: {}
                        }
                        FastIm.socket.send(message);
                    }],
                    ['fastim-open-link', () => {
                        $('.fastim-link-url').html(data)
                        $('.fastim-link-url-a').attr('href', data)
                        FastIm.showWindow('fastim-link-popup')
                    }],
                    ['show-group-chat-notice', () => {
                        FastIm.socket.send({
                            c: 'Message',
                            a: 'groupChatNoticeOpt',
                            data: {
                                id: data,
                                type: 'get'
                            }
                        });
                    }],
                    ['show-group-notice-users', () => {
                        FastIm.showWindow('im-user-list-popup')
                        FastIm.socket.send({
                            c: 'Message',
                            a: 'groupChatNoticeOpt',
                            data: {
                                id: data,
                                type: 'get-users',
                                method: e.data('type'),
                                page: 1
                            }
                        });
                    }],
                    ['confirm-at', () => {
                        FastIm.common.insertAtUser({
                            id: data,
                            nickname: e.find('.fastim-at-user-nickname').text()
                        })
                    }],
                    ['default', () => {
                    }]
                ]);

                FastIm.common.stopPropagation(event);
                let actionFun = funMap.get(action) || funMap.get('default')
                return actionFun.call(this);
            });

            // 滚动事件
            document.addEventListener('scroll', function (event) {
                var e = $(event.target);
                var scrollTop = e.scrollTop();
                FastIm.hideWindow('light-prompt');
                if (e.hasClass('to-do-list') && FastIm.loadTODO) {
                    var scrollHeight = $('.to-do-list')[0].scrollHeight;
                    if ((scrollHeight - scrollTop) < 660) {
                        FastIm.loadTODO.page++;
                        let message = {c: 'ImBase', a: 'loadTODO', data: FastIm.loadTODO}
                        FastIm.socket.send(message);
                        FastIm.loadTODO = false
                    }
                } else if (e.hasClass('fast-reply-tbody') && FastIm.loadFastReply) {
                    var scrollHeight = $('.fast-reply-tbody')[0].scrollHeight;
                    if ((scrollHeight - scrollTop) < 340) {
                        FastIm.loadFastReply.page++;
                        let message = {c: 'ImBase', a: 'fastReply', data: FastIm.loadFastReply}
                        FastIm.socket.send(message);
                        FastIm.loadFastReply = false
                    }
                } else if (e.hasClass('chat-records') && (scrollTop == 0) && FastIm.loadRecordsData && parseInt(FastIm.loadRecordsData.session_id) == FastIm.sessionId) {
                    FastIm.recordScrollHeight = $('.chat-records')[0].scrollHeight;
                    FastIm.loadRecordsData.page++;
                    let message = {c: 'User', a: 'loadSession', data: FastIm.loadRecordsData}
                    FastIm.socket.send(message);
                    FastIm.loadRecordsData = false
                } else if (e.hasClass('service-message') && FastIm.serviceMessageData && parseInt(FastIm.serviceMessageData.session_id) == FastIm.sessionId) {
                    var scrollHeight = $('.service-message')[0].scrollHeight;
                    if ((scrollHeight - scrollTop) < 660) {
                        FastIm.serviceMessageData.page++;
                        let message = {c: 'User', a: 'loadSession', data: FastIm.serviceMessageData}
                        FastIm.socket.send(message);
                        FastIm.serviceMessageData = false
                    }
                } else if ((e.hasClass('contacts-list') || e.hasClass('collection-data-tbody')) && FastIm.loadGroup) {
                    var scrollHeight = e[0].scrollHeight;
                    if ((scrollHeight - scrollTop) < 660) {
                        FastIm.loadGroup.page++;
                        let message = {c: 'User', a: 'loadGroup', data: FastIm.loadGroup}
                        FastIm.socket.send(message);
                        FastIm.loadGroup = false
                    }
                } else if ((e[0].id == 'new-contact-friends-list' || e[0].id == 'new-contact-group-list') && FastIm.newContactData) {
                    var scrollHeight = $('#' + e[0].id)[0].scrollHeight;
                    if ((scrollHeight - scrollTop) < 360) {
                        FastIm.newContactData.page++;
                        let message = {c: 'User', a: 'searchNewContact', data: FastIm.newContactData}
                        FastIm.socket.send(message);
                        FastIm.newContactData = false
                    }
                } else if (e.hasClass('groupinfo-notice-list') && FastIm.loadGroupChatNotice) {
                    var scrollHeight = $('.groupinfo-notice-list')[0].scrollHeight;
                    if ((scrollHeight - scrollTop) < 360) {
                        FastIm.loadGroupChatNotice.page++;
                        let message = {c: 'Message', a: 'groupChatNotice', data: FastIm.loadGroupChatNotice}
                        FastIm.socket.send(message);
                        FastIm.loadGroupChatNotice = false
                    }
                } else if (e.hasClass('im-user-list-items') && FastIm.loadGroupNoticeUsers) {
                    var scrollHeight = $('.im-user-list-items')[0].scrollHeight;
                    if ((scrollHeight - scrollTop) < 360) {
                        FastIm.loadGroupNoticeUsers.page++;
                        let message = {c: 'Message', a: 'groupChatNoticeOpt', data: FastIm.loadGroupNoticeUsers}
                        FastIm.socket.send(message);
                        FastIm.loadGroupNoticeUsers = false
                    }
                }
            }, true);

            // 按键监听
            $(document).on('keydown', function (event) {
                var e = $(event.target),
                    action = e.data('action'),
                    key = '';

                switch (event.keyCode) {
                    case 13:
                        key = 'enter';
                        break;
                    case 40:
                        key = 'down';
                        break;
                    case 38:
                        key = 'up';
                        break;
                }

                action = (event.ctrlKey ? 'ctrl-' : '') + (key + '-' + action);

                let actionFun = new Map([
                    ['up-at-users-search', () => {
                        var atUserLength = $('.fastim-at-users .fastim-at-user-item').length
                        if (!FastIm.atUserSelect || FastIm.atUserSelect == 1) {
                            FastIm.atUserSelect = atUserLength
                        } else {
                            FastIm.atUserSelect--;
                        }

                        $('.fastim-at-users .fastim-at-user-item.active').removeClass('active')
                        var itemEl = $('.fastim-at-users .fastim-at-user-item').eq(FastIm.atUserSelect - 1)
                        itemEl.addClass('active')
                        var resultEl = $('.fastim-at-users-box')
                        resultEl.animate({
                            scrollTop: itemEl.offset().top - resultEl.offset().top + resultEl.scrollTop()
                        }, 100);
                        event.preventDefault();
                        return false;
                    }],
                    ['down-at-users-search', () => {
                        var atUserLength = $('.fastim-at-users .fastim-at-user-item').length
                        if (!FastIm.atUserSelect) {
                            FastIm.atUserSelect = 1;
                        } else {
                            FastIm.atUserSelect = (FastIm.atUserSelect + 1) > atUserLength ? 1 : FastIm.atUserSelect + 1;
                        }

                        $('.fastim-at-users .fastim-at-user-item.active').removeClass('active')
                        var itemEl = $('.fastim-at-users .fastim-at-user-item').eq(FastIm.atUserSelect - 1)
                        itemEl.addClass('active')
                        var resultEl = $('.fastim-at-users-box')
                        resultEl.animate({
                            scrollTop: itemEl.offset().top - resultEl.offset().top + resultEl.scrollTop()
                        }, 100);
                        event.preventDefault();
                        return false;
                    }],
                    ['enter-send-message', () => {
                        if (parseInt(FastIm.config.user_config.send_message_key) == 0) {
                            FastIm.sendMessage($('#fastim-message').html(), 'default');
                            $('.chat-window-input-bottom .im-button').removeClass('fastim-bg-blue')
                            FastIm.inputStatusNotice = 0;
                        }
                    }],
                    ['ctrl-enter-send-message', () => {
                        var messageEl = $('#fastim-message'),
                            message = messageEl.html();
                        if (parseInt(FastIm.config.user_config.send_message_key) == 1) {
                            FastIm.sendMessage(message, 'default');
                            $('.chat-window-input-bottom .im-button').removeClass('fastim-bg-blue')
                            FastIm.inputStatusNotice = 0;
                        } else {
                            // 换行
                            FastIm.common.messageInputLineFeed()
                        }
                    }],
                    ['default', () => {}]
                ]);

                action = actionFun.get(action) || actionFun.get('default')
                return action.call(this);
            });

            $(document).on('keyup', function (event) {
                // console.log('当前按钮的code:', event.keyCode);
                var e = $(event.target),
                    action = e.data('action'),
                    key = '';

                switch (event.keyCode) {
                    case 13:
                        key = 'enter';
                        break;
                    case 40:
                        key = 'down';
                        break;
                    case 38:
                        key = 'up';
                        break;
                    case 37:
                        key = 'left';
                        break;
                    case 39:
                        key = 'right';
                        break;
                    case 123:
                        key = 'f12';
                        action = 'reset-button';
                        break;
                    case 191: // 消息提取快捷键
                        key = 'extract';
                        action = 'message';
                        break;
                }

                if (event.keyCode == 27 && parseInt(FastIm.config.user_config.ecs_exit) == 1) {
                    if (!FastIm.escape.size || !FastIm.windowState.fastimWindow) {
                        return false;
                    }
                    action = 'close-window'
                    key = 'escape';
                }

                if (!action || !key) {
                    return false;
                }
                action = (event.ctrlKey ? 'ctrl-' : '') + (key + '-' + action);

                let actionFun = new Map([
                    ['escape-close-window', () => {
                        let escapeArray = Array.from(FastIm.escape);
                        var escapeEvent = escapeArray[escapeArray.length - 1];
                        if (FastIm.fullScreen && escapeEvent == 'fastim') {
                            return false;
                        }
                        FastIm.hideWindow(escapeEvent)
                    }],
                    ['enter-search-new-friends', () => {
                        FastIm.searchNewContact('friends');
                    }],
                    ['enter-search-new-group', () => {
                        FastIm.searchNewContact('group');
                    }],
                    ['enter-search-user', () => {
                        var val = e.val()
                        if (!val) {
                            $('.fastim-pickuser .pick-left .pick-tab.active').removeClass('active').trigger('click')
                            return;
                        }
                        let message = {
                            c: 'User',
                            a: 'search',
                            data: {
                                'keyword': val,
                                'method': 'friends|lately-session-user',
                                'type': 'pick'
                            }
                        }
                        FastIm.socket.send(message);
                    }],
                    ['enter-search-session', () => {
                        if (!FastIm.searchSessionSelect) {
                            let message = {
                                c: 'User',
                                a: 'search',
                                data: {
                                    'keyword': e.val(),
                                    'method': 'friends|lately-session|group-chat',
                                    'type': 'all'
                                }
                            }
                            FastIm.socket.send(message);
                        } else {
                            var itemEl = $('.fastim-search-result .search-res-item').eq((FastIm.searchSessionSelect - 1)).trigger('click')
                            FastIm.searchSessionSelect = 0;
                        }
                    }],
                    ['enter-save-userinfo-remark', () => {
                        FastIm.common.saveUserRemark();
                    }],
                    ['enter-save-groupmember-nickname', () => {
                        FastIm.common.saveGroupmemberNickname();
                    }],
                    ['up-at-users-search', () => {}],
                    ['down-at-users-search', () => {}],
                    ['down-search-session', () => {
                        var sessionLength = $('.fastim-search-result .search-res-item').length
                        if (!FastIm.searchSessionSelect) {
                            FastIm.searchSessionSelect = 1;
                        } else {
                            FastIm.searchSessionSelect = (FastIm.searchSessionSelect + 1) > sessionLength ? 0 : FastIm.searchSessionSelect + 1;
                        }

                        if (FastIm.searchSessionSelect === 0) {
                            $('.fastim-window input[name="search-session"]').select()
                            return;
                        }

                        $('.fastim-search-result .search-res-item.active').removeClass('active')
                        var itemEl = $('.fastim-search-result .search-res-item').eq((FastIm.searchSessionSelect - 1))
                        itemEl.addClass('active')
                        var resultEl = $('.fastim-search-result')
                        resultEl.animate({
                            scrollTop: itemEl.offset().top - resultEl.offset().top + resultEl.scrollTop()
                        }, 100);
                    }],
                    ['up-search-session', () => {
                        var sessionLength = $('.fastim-search-result .search-res-item').length
                        if (!FastIm.searchSessionSelect) {
                            FastIm.searchSessionSelect = sessionLength
                        } else if (FastIm.searchSessionSelect == 1) {
                            FastIm.searchSessionSelect = 0;
                        } else {
                            FastIm.searchSessionSelect--;
                        }

                        if (FastIm.searchSessionSelect === 0) {
                            $('.fastim-window input[name="search-session"]').select()
                            return;
                        }

                        $('.fastim-search-result .search-res-item.active').removeClass('active')
                        var itemEl = $('.fastim-search-result .search-res-item').eq((FastIm.searchSessionSelect - 1))
                        itemEl.addClass('active')
                        var resultEl = $('.fastim-search-result')
                        resultEl.animate({
                            scrollTop: itemEl.offset().top - resultEl.offset().top + resultEl.scrollTop()
                        }, 100);
                    }],
                    ['enter-at-users-search', () => {
                        let itemEl = $('.fastim-at-users .fastim-at-user-item').eq(FastIm.atUserSelect - 1)
                        FastIm.common.insertAtUser({
                            id: itemEl.data('data'),
                            nickname: itemEl.find('.fastim-at-user-nickname').text()
                        })
                        return false;
                    }],
                    ['left-send-message', () => {
                        FastIm.messageRange = window.getSelection().getRangeAt(0)
                    }],
                    ['right-send-message', () => {
                        FastIm.messageRange = window.getSelection().getRangeAt(0)
                    }],
                    ['enter-update-group-name', () => {
                        FastIm.common.saveUpdateGroupName(e)
                    }],
                    ['enter-add-new-group', () => {
                        e.blur()
                    }],
                    ['f12-reset-button', () => {
                        console.warn('您按下了F12，因窗口大小变化，已恢复悬浮按钮至初始位置')
                        localStorage.removeItem("fastim_button_coordinate");
                        $('.fastim-button-container').css({
                            top: '78%',
                            left: 'unset',
                            right: '60px'
                        });
                        FastIm.hideWindow('light-prompt')
                    }],
                    ['ctrl-extract-message', () => {
                        FastIm.common.extractMessage();
                    }],
                    ['default', () => {}]
                ]);

                action = actionFun.get(action) || actionFun.get('default')
                return action.call(this);
            });

            // 拖动悬浮按钮
            $(document).on('mousedown', '.fastim-button-container,.fastim-preview-img', function (e) {
                var eEl = $(e.currentTarget)
                //解决拖动会选中文字的问题
                document.onselectstart = function () {
                    return false;
                };
                document.ondragstart = function () {
                    return false;
                };

                $(document).find("iframe").css("pointer-events", "none");
                FastIm.originalX = e.pageX - parseInt(eEl.css("left"));
                FastIm.originalY = e.pageY - parseInt(eEl.css("top"));
                FastIm.elMove = true;

                var mouseupFun = () => {
                    document.onselectstart = null;
                    document.ondragstart = null;
                    $(document).find("iframe").css("pointer-events", "auto");
                    FastIm.elMove = false;
                }

                if (eEl.hasClass('fastim-button-container')) {
                    FastIm.clickImButton = false;
                    FastIm.hideWindow('light-prompt')
                    var _w = $(window).width(), _h = $(window).height()

                    // 事件不能挂在元素上，易脱落
                    $(document).on('mousemove', function (e) {
                        if (FastIm.elMove) {
                            var x = e.pageX - FastIm.originalX;
                            var y = e.pageY - FastIm.originalY;

                            if (!FastIm.clickImButton && Math.abs(parseInt(eEl.css("left")) - x) + Math.abs(parseInt(eEl.css("top")) - y) > 2) {
                                FastIm.clickImButton = true;
                            }

                            y = (y < 10) ? 10 : y;
                            x = (x < 10) ? 10 : x;

                            localStorage.setItem("fastim_button_coordinate", [y, x]); // 保存按钮位置
                            eEl.css({"top": y, "left": x, "right": 'unset'});
                        }
                    }).mouseup(mouseupFun);
                } else if (eEl.hasClass('fastim-preview-img')) {
                    $(document).on('mousemove', function (e) {
                        if (FastIm.elMove) {
                            var x = e.pageX - FastIm.originalX;
                            var y = e.pageY - FastIm.originalY;
                            eEl.css({"top": y, "left": x, "right": 'unset'});
                        }
                    }).mouseup(mouseupFun);
                }
            });
        });
    },
    pop: {
        confirm: function ({point, msg, success}) {
            $('.fastim-contextmenu').remove();
            FastIm.hideWindow('fastim-confirm')

            var newHTML = FastIm.build.widget('confirm', {
                msg: msg,
                style: 'left: ' + point.left + 'px; top: ' + point.top + 'px;'
            });
            $('.fastim-ins').append(newHTML)

            // 确认确认框
            $('.fastim-confirm-ok').on('click', function (e) {
                typeof success == 'function' && success()
                success = null
            });

            FastIm.common.editEscape('fastim-confirm', 'add')
        },
        /**
         * 显示提醒消息
         * 按窗口打开状态
         * @param  {str}  msg       提醒内容
         * @param  {str}  type      提醒图标:success,tips,error
         * @param  {Bool} autoClose 是否自动关闭
         * @param  {int}  seconds   自动关闭时间(秒)
         */
        msg: function (msg, type = 'tips', autoClose = true, seconds = 3) {
            FastIm.pop.hideMsg();
            if (FastIm.windowState.fastimWindow) {
                FastIm.pop.inWindowMsg(msg, type, autoClose, seconds);
            } else {
                FastIm.pop.outOfWindow(msg, autoClose, seconds);
            }
        },
        /**
         * 窗口未打开时,显示3秒窗外提示
         * 永久显示窗口内提示
         */
        absolutePrompt: function (msg, type = 'tips', autoClose = true, seconds = 3) {
            FastIm.pop.hideMsg();
            FastIm.pop.inWindowMsg(msg, type, autoClose, seconds);
            if (!FastIm.windowState.fastimWindow) {
                FastIm.pop.outOfWindow(msg);
            }
        },
        /**
         * Im窗口外的提醒消息
         */
        outOfWindow: function (msg, autoClose = true, seconds = 3) {
            let msgWidthEl = $('<div class="fastim-pop-msg im-prompt-content-temp">' + msg + '</div>').css({
                position: 'fixed',
                left: '-10000px',
                top: '0'
            });
            $('body').append(msgWidthEl);

            var html = '<div style="left:calc(50% - ' + (msgWidthEl.width() / 2) + 'px);" class="fastim-pop-msg"><div aria-hidden="true" data-action="close-fastim-pop-msg" class="fastim-pop-msg-close fastim-click">×</div>' + msg + '</div>';
            $("body").append(html);
            $('.im-prompt-content-temp').remove();
            if (autoClose) {
                setTimeout(function () {
                    $(".fastim-pop-msg").remove();
                }, seconds * 1000);
            }
        },
        /**
         * Im窗口内的提醒消息
         */
        inWindowMsg: function (msg, type = 'tips', autoClose = true, seconds = 3) {
            let msgData = {
                msg: msg,
                icon: FastIm.common.imgUrl('/assets/addons/fastim/icon/' + type + '.png')
            };
            let newHtml = FastIm.build.widget('popup-msg', msgData)
            $('.fastim-window-right').prepend(newHtml)
            if (autoClose) {
                setTimeout(function () {
                    $(".fastim-window-msg").remove()
                }, seconds * 1000);
            }
        },
        /**
         * 隐藏提醒消息
         */
        hideMsg: function () {
            $(".fastim-pop-msg").remove();
            $(".fastim-window-msg").remove()
        },
    },
    common: {
        insertAtUser: function (obj) {
            FastIm.hideWindow('fastim-at-users')
            obj.nickname += ' '
            if (!FastIm.atUsers) {
                FastIm.atUsers = []
            }
            FastIm.atUsers.push(obj)

            var atNode = document.createTextNode(obj.nickname)
            FastIm.common.cursorInsert(atNode, FastIm.messageRange)
        },
        atInput: function (type, keywords = '') {
            var sessionElement = $('.fastim-session-li[data-data="' + FastIm.sessionId + '"]')
            if (sessionElement.data('type') != 'group') {
                return false;
            }

            if (type == 'start') {
                FastIm.atInputClassName = 'at-' + new Date().getTime();
                var range = window.getSelection().getRangeAt(0)
                var atNode = document.createElement('span')
                atNode.innerText = ''
                atNode.className = FastIm.atInputClassName
                FastIm.common.cursorInsert(atNode)

                var atSpan = $('.' + FastIm.atInputClassName), atUsersEl = $('.fastim-at-users'), atUsersElBox = $('.fastim-at-users-box'), offset = atSpan.offset()
                atUsersEl.css({left: (offset.left + 10 ) + 'px', top: (offset.top - 60) + 'px'});
                atUsersElBox.empty();
                atUsersElBox.append('<div class="fastim-data-none">加载中...</div>')
                FastIm.showWindow('fastim-at-users')
            }

            // 加载群成员列表
            FastIm.socket.send({
                c: 'Message',
                a: 'groupChatAt',
                data: {
                    'session_id': FastIm.sessionId,
                    'method': 'get-all',
                    'at_class_name': FastIm.atInputClassName,
                    'keywords': keywords
                }
            });
        },
        /**
         * 在光标处插入元素
         */
        cursorInsert: function (node, range = null) {
            var selection = window.getSelection()
            if (!range) {
                range = window.getSelection().getRangeAt(0)
            } else {
                $('#fastim-message').trigger('focus')
            }
            range.insertNode(node)
            range.setEndAfter(node);
            range.setStartAfter(node);
            selection.removeAllRanges();
            selection.addRange(range);
        },
        showNewGroupChatNotice: function (unreadFixedMsg) {
            if (unreadFixedMsg && parseInt(unreadFixedMsg.popup) == 1) {
                let info = unreadFixedMsg.publisher.nickname_origin + (unreadFixedMsg.publisher.remark ? '(' + unreadFixedMsg.publisher.remark + ') ':' ') + '发表于 ' + unreadFixedMsg.createtime;
                $('.fastim-popup.new-group-notice-popup .new-group-notice-content').text(unreadFixedMsg.content)
                $('.fastim-popup.new-group-notice-popup .group-notice-footer-info').text(info);
                var imagesHtml = '';
                for (x in unreadFixedMsg.images) {
                    imagesHtml += '<div class="upload-box-image"><a href="' + unreadFixedMsg.images[x] + '" target="_blank"><img draggable="false" src="' + unreadFixedMsg.images[x] + '" /></a></div>';
                }
                $('.fastim-popup.new-group-notice-popup .im-group-notice-imgs').html(imagesHtml)
                $('.fastim-popup.new-group-notice-popup .group-notice-footer-reading-number').text(unreadFixedMsg.reading_number + '人已读');
                $('.fastim-popup.new-group-notice-popup .notice-receipt-button').data('data', unreadFixedMsg.id).attr('data-data', unreadFixedMsg.id);
                $('.fastim-popup.new-group-notice-popup .group-notice-footer-button').show()
                if (unreadFixedMsg.receipt) {
                    $('.fastim-popup.new-group-notice-popup .group-notice-footer-button.im-button-grey').hide()
                    if (unreadFixedMsg.receipted) {
                        $('.fastim-popup.new-group-notice-popup .group-notice-footer-button.notice-receipt-button').addClass('im-button-grey').text('您已确认').data('action', 'close-window').attr('data-action', 'close-window').data('data', 'new-group-notice-popup').attr('data-data', 'new-group-notice-popup').show()
                    }
                } else {
                    $('.fastim-popup.new-group-notice-popup .group-notice-footer-button.notice-receipt-button').hide()
                }
                FastIm.showWindow('new-group-notice-popup')
            }
        },
        extractMessage: function () {
            // 寻找未读消息
            var sessionList = $('.fastim-session-li .session-unread-count').eq(0), lastSession = null;
            if (sessionList.length) {
                lastSession = sessionList.parent().parent().parent().data('data');
            }

            // 打开窗口
            if (!FastIm.windowState.fastimWindow) {
                FastIm.showWindow('fastim')
            } else if (!lastSession) {
                FastIm.hideWindow('fastim')
            }

            // 提取消息
            if (lastSession) {
                // 切换到message面板
                if (!FastIm.windowState.messageWindow) {
                    $('.fastim-sidebar-tab img[data-data="message"]').trigger('click')
                }
                // 切换到最新消息的会话
                if (FastIm.sessionId != lastSession) {
                    $('.fastim-session-li[data-data="' + lastSession + '"]').trigger('click')
                }
            }
        },
        lightPrompt: function (message) {
            if (!message || FastIm.windowState.fastimWindow) {
                return false;
            }

            // 获取宽度
            let msgWidthEl = $('<div class="fastim-light-prompt-content im-prompt-content-temp">' + message + '</div>').css({
                position: 'fixed',
                left: '-10000px',
                top: '0'
            });
            $('body').append(msgWidthEl);

            var offset = $('.fastim-button-container').offset(), msgWidth = msgWidthEl.width() + 28, left = offset.left,
                contentEl = $('.fastim-light-prompt-content');
            msgWidth = msgWidth > 236 ? 236 : msgWidth;

            $('.im-prompt-content-temp').remove()

            if (left < 300) {
                left = left + 52;
                contentEl.addClass('left')
            } else {
                left = (left - msgWidth - 8);
                contentEl.removeClass('left')
            }

            contentEl.text(message);
            $('.fastim-light-prompt').css({width: msgWidth, top: offset.top + 'px', left: left + 'px'}).fadeIn()
        },
        contextMenu: function (point, option, id) {
            $('.fastim-contextmenu').remove();
            FastIm.hideWindow('fastim-confirm')
            var menu = document.createElement("div");
            menu.setAttribute('class', 'fastim-contextmenu');
            $(menu).css({left: (point.left - 10) + 'px', top: (point.top - 10) + 'px'});
            for (var x in option) {
                var tempNode = document.createElement("div");
                tempNode.setAttribute('data-action', option[x].action);
                tempNode.setAttribute('data-id', id);
                tempNode.setAttribute('class', 'fastim-contextmenu-item');
                tempNode.innerText = option[x].name
                menu.appendChild(tempNode);
            }
            $('.fastim-ins').append(menu);
        },
        /**
         * 日期下拉初始化
         */
        YMDselect: function (ymdClass) {
            var dDate = new Date();
            var dCurYear = dDate.getFullYear();

            var year = $('.' + ymdClass + ' select[name="year"]')
            var month = $('.' + ymdClass + ' select[name="month"]')
            var day = $('.' + ymdClass + ' select[name="day"]')

            for (var i = 0; i < 100; i++) {
                year.append('<option value="' + (dCurYear - i) + '">' + (dCurYear - i) + '年</option>');
            }

            $(document).on('change', year, function () {
                for (var i = 1; i <= 12; i++) {
                    month.append('<option value="' + (i < 10 ? '0' + i : i) + '">' + i + '月</option>');
                }
            })
            $(document).on('change', month, function () {
                var YI = year.val();
                var MI = month.val();
                var dPrevDate = new Date(YI, MI, 0);
                var daysInMonth = dPrevDate.getDate();
                for (d = 1; d <= parseInt(daysInMonth); d++) {
                    day.append('<option value="' + (d < 10 ? '0' + d : d) + '">' + d + '日</option>');
                }
            })

            year.trigger('change')
        },
        /**
         * 播放消息提醒声音
         */
        playSound: function () {
            FastIm.audio.source = FastIm.audio.context.createBufferSource();
            FastIm.audio.source.buffer = FastIm.audio.buffer;
            FastIm.audio.source.loop = false;
            FastIm.audio.source.connect(FastIm.audio.context.destination);
            FastIm.audio.source.start(0); //立即播放
        },
        /**
         * 加载消息提醒声音
         */
        loadAudioFile: function (url) {
            var xhr = new XMLHttpRequest(); //通过XHR下载音频文件
            xhr.open('GET', url, true);
            xhr.responseType = 'arraybuffer';
            xhr.onload = function (e) { //下载完成
                FastIm.audio.context.decodeAudioData(this.response,
                    function (buffer) { //解码成功时的回调函数
                        FastIm.audio.buffer = buffer;
                        FastIm.common.playSound();
                    },
                    function (e) { //解码出错时的回调函数
                        console.log('音频解码失败', e);
                    }
                );
            };
            xhr.send();
        },
        sendNotification: function (title, options) {
            const NotificationInstance = Notification || window.Notification;
            if (!!NotificationInstance) {
                return new NotificationInstance(title, options);
            } else if ("mozNotification" in navigator) {
                return navigator.mozNotification
                .createNotification(title, options.body, options.icon)
                .show();
            } else {
                return false;
            }
        },
        /**
         * 新消息通知
         * @param  {str} title 消息标题
         * @param  {str} title 消息标题
         * @param  {str} body  消息内容
         * @param  {str} icon  用户头像图片
         */
        /**
         * 新消息通知
         * @param  {[type]} title       消息标题
         * @param  {[type]} body        消息内容
         * @param  {[type]} icon        用户头像图片
         * @param  {[type]} sessionType 会话类型
         */
        newMessageNotice: function (title, body, icon, sessionType = 'single', push = true) {
            if (parseInt(FastIm.config.user_config.new_message_shake) == 1) {
                $('.fastim-window').addClass('fastim-shake');
                setTimeout(function () {
                    $('.fastim-window').removeClass('fastim-shake');
                }, 400);
            }

            // 播放铃声
            if (parseInt(FastIm.config.user_config.new_message_sound) == 1) {
                if (FastIm.audio.buffer) {
                    FastIm.common.playSound();
                } else {
                    let url = FastIm.build.url('load_message_prompt')
                    FastIm.common.loadAudioFile(url);
                }
            }

            // 悬浮按钮提示
            FastIm.common.lightPrompt((sessionType == 'group' ? '' : title + ':') + body);

            // 推送通知
            if (parseInt(FastIm.config.user_config.new_message_push_notice) == 1 && FastIm.notificationStatus && push) {

                if (parseInt(FastIm.config.user_config.window_display_push_notice) == 0 && !FastIm.common.windowHidden()) {
                    return false;
                }
                var n = FastIm.common.sendNotification(title ? title : 'FastIm', {
                    body: body ? body : '收到一条新的消息~',
                    icon: icon,
                    tag: 'fastim-notification',
                    renotify: true,
                    silent: true, // 多数浏览器不支持播放声音,直接静音,在下面进行单独的音频播放
                });
                n.onclick = function (e) {
                    if (!FastIm.showFastimWindow) {
                        FastIm.showWindow('fastim')
                    }
                    n.close();
                }
            }
        },
        saveUserRemark: function () {
            if (!FastIm.saveUserRemarkBool) {
                var e = $('.imedit_userinfo_remark_input'),
                    remark = e.data('remark').toString(),
                    new_remark = e.val()

                if (remark != new_remark) {
                    let message = {
                        c: 'User',
                        a: 'updateFriendInfo',
                        data: {
                            id: e.data('id'),
                            new_remark: new_remark,
                            method: 'update_remark'
                        }
                    }
                    FastIm.socket.send(message);
                }
            }
            FastIm.hideWindow('save-userinfo-remark')
            FastIm.saveUserRemarkBool = false
        },
        saveGroupmemberNickname: function () {
            if (!FastIm.saveGroupmemberNicknameBool) {
                var e = $('.save-groupmember-nickname-input'),
                    value = e.data('value').toString(),
                    new_nickname = e.val()
                if (value != new_nickname) {
                    let message = {
                        c: 'Message',
                        a: 'groupChatMember',
                        data: {
                            id: e.data('data'),
                            new_nickname: new_nickname,
                            method: 'save-groupmember-nickname'
                        }
                    }
                    FastIm.socket.send(message);
                }
            }
            FastIm.hideWindow('edit-groupmember-nickname')
            FastIm.saveGroupmemberNicknameBool = false
        },
        saveUpdateGroupName: function (el) {
            if (!FastIm.saveUpdateGroupNameBool) {
                var id = el.data('id'),
                    value = el.data('value'),
                    newname = el.val();

                if (value != newname) {
                    let message = {
                        c: 'ImBase',
                        a: 'updateGroup',
                        data: {
                            id: id,
                            newname: newname,
                            action: 'update-group-name'
                        }
                    }
                    FastIm.socket.send(message);
                }
            }

            FastIm.hideWindow('update-group-name-input')
            FastIm.saveUpdateGroupNameBool = false
        },
        onUploadResponse: function (response) {
            try {
                var ret = typeof response === 'object' ? response : JSON.parse(response);
                if (!ret.hasOwnProperty('code')) {
                    $.extend(ret, {code: -2, msg: response, data: null});
                }
            } catch (e) {
                var ret = {code: -1, msg: e.message, data: null};
            }
            return ret;
        },
        /**
         * 将消息格式化为可直接展示的内容
         * @param  {str} data 消息内容
         */
        formatMessage: function (data) {
            if (data.type == 'image') {
                message = '[图片]';
            } else if (data.type == 'audio') {
                message = '[音频]';
            } else if (data.type == 'video') {
                message = '[视频]';
            } else if (data.type == 'file') {
                message = '[文件]';
            } else if (data.type == 'link') {
                message = '[链接]';
            } else if (data.type == 'kbs_list') {
                message = data.message.title;
            } else {
                message = data.message.toString().replace(/<img.*?title="(.+?)".*?>/g, "[$1]");
                message = message.replace(/<img.*?src="(.+?)".*?>/g, "[图片]");
            }
            return message;
        },
        /**
         * 消息的阅读状态
         * @param  {int}  status     状态
         * @param  {int}  messageId  消息ID
         * @param  {Bool} isUpdate   是否是更新原来的消息
         * @param  {int}  readNumber 消息阅读数
         * @return htmlTemplate
         */
        messageStatus: function (status, messageId, isUpdate = false, readNumber = 0) {
            // 0=送达未读,1=已读,2=发送中,3=隐藏,4=发送失败,5=阅读数,6=全部已读
            var statusClass = '';
            switch (status) {
                case 0:
                    status = '未读', statusClass = 'fastim-color-blue';
                    break;
                case 1:
                    status = '已读', statusClass = '';
                    break;
                case 4:
                    status = '失败', statusClass = 'fastim-color-red send-message-fail fail-message-' + messageId;
                    break;
                case 5:
                    status = readNumber + '人已读', statusClass = '';
                    break;
                case 6:
                    status = '全部已读', statusClass = '';
                    break;
                default:
                    status = '', statusClass = 'fastim-hidden';
                    break;
            }

            var newEl = '<div class="fastim-message-status ' + statusClass + '">' + status + '</div>';
            if (isUpdate) {
                $('.chat-record-item[data-id="' + messageId + '"] .fastim-message-status').replaceWith(newEl);
            } else {
                return newEl;
            }
        },
        /**
         * 未读消息数量处理
         * @param  {int} sessionId            会话ID
         * @param  {int} unreadMessagesNumber 未读消息数
         * @param  {int} shield               是否屏蔽
         */
        unreadMessagesDisplay: function (sessionId, unreadMessagesNumber, shield = false) {
            var sessionElement = $('.fastim-session-li[data-data="' + sessionId + '"]')
            if (sessionElement.length) {
                if (parseInt(unreadMessagesNumber) > 0) {
                    if (sessionElement.find(".session-unread-count").length) {
                        sessionElement.find(".session-unread-count").eq(0).html(unreadMessagesNumber)
                    } else {
                        shield = shield ? 'session-shield' : ''
                        sessionElement.find(".session-last-message").eq(0).after('<span class="session-unread-count ' + shield + '">' + unreadMessagesNumber + '</span>')
                    }
                } else {
                    sessionElement.find(".session-unread-count").remove()
                    sessionElement.find(".last-message-fixed").eq(0).html('')
                }
            }
        },
        editEscape: function (val, type = 'remove') {
            if (type == 'remove') {
                FastIm.escape.delete(val)
            } else if (type == 'add') {
                FastIm.escape.add(val)
            } else if (type == 'empty') {
                FastIm.escape = new Set(['fastim'])
            } else if (type == 'demand') {
                FastIm.escape = new Set(['fastim', val])
            }
        },
        /**
         * 申请浏览器通知权限
         */
        applyNotificationAuth: function () {
            FastIm.notificationStatus = false;
            const NotificationInstance = Notification || window.Notification;
            if (!!NotificationInstance) {
                const permissionNow = NotificationInstance.permission;
                if (permissionNow == 'granted') {
                    FastIm.notificationStatus = true;
                } else if (permissionNow === 'default') {
                    // 申请权限
                    NotificationInstance.requestPermission().then(permission => {
                        if (permission === 'granted') {
                            FastIm.notificationStatus = true;
                        } else if (permission === 'default') {
                            // 用户关闭授权,下次再申请
                        } else {
                            // denied
                            console.log('用户拒绝授权,不能显示通知');
                        }
                    });
                } else {
                    // denied 用户拒绝
                    console.log('用户曾经拒绝显示通知或非https');
                }
            }
        },
        /**
         * 图像URL处理
         * @param  {[type]} url 图片URL
         * @return {str}        完整的图片url
         */
        imgUrl: function (url) {
            var ret = /^http(s)?:\/\//;
            var retDataImage = /^data:image/;
            if (ret.test(url) || retDataImage.test(url)) {
                return url;
            } else {
                if (FastIm.config.__CDN__) {
                    return FastIm.config.__CDN__ + url;
                } else {
                    return FastIm.build.url('default') + url;
                }
            }
        },
        /**
         * 插入node到消息输入框
         * @param  待插入node
         */
        insertToMessage: function (obj) {
            var messageEl = $('#fastim-message')[0]
            messageEl.focus()
            var selection = getSelection()
            var range = selection.getRangeAt(0)
            range.insertNode(obj)
            range.collapse(false)
            $('.chat-window-input-bottom .im-button').addClass('fastim-bg-blue')
        },
        /**
         * 消息输入框光标移动到最后
         */
        messageCursorToEnd: function () {
            $('#fastim-message').trigger('focus')
            var obj = $('#fastim-message')[0];
            if (window.getSelection) {
                var range = window.getSelection();
                range.selectAllChildren(obj);
                range.collapseToEnd();
            } else if (document.selection) {
                var range = document.selection.createRange();
                range.moveToElementText(obj);
                range.collapse(false);
                range.select();
            }
        },
        /**
         * 消息输入框添加换行
         */
        messageInputLineFeed: function () {
            var messageEl = $('#fastim-message'),
                message = messageEl.html();
            if (FastIm.common.browserType() == "IE" || FastIm.common.browserType() == "Edge") {
                messageEl.html(message + "<div></div>");
            } else if (FastIm.common.browserType() == "FF") {
                messageEl.html(message + "<br /><br />");
            } else {
                messageEl.html(message + "<div><br /></div>");
            }
            FastIm.common.messageCursorToEnd();
        },
        /**
         * 对htmlTemplate进行解析
         * 一行只转换一次变量值，支持"."
         * @param  {obj} funObj htmlTemplate方法
         */
        parseHTMLTemplate: function (funObj) {
            return function (scope) {
                return funObj.toString().match(/\/\*([\s\S]*?)\*\//)[1].replace(/\$\{\w.+\}/g, function (variable) {
                    var value = scope;
                    variable = variable.replace('${', '').replace('}', '').split('.');
                    for (var i = 0; i < variable.length; i++) {
                        if (value[variable[i]]) {
                            value = value[variable[i]]
                        } else {
                            return '';
                        }
                    }
                    return value;
                });
            }
        },
        copyText: function (text) {
            var textarea = document.createElement("textarea");
            var currentFocus = document.activeElement;
            document.body.appendChild(textarea);
            textarea.value = text;
            textarea.focus()
            if (textarea.setSelectionRange) {
                textarea.setSelectionRange(0, textarea.value.length);
            } else {
                textarea.select();
            }
            try {
                var flag = document.execCommand("copy");
            } catch (eo) {
                var flag = false;
            }
            document.body.removeChild(textarea);
            currentFocus.focus()
            return flag;
        },
        /**
         * 获得当前的时间
         * @return {[str]} 类似00:00
         */
        getHoursMinutes: function () {
            var dateObj = new Date();
            var hours = dateObj.getHours();
            hours = hours < 10 ? '0' + hours : hours;
            var minutes = dateObj.getMinutes();
            minutes = minutes < 10 ? '0' + minutes : minutes;
            return hours + ':' + minutes;
        },
        /**
         * 当前网页窗口是否隐藏
         * @return {bool}
         */
        windowHidden() {
            var hidden;
            if (typeof document.hidden !== "undefined") {
                hidden = "hidden";
            } else if (typeof document.mozHidden !== "undefined") {
                hidden = "mozHidden";
            } else if (typeof document.msHidden !== "undefined") {
                hidden = "msHidden";
            } else if (typeof document.webkitHidden !== "undefined") {
                hidden = "webkitHidden";
            }
            return document[hidden];
        },
        stopPropagation: function (e) {
            e = e || window.event;
            if (e.stopPropagation) { //W3C阻止冒泡方法
                e.stopPropagation();
            } else {
                e.cancelBubble = true; //IE阻止冒泡方法
            }
        },
        browserType: function () {
            var userAgent = navigator.userAgent; //浏览器的userAgent字符串
            var isOpera = false;
            if (userAgent.indexOf('Edge') > -1) {
                return "Edge";
            }
            if (userAgent.indexOf('.NET') > -1) {
                return "IE";
            }
            if (userAgent.indexOf("Opera") > -1 || userAgent.indexOf("OPR") > -1) {
                isOpera = true;
                return "Opera"
            }
            if (userAgent.indexOf("Firefox") > -1) {
                return "FF";
            }
            if (userAgent.indexOf("Chrome") > -1) {
                return "Chrome";
            }
            if (userAgent.indexOf("Safari") > -1) {
                return "Safari";
            }
            if (userAgent.indexOf("compatible") > -1 && userAgent.indexOf("MSIE") > -1 && !isOpera) {
                return "IE";
            }
        },
        isPc: function () {
            var is_touch = !!("ontouchstart" in window);
            if (!is_touch) {
                return true;
            }
            var userAgentInfo = navigator.userAgent;
            var Agents = [
                "Android",
                "iPhone",
                "SymbianOS",
                "Windows Phone",
                "iPad",
                "iPod"
            ];
            var flag = true;
            for (var v = 0; v < Agents.length; v++) {
                if (userAgentInfo.indexOf(Agents[v]) > 0) {
                    flag = false;
                    break;
                }
            }
            return flag;
        }
    },
    template: {
        atUserItem: function () {
            return FastIm.common.parseHTMLTemplate(function () {
                /*<div data-action="confirm-at" data-data="${id}" class="fastim-at-user-item fastim-click">
                    <img
                    class="fastim-at-user-avatar"
                    src="${avatar}"
                    alt="${nickname}" />
                    <span class="fastim-at-user-nickname">${nickname}</span>
                </div>*/
            });
        },
        serviceMessage: function () {
            // 服务消息模板,目前支持：好友申请、发送给管理员的入群申请、邀请入群(管理员/被邀请人)申请消息
            return FastIm.common.parseHTMLTemplate(function () {
                /*<div class="service-message-item">
                    <div class="service-message-info">
                        <img
                        data-action="get-info"
                        data-type="${avatar_type}"
                        data-session-user="${avatar_session_user}"
                        draggable="false" class="service-message-info-avatar fastim-click" src="${avatar}" />
                        <div class="service-message-info-right">
                            <div>
                            ${title}
                            <span class="service-message-source"> ${source}</span></div>
                            <div class="service-message-row">
                                <img draggable="false" src="${gender_icon}">
                                <span>${status}</span>
                            </div>
                            <div class="service-message-note">附加消息：<span>${note}</span></div>
                        </div>
                    </div>
                    <div class="service-message-option">
                        <button
                        data-action="${action}"
                        data-id="${id}"
                        data-sender="${sender_id}"
                        class="fastim-click im-button ${option}"
                        data-data="agree">同意</button>

                        <button
                        data-action="${action}"
                        data-id="${id}"
                        data-data="refuse"
                        class="im-button im-button-grey fastim-click ${option}">拒绝</button>

                        <div class="fastim-color-gray">${result}</div>
                    </div>
                </div>*/
            });
        },
        newContactInfo: function () {
            // 新联系资料
            return FastIm.common.parseHTMLTemplate(function () {
                /*<div class="new-contact-apply-info">
                    <div class="new-contact-apply-info-box">
                        <img draggable="false" class="new-contact-apply-avatar" src="${avatar}">
                        <div class="new-contact-apply-row">
                            <img draggable="false" src="${icon}" />
                            <span>${other}</span>
                        </div>
                        <div
                        title="${nickname}"
                        class="new-contact-apply-nickname">${nickname}</div>
                    </div>
                </div>*/
            });
        },
        // 消息内容的html模板
        message: function () {
            return FastIm.common.parseHTMLTemplate(function () {
                /*<li
                class="chat-record-item ${messageData.sender}"
                data-id="${messageData.id}"
                data-type="${messageData.type}">
                    <div
                    data-action="get-info"
                    data-type="${sessionInfo.pushUser.type}"
                    class="record-avatar-img fastim-click"
                    data-session-user="${sessionInfo.pushUser.id}">
                        <img draggable="false" src="${sessionInfo.pushUser.avatar}" />
                    </div>
                    <div class="record-content-box ${messageData.record_content_before}">
                        <div class="chat-record-nickname ${sessionInfo.pushUser.hidden_nickname}">
                        ${sessionInfo.pushUser.nickname}
                        </div>
                        <div class="record-content">${messageData.message}</div>
                    </div>
                    ${messageData.status}
                </li>*/
            });
        },
        // 会话项的html模板
        session: function () {
            return FastIm.common.parseHTMLTemplate(function () {
                /*<li
                data-data="${sessionInfo.id}"
                data-type="${sessionInfo.type}"
                data-action="change-session"
                class="fastim-click fastim-session-li ${sessionInfo.top}">
                    <img
                    draggable="false"
                    class="session-avatar ${sessionInfo.pushUser.avatar_gray}"
                    src="${sessionInfo.pushUser.avatar}">
                    <div class="session-info-right">
                        <div class="session-info-item">
                            <span class="session-info-name">${sessionInfo.pushUser.nickname}</span>
                            <span class="session-info-time fastim-color-gray">${lastMessage.last_time}</span>
                        </div>
                        <div class="session-info-item">
                            <span class="session-last-message">
                            ${sessionInfo.pushUser.user_status}
                            <span class="last-message-text">
                                <span class="last-message-fixed">${lastMessage.unread_fixed_msg.message}</span>
                                <span class="last-message">${lastMessage.last_message}</span>
                            </span>
                            </span>
                            ${unreadMessagesNumber}
                        </div>
                    </div>
                </li>*/
            });
        }
    },
    socket: {
        /*
         * 连接WebSocket
         * socketTask.readyState:0=连接中,1=已经连接,2=连接正在关闭,3=连接已经关闭或没有连接成功
         */
        connect: function () {
            if ((FastIm.ws.socketTask && (FastIm.ws.socketTask.readyState == 0 || FastIm.ws.socketTask.readyState == 1)) || !FastIm.ws.needReconnect) {
                return false;
            }

            if (window.location.protocol == 'https:' && parseInt(FastIm.config.wss_switch) != 1) {
                // FastIm.pop.absolutePrompt(FastIm.config.im_name + ' https下须创建wss服务才能连接网络！', 'error', false)
                // return false;
            }

            if ("WebSocket" in window) {
                // var ws = new WebSocket(FastIm.build.url('ws'));
                var ws = new WebSocket('wss://jx.xicheda.com/');
                FastIm.ws.socketTask = ws;

                ws.onopen = function () {
                    // 重新发送所有出错的消息
                    if (FastIm.ws.errorMsg.length > 0) {

                        for (let i in FastIm.ws.errorMsg) {
                            FastIm.socket.send(FastIm.ws.errorMsg[i]);
                        }

                        FastIm.ws.errorMsg = [];
                    }

                    // 重置重连次数
                    FastIm.ws.currentReconnectCount = 0;
                    FastIm.ws.needReconnect = true;

                    if (FastIm.ws.timer != null) {
                        clearInterval(FastIm.ws.timer);
                    }

                    FastIm.ws.timer = setInterval(function () {
                        FastIm.socket.send({c: 'ImBase', a: 'ping'});
                    }, 28000); //定时发送心跳
                };

                ws.onmessage = function (evt) {
                    var msg = $.parseJSON(evt.data);
                    FastIm.onMessage(msg);
                };

                ws.onclose = function (e) {

                    if (FastIm.ws.timer != null) {
                        clearInterval(FastIm.ws.timer);
                    }

                    if (FastIm.ws.needReconnect) {
                        FastIm.pop.hideMsg();
                        FastIm.pop.inWindowMsg(FastIm.config.im_name + ' 网络不给力，正在自动重连。', 'error', false)
                    }

                    FastIm.socket.reconnect(); // 重连

                    $('.fastim-orbit').removeClass('link-success')
                };

                ws.onerror = function (e) {
                    // 错误
                    console.error('websocket 错误:', e)
                    FastIm.pop.absolutePrompt(FastIm.config.im_name + ' Websocket 发生错误！', 'error', false)
                };
            } else {
                FastIm.pop.absolutePrompt(FastIm.config.im_name + ' 您的浏览器不支持 WebSocket！', 'error', false)
            }
        },
        reconnect: function () {
            if (!FastIm.ws.needReconnect) {
                return false;
            }

            if (FastIm.ws.currentReconnectCount < FastIm.ws.maxReconnectCount) {
                FastIm.ws.currentReconnectCount++;
                if (FastIm.ws.currentReconnectCount == 1) {
                    FastIm.socket.connect();
                    console.log('正在重连 WebSocket 第' + FastIm.ws.currentReconnectCount + '次');
                } else {
                    console.log('6秒后重连 WebSocket 第' + FastIm.ws.currentReconnectCount + '次');
                    FastIm.ws.timer = setTimeout(function () {
                        FastIm.socket.connect();
                        console.log('正在重连 WebSocket 第' + FastIm.ws.currentReconnectCount + '次');
                    }, 6000)
                }
            } else {
                if (FastIm.ws.timer != null) {
                    clearInterval(FastIm.ws.timer);
                }

                console.log('18秒后将再次尝试重连 WebSocket')
                FastIm.ws.timer = setTimeout(FastIm.socket.connect, 18000); //每18秒重新连接一次
            }
        },
        send: function (message) {
            if (!message) {
                return;
            }

            let noNeedLogin = [
                'ImBaselogin'
            ];

            if (!noNeedLogin.includes(message.c + message.a) && !FastIm.ws.ready) {
                FastIm.pop.msg(FastIm.config.im_name + ' 请稍等网络连接成功后再试哦~');
                return;
            }

            if (FastIm.ws.socketTask && FastIm.ws.socketTask.readyState == 1) {
                FastIm.ws.socketTask.send(JSON.stringify(message));
            } else {
                console.log(message)
                FastIm.ws.errorMsg.push(message);
            }
        }
    },
    build: {
        /**
         * 构建服务消息的源数据
         * 构建完成后通过服务消息模板动态渲染到页面
         * @return obj
         */
        serviceMessageInfo: function (info, type, extendData = false) {

            var userEl = '',
                resultObj = {
                    'agree': '已同意',
                    'refuse': '已拒绝',
                    'autoagree': '已自动同意'
                };
            info.id = info.messageData.id
            info.sender_id = info.messageData.sender_id
            info.note = info.messageData.message.note
            info.source = (info.messageData.message.source ? '来自' + info.messageData.message.source : '')

            if (extendData.userInfo) {
                // userEl:好友申请时被申请人,群通知是发送人
                userEl = '\
                <span data-session-user="' + extendData.userInfo.id + '"\
                data-action="get-info"\
                class="fastim-color-blue fastim-click">' + extendData.userInfo.nickname + '</span>';
            }

            if (type == 'new_friends_message') {
                // 好友申请
                info.status = extendData.userInfo.status.chinese
                info.avatar = FastIm.common.imgUrl(extendData.userInfo.avatar)
                info.avatar_type = 'user'
                info.avatar_session_user = extendData.userInfo.id
                info.gender_icon = FastIm.common.imgUrl('/assets/addons/fastim/icon/' + extendData.userInfo.gender.value + '.png')
                if (info.messageData.message.method == 'notice') {
                    if (info.messageData.message.result == 'refuse') {
                        if (info.messageData.sender_id == FastIm.userinfo.id) {
                            info.title = '<span class="service-message-title">你已经拒绝了' + userEl + '的好友申请。</span>';
                        } else {
                            info.title = '<span class="service-message-title">' + userEl + '拒绝了你的好友申请。</span>';
                        }
                    } else {
                        info.title = '<span class="service-message-title">' + info.messageData.message.msg + '</span>';
                    }
                    info.messageData.message.result = ''
                    info.option = 'fastim-hidden'
                } else if (info.messageData.message.method == 'apply') {
                    info.action = 'service-message-option'
                    if (info.messageData.sender_id == FastIm.userinfo.id) {
                        info.title = '<span class="service-message-title">申请添加' + userEl + '为好友</span>';
                    } else {
                        info.title = '<span class="service-message-title">' + userEl + '申请添加你为好友</span>';
                    }

                    // 对 result 进行处理
                    if (info.messageData.message.result) {
                        if (resultObj[info.messageData.message.result]) {
                            info.messageData.message.result = resultObj[info.messageData.message.result]
                        } else if (info.messageData.sender_id == FastIm.userinfo.id) {
                            info.messageData.message.result = '等待对方验证'
                        } else {
                            info.messageData.message.result = false
                        }
                    } else {
                        info.messageData.message.result = false
                    }
                }
            } else if ('group_notice') {

                var groupEl = '\
                <span data-type="group"\
                data-action="get-info"\
                data-session-user="' + extendData.groupChatInfo.id + '"\
                class="fastim-color-blue fastim-click">' + extendData.groupChatInfo.nickname + '</span>';

                info.status = extendData.groupChatInfo.user_count + '/' + extendData.groupChatInfo.max_user_count
                info.avatar = FastIm.common.imgUrl(extendData.groupChatInfo.avatar)
                info.avatar_type = 'group'
                info.avatar_session_user = extendData.groupChatInfo.id
                info.gender_icon = FastIm.common.imgUrl('/assets/addons/fastim/icon/group.png')

                // 对 result 进行处理
                if (info.messageData.message.result) {
                    if (resultObj[info.messageData.message.result]) {
                        info.messageData.message.result = resultObj[info.messageData.message.result]
                    } else if (info.messageData.sender_id == FastIm.userinfo.id) {
                        info.messageData.message.result = '等待管理员验证'
                    } else {
                        info.messageData.message.result = false
                    }
                } else {
                    info.messageData.message.result = false
                }

                // 申请入群
                if (info.messageData.type == 'group_apply') {
                    if (info.messageData.sender_id == FastIm.userinfo.id) {
                        info.title = '<span class="service-message-title">申请加入群聊' + groupEl + '</span>';
                    } else {
                        info.action = 'join-group-option'
                        info.title = '<span class="service-message-title">' + userEl + '申请加入群聊' + groupEl + '</span>';
                    }
                } else if (info.messageData.type == 'group_invitation') {
                    // 入群邀请
                    let invitationUserEl = '\
                        <span data-type="user"\
                        data-action="get-info"\
                        data-session-user="' + info.messageData.invitationUser.id + '"\
                        class="fastim-color-blue fastim-click">' + info.messageData.invitationUser.nickname + '</span>';

                    var leader_result = parseInt(info.messageData.message.leader_result),
                        user_result = parseInt(info.messageData.message.user_result);

                    info.messageData.message.result = false
                    if (info.messageData.message.identity == 'admin') {
                        // xx邀请xx加入群聊xx
                        let invitedUserEl = '\
                        <span data-type="user"\
                        data-action="get-info"\
                        data-session-user="' + info.messageData.invitedUser.id + '"\
                        class="fastim-color-blue fastim-click">' + info.messageData.invitedUser.nickname + '</span>';

                        info.title = '<span class="service-message-title">' + invitationUserEl + '邀请' + invitedUserEl + '加入群聊' + groupEl + '</span>';

                        if (user_result == 3) {
                            info.messageData.message.result = '失效'
                        } else if (leader_result == 1 && user_result == 1) {
                            info.messageData.message.result = '已同意'
                        } else if (leader_result == 2) {
                            info.messageData.message.result = '已拒绝'
                        } else if (user_result == 2) {
                            info.messageData.message.result = '用户已拒绝'
                        } else if (leader_result == 1 && user_result == 0) {
                            info.messageData.message.result = '等待被邀请人验证'
                        } else if (leader_result == 0) {
                            info.messageData.message.result = false
                        }
                    } else if (info.messageData.message.identity == 'user') {
                        // xx邀请你加入群聊xx
                        if (user_result == 3) {
                            info.messageData.message.result = '失效'
                        } else if (leader_result == 1 && user_result == 1) {
                            info.messageData.message.result = '已同意'
                        } else if (leader_result == 2) {
                            info.messageData.message.result = '群管理员已拒绝'
                        } else if (user_result == 2) {
                            info.messageData.message.result = '已拒绝'
                        } else if (leader_result == 0 && user_result == 1) {
                            info.messageData.message.result = '等待管理员验证'
                        } else if (user_result == 0) {
                            info.messageData.message.result = false
                        }
                        info.title = '<span class="service-message-title">' + invitationUserEl + '邀请你加入群聊' + groupEl + '</span>';
                    }
                    info.action = 'invitation-joingroup-option'
                } else if (info.messageData.type == 'group_notice') {
                    info.option = 'fastim-hidden'
                    if (info.messageData.message.action && info.messageData.message.action == 'dismiss') {
                        info.title = '<span class="service-message-title">' + groupEl + '</span>';
                        info.messageData.message.result = info.messageData.message.notice
                        info.status = '0'
                    } else if (info.messageData.message.action && info.messageData.message.action == 'quit') {
                        info.title = '<span class="service-message-title">' + userEl + '退出了群聊' + groupEl + '</span>';
                        info.messageData.message.result = false
                    } else if (info.messageData.message.action && info.messageData.message.action == 'refuse') {

                        var leader = parseInt(extendData.groupChatInfo.leader); // 群管理
                        var sender = parseInt(info.messageData.sender_id); // 执行拒绝操作的用户
                        if (FastIm.userinfo.id == parseInt(info.messageData.message.refuse_user)) {
                            if (sender == leader) {
                                info.title = '<span class="service-message-title">管理员拒绝你加入群聊' + groupEl + '</span>';
                            } else if (sender == FastIm.userinfo.id) {
                                info.title = '<span class="service-message-title">你拒绝加入群聊' + groupEl + '</span>';
                            }
                        } else if (FastIm.userinfo.id == leader) {
                            if (sender == leader) {
                                info.title = '<span class="service-message-title">你拒绝了' + userEl + '加入群聊' + groupEl + '</span>';
                            } else {
                                info.title = '<span class="service-message-title">' + userEl + '拒绝加入群聊' + groupEl + '</span>';
                            }
                        } else {
                            if (sender == leader) {
                                info.title = '<span class="service-message-title">管理员拒绝了' + userEl + '加入群聊' + groupEl + '</span>';
                            } else {
                                info.title = '<span class="service-message-title">' + userEl + '拒绝加入群聊' + groupEl + '</span>';
                            }
                        }
                    } else if (info.messageData.message.action && info.messageData.message.action == 'propose') {
                        info.title = '<span class="service-message-title">管理员已将你请出群聊' + groupEl + '</span>';
                    }
                }
            }

            // 不显示验证按钮
            if (info.messageData.message.result && info.messageData.message.result != 'pending') {
                info.option = 'fastim-hidden'
                info.result = info.messageData.message.result
            }
            return info;
        },
        /**
         * 群聊详情的html代码生成
         * @return html
         */
        groupInfo: function (data, method = 'create') {
            // 按条件动态构建，不使用 parseHTMLTemplate

            var info = '<div class="fastim-userinfo-detail fastim-userinfo-form">\
                        <form name="groupinfo" novalidate>\
                        <input type="hidden" name="id" value="' + data.id + '" />\
                        <input type="hidden" name="method" value="' + method + '" />\
                        ';
            info += FastIm.build.widget('im-input', {title: '群名', name: 'nickname', value: data.nickname});
            info += FastIm.build.widget('im-textarea', {title: '群介绍', name: 'bio', value: data.bio});
            info += FastIm.build.widget('im-radio', {
                title: '加群模式',
                name: 'add_mode',
                value: data.add_mode,
                data: [
                    '需管理员审核',
                    '无需审核'
                ]
            });
            info += FastIm.build.widget('im-radio', {
                title: '邀请免审',
                name: 'invite_join_group',
                value: data.invite_join_group,
                data: [
                    '成员邀请好友需审核',
                    '成员邀请好友免审核'
                ]
            });
            info += FastIm.build.widget('im-radio', {
                title: '历史消息',
                name: 'history_message',
                value: data.history_message,
                data: [
                    '不允许新入群用户查看',
                    '允许新入群用户查看'
                ]
            });
            info += FastIm.build.widget('im-radio', {
                title: '成员发言',
                name: 'speak',
                value: data.speak,
                data: [
                    '允许发言',
                    '禁止发言'
                ]
            });
            info += FastIm.build.widget('im-radio', {
                title: '检索设置',
                name: 'retrieval_settings',
                value: data.retrieval_settings,
                data: [
                    '禁用搜素加群',
                    '允许被搜素到'
                ]
            });

            info += '<div class="fastim-center-buttons">';
            info += '<button type="button" data-action="close-window" data-data="fastim-userinfo" class="im-button im-max-button fastim-click im-button-grey">取消</button>';
            info += '<button type="button" data-action="save-groupinfo" class="im-button im-max-button fastim-click">' + (method == 'create' ? '创建群聊' : '保存') + '</button>';
            info += '</div>';
            info += '</form>';
            info += '</div>';
            return info;
        },
        /**
         * 分类型构建消息内容的html
         * @return html
         */
        message: function (data) {
            if (data.type == 'image') {
                return '<img draggable="false" class="record-img" src="' + data.message + '" />'
            } else if (data.type == 'audio') {
                return '<audio src="' + data.message + '" controls>您的浏览器不支持播放音频文件。</audio>' + (data.buildType && data.buildType == 'collection' ? '' : '<div class="audio-tips">音频文件可转发，请注意甄别。</div>');
            } else if (data.type == 'video') {
                return '<video src="' + data.message + '" width="320" height="240" controls>您的浏览器不支持播放视频。</video>';
            } else if (data.type == 'file') {
                var fileMessageEl = FastIm.common.parseHTMLTemplate(function () {
                    /*<a href="${url}">
                        <div class="record-file">
                            <img
                            src="/addons/fastim/index/icon/suffix/${suffix}"
                            alt="${suffix}文件" />
                            <div>
                                <p class="file-name">${suffix}文件</p>
                                <p class="file-size">${size}</p>
                            </div>
                            <div class="down-file">
                                <button class="im-button">下载</button>
                            </div>
                        </div>
                    </a>
                    */
                });
                data.message.size = data.message.size ? data.message.size : '未知大小';
                return fileMessageEl(data.message);
            } else if (data.type == 'link') {
                data.message.link_name = data.message.link_name ? data.message.link_name : data.message.link_url
                if (data.buildType && data.buildType == 'collection') {
                    return '<a class="record-a" target="_blank" href="' + data.message.link_url + '">' + data.message.link_name + ' ' + data.message.link_url + '</a>';
                }
                return '<div class="record-a fastim-click" data-action="fastim-open-link" data-data="' + data.message.link_url + '">' + data.message.link_name + '</div>'
            } else if (data.type == 'kbs_list') {
                var contactCSR = '', kbs = ''
                if (data.message.type == 'distribution-group-select') {
                    // 选择客服代表分组
                    if (data.message.csr_group.length) {
                        contactCSR = '<div class="csr-message-box">';
                        for (var i = 0; i < data.message.csr_group.length; i++) {
                            let iEl = '<span>(' + data.message.csr_group[i].csr_count + ')</span>'
                            contactCSR += '<div class="csr-message-opt csr-message-opt-name fastim-click" data-action="distribution-group-select" data-data="' + data.message.csr_group[i].id + '">' + data.message.csr_group[i].name + iEl + '</div>';
                        }
                        contactCSR += '</div>';
                    } else {
                        contactCSR = '<b>没有可以选择的客服代表分组,请联系管理员添加！</b>';
                    }
                } else if (data.message.type == 'csrs') {
                    // 选择客服代表
                    contactCSR = '<div class="csr-message-box">';
                    for (var i = 0; i < data.message.csrs.length; i++) {

                        contactCSR += '<div class="csr-group-name">' + (data.message.csrs[i].name ? data.message.csrs[i].name : '无分组') + '：</div>';

                        for (var y = 0; y < data.message.csrs[i].csrs.length; y++) {
                            let iEl = '<span class="csr-message-i-el">' + (y + 1) + '、</span>'
                            contactCSR += '<div class="csr-message-opt csr-message-opt-name fastim-click" data-action="open-session" data-type="user" data-data="' + data.message.csrs[i].csrs[y].id + '">' + iEl + data.message.csrs[i].csrs[y].nickname + '</div>';
                        }
                    }
                    contactCSR += '</div>';
                } else if (data.message.type == 'included') {
                    kbs = '<div class="csr-message-box">'
                    for (var i = 0; i < data.message.kbs.length; i++) {
                        let iEl = '<span class="csr-message-i-el">' + (i + 1) + '、</span>'
                        let titleEl = '<span class="csr-message-opt-name">' + data.message.kbs[i].title + '</span>';
                        kbs += '<div class="csr-message-opt fastim-click" data-action="kbs-click" data-data="' + data.message.kbs[i].id + '">' + iEl + titleEl + '</div>'
                    }
                    kbs += '</div>'
                } else if (data.message.type == 'not-included') {
                    contactCSR = '<div class="fastim-color-blue fastim-click manual-csr" data-action="manual-csr">联系人工客服</div>'
                }

                return '<div class="fastim-kbs">' + data.message.title + '</div>' + contactCSR + kbs;
            } else if (data.type == 'group_chat_notice') {
                return '<div class="group-chat-notice-message"><div class="group-chat-notice-title">群公告</div><div class="group-chat-notice-content">' + data.message.content + '</div><hr class="fastim-hr" /><div data-action="show-group-chat-notice" data-data="' + data.message.id + '" class="fastim-click group-chat-notice-footer">查看详情</div></div>';
            } else if (data.type == 'voice') {
                let voice = '<audio id="voice_' + data.id + '" data-id="' + data.id + '" src="' + data.message.url + '">您的浏览器不支持播放音频文件。</audio>';
                voice += '<div data-action="play-voice" data-data="' + data.id + '" data-sender="' + data.sender + '" class="fastim-message-voice fastim-click">';
                if (data.sender == 'me') {
                    voice += '<div>' + data.message.duration + ' "</div><img class="voice-img" id="img_play_' + data.id + '" src="' + FastIm.common.imgUrl('/assets/addons/fastim/img/' + data.sender + '-voice.png') + '" alt="播放语音" />';
                } else {
                    voice += '<img class="voice-img" id="img_play_' + data.id + '" src="' + FastIm.common.imgUrl('/assets/addons/fastim/img/' + data.sender + '-voice.png') + '" alt="播放语音" /><div>" ' + data.message.duration + '</div>';
                    voice += data.voice_playback ? '':'<div class="unread-voice">•</div>';
                }
                voice += '</div>';
                return voice;
            } else {
                return data.message;
            }
        },
        /**
         * 构建组件Html
         * @param  {str} name    组件name
         * @param  {obj} data    传递给组件的数据
         * @param  {obj} extend  扩展数据
         * @return htmlTemplate
         */
        widget: function (name, data, extend) {

            let widgetFun = new Map([
                ['group-chat-notice-item', () => {
                    var tagHtml = '', receipt = '', buttons = '', imgs = ''
                    if (parseInt(data.top) == 1) {
                        tagHtml += '<div class="blue-tag">置顶</div>';
                    }
                    if (parseInt(data.popup) == 1) {
                        tagHtml += '<div class="blue-tag">弹窗展示</div>';
                    }
                    if (data.receipt) {

                        if (!data.receipted) {
                            receipt = '<div data-action="show-group-notice-users" data-type="receipted-number" data-data="' + data.id + '" class="receipted-count fastim-click fastim-hidden" id="receipted-count-' + data.id + '">' + (data.receiptedCount + 1) + '人已确认</div>';
                            receipt += '<button id="group-notice-receipt-' + data.id + '" data-action="group-notice-receipt" data-type="list" data-data="' + data.id + '" type="button" class="im-button fastim-bg-red fastim-click">确认收到</button>';
                        } else {
                            receipt = '<div data-action="show-group-notice-users" data-type="receipted-number" data-data="' + data.id + '" class="receipted-count fastim-click">' + data.receiptedCount + '人已确认</div>';
                        }
                    }

                    buttons = '<button type="button" data-action="show-group-chat-notice" data-data="' + data.id + '" class="im-button fastim-click">查看</button>';
                    if (extend) {
                        // 是群主
                        buttons += '<button type="button" data-action="opt-group-chat-notice" data-type="edit" data-data="' + data.id + '" class="im-button edit-group-chat-notice-button fastim-click">编辑</button>';
                        buttons += '<button type="button" data-action="opt-group-chat-notice" data-type="del" data-data="' + data.id + '" class="im-button fastim-bg-red fastim-click">删除</button>';
                    }

                    imgs = '<div class="im-group-notice-imgs">';
                    for (let i = 0; i < data.images.length; i++) {
                        imgs += '<div class="upload-box-image"><a href="' + data.images[i] + '" target="_blank"><img draggable="false" src="' + data.images[i] + '" /></a></div>';
                    }
                    imgs += '</div>';

                    return '<div class="notice-item">\
                                <div class="notice-item-content">' + tagHtml + data.content + '</div>' + imgs + '\
                                <div class="notice-item-footer">' + data.publisher.nickname_origin + (data.publisher.remark ? '(' + data.publisher.remark + ')':'') + ' 发表于' + data.createtime + '\
                                    <div data-action="show-group-notice-users" data-type="read-number" data-data="' + data.id + '" class="notice-item-footer-read fastim-click">' + data.reading_number + '人已读</div>\
                                    <div class="notice-item-footer-right">' + receipt + buttons + '</div>\
                                </div>\
                            </div>';
                }],
                ['date-three-linkage', () => {
                    return '<div class="fastim-form">\
                                <div class="fastim-form-title">' + data.title + '</div>\
                                <div class="date-three-linkage ' + data.class + '">\
                                    <select name="year"></select>\
                                    <select name="month"></select>\
                                    <select name="day"></select>\
                                </div>\
                            </div>';
                }],
                ['create-new-contact', () => {

                    var {showEl, buttons} = extend;

                    // 构建创建新联系的窗口HTML(加好友/加群/同意好友)
                    var newContactInfo = FastIm.template.newContactInfo(),
                        newHTML;

                    newContactInfo = newContactInfo(data)

                    newHTML = '<div class="fastim-popup new-contact-apply-box">\
                                <div class="new-contact-apply">';
                    newHTML += newContactInfo;
                    newHTML += '<div class="new-contact-apply-right">\
                                <form name="new-contact-apply" novalidate>';

                    let hiddenInput = ['id', 'type', 'action'];
                    for (var i = 0; i < hiddenInput.length; i++) {
                        newHTML += '<input type="hidden" name="' + hiddenInput[i] + '" value="' + data[hiddenInput[i]] + '" />'
                    }

                    if (showEl.group) {
                        newHTML += FastIm.build.widget('im-select', {
                            name: 'new_contact_user_group',
                            title: '选择分组',
                            data: []
                        });
                    }
                    if (showEl.remark) {
                        newHTML += FastIm.build.widget('im-input', {
                            name: 'new_contact_remark',
                            title: '备注名称',
                            value: ''
                        })
                    }
                    if (showEl.note) {
                        newHTML += FastIm.build.widget('im-textarea', {
                            name: 'new_contact_note',
                            title: '验证消息',
                            value: ''
                        })
                    }
                    if (showEl.tis) {
                        newHTML += FastIm.build.widget('im-text', {
                            title: '提示',
                            value: showEl.tis
                        })
                    }
                    newHTML += '<div class="fastim-center-buttons">';
                    for (var i = 0; i < buttons.length; i++) {
                        newHTML += '<button type="button" ' + buttons[i].attr + ' data-action="' + buttons[i].action + '" class="fastim-click im-button ' + buttons[i].class + '">' + buttons[i].title + '</button>';
                    }

                    newHTML += '</div>';
                    newHTML += '</form>';
                    newHTML += '</div>';
                    newHTML += '</div>';
                    newHTML += '</div>';
                    return newHTML;
                }],
                ['fastim-userinfo-box', () => {
                    if (data.type == 'group') {
                        data.groupmenu = '<div class="groupinfo-menu">\
                            <div class="groupinfo-menu-tab fastim-tab active" data-toggle="groupinfo-menu-tab" data-tab="groupinfo-basic"><span>资料</span></div>\
                            <div class="groupinfo-menu-tab fastim-tab" data-toggle="groupinfo-menu-tab" data-tab="groupinfo-member" data-data="' + data.id + '"><span>成员</span></div>\
                            <div class="groupinfo-menu-tab fastim-tab" data-toggle="groupinfo-menu-tab" data-tab="groupinfo-notice" data-data="' + data.id + '"><span>群公告</span></div>\
                            </div>';
                    }
                    if (data.friend && data.id != FastIm.userinfo.id) {
                        data.edit_remark = '<div data-id="' + data.id + '" data-action="edit_remark" data-data="' + data.remark + '" class="fastim-click imedit_userinfo_remark">修改备注</div>';

                        if (data.remark) {
                            data.remark = '(<div>' + data.remark + '</div>)'
                        }
                    }
                    var userinfo = FastIm.common.parseHTMLTemplate(function () {
                        /*<div class="fastim-userinfo-box">
                            <div class="userinfo-detail-box">
                                <img class="userinfo-avatar" src="${avatar}" />
                                <img class="upload-avatar" src="${uploadicon}" />
                                <input id="uploadavatar" class="fastim-upload" title="上传头像" size="1" width="18" type="file" name="uploadavatar" accept="image/*" />
                                <div class="userinfo-detail">
                                    <div class="userinfo-nickname">
                                        <div>${nickname}</div>
                                        ${remark}
                                        ${edit_remark}
                                    </div>
                                    <div class="userinfo-other">${other}</div>
                                    <div class="userinfo-bio">${bio}</div>
                                </div>
                            </div>
                            ${groupmenu}
                        </div>*/
                    });
                    return userinfo(data);
                }],
                ['im-text', () => {
                    return '<div class="fastim-form">\
                                <div class="fastim-form-title">' + data.title + '</div>\
                                <div class="im-text-content">' + (data.value ? data.value : '-') + '</div>\
                            </div>';
                }],
                ['im-select', () => {
                    var obj = '<select name="' + data.name + '">';
                    for (var i = 0; i < data.data.length; i++) {
                        let selected = (parseInt(data.value) == i) ? 'selected' : ''
                        obj += '<option value="' + i + '" ' + selected + '>' + data.data[i] + '</option>'
                    }
                    obj += '</select>';

                    obj = '<div class="fastim-form">\
                            <div class="fastim-form-title">' + data.title + '</div>\
                            ' + obj + '\
                        </div>';
                    return obj;
                }],
                ['im-select-obj', () => {
                    var obj = '<select data-id="' + data.id + '" class="' + data.name + ' ' + (data.imchange ? 'im-change' : '') + '" name="' + data.name + '">';
                    for (var i = 0; i < data.data.length; i++) {
                        let selected = ((data.value).toString() == (data.data[i].id).toString()) ? 'selected' : ''
                        obj += '<option value="' + data.data[i].id + '" ' + selected + '>' + data.data[i].name + '</option>'
                    }
                    obj += '</select>';

                    obj = '<div class="fastim-form">\
                            <div class="fastim-form-title">' + data.title + '</div>\
                            ' + obj + '\
                        </div>';
                    return obj;
                }],
                ['im-input', () => {
                    var value = data.value ? data.value : ''
                    return '<div class="fastim-form">\
                                    <label class="fastim-form-title">' + data.title + '</label>\
                                    <input type="text" name="' + data.name + '" placeholder="请输入' + data.title + '" value="' + value + '" />\
                                </div>';
                }],
                ['im-textarea', () => {
                    return '<div class="fastim-form">\
                                    <label class="fastim-form-title">' + data.title + '</label>\
                                    <textarea rows="3" maxlength="100" placeholder="请输入' + data.title + '" name="' + data.name + '">' + data.value + '</textarea>\
                                </div>';
                }],
                ['im-radio', () => {
                    var obj = '';
                    for (var i = 0; i < data.data.length; i++) {
                        var checked = (parseInt(data.value) == i) ? 'checked' : ''
                        obj += '<label for="' + data.name + '-' + i + '"><input ' + checked + ' id="' + data.name + '-' + i + '" name="' + data.name + '" type="radio" value="' + i + '"> ' + data.data[i] + '</label>';
                    }
                    return '<div class="fastim-form">\
                                    <label class="fastim-form-title">' + data.title + '</label>\
                                    <div class="fastim-form-radio">\
                                        ' + obj + '\
                                    </div>\
                                </div>';
                }],
                ['load-more', () => {
                    return '<div class="fastim-load-more fastim-click" data-action="load-more" data-type="' + extend + '" data-data="' + data + '">查看更多<img class="rotate-90" src="' + FastIm.common.imgUrl('/assets/addons/fastim/icon/right.png') + '" /></div>';
                }],
                ['search-all-item', () => {
                    if (data.remark) {
                        data.nickname = data.remark + ' (' + data.nickname_origin + ')';
                    }
                    return '<div class="search-res-item fastim-click" data-action="open-session" data-type="' + extend + '" data-data="' + data.id + '">\
                                    <img draggable="false" class="session-avatar" src="' + FastIm.common.imgUrl(data.avatar) + '">\
                                    <div class="session-info-right">\
                                        <div class="session-info-item">\
                                            <span class="session-info-name" title="' + data.nickname + '">' + data.nickname + '</span>\
                                        </div>\
                                        <div class="session-info-item">' + (data.groupname ? '来自分组：' + data.groupname : data.last_time) + '</div>\
                                    </div>\
                                </div>';
                }],
                ['pickgroup', () => {
                    if (!FastIm.pickUserGroup.includes(data.groupId.toString())) {
                        FastIm.pickUserGroup.push(data.groupId.toString())
                    }
                    return '\
                        <div class="pick-group fastim-click" data-action="open-pick-group" data-data="' + data.id + '">\
                            <img class="pick-group-status" src="' + FastIm.common.imgUrl('/assets/addons/fastim/icon/right.png') + '" />\
                            <div>' + data.name + '</div>\
                        </div>';
                }],
                ['pickuser', () => {
                    if (!FastIm.pickUserGroup.includes(data.groupId.toString())) {
                        FastIm.pickUserGroup.push(data.groupId.toString())
                    }

                    // 如果已选择,则 checked
                    let checked = (FastIm.pickChecked && FastIm.pickChecked.has(data.id.toString())) ? true : false;
                    let inputEl = '<input type="checkbox" class="pick-checkbox impick-value-' + data.id + '" id="pick-' + data.groupId + '-item-' + data.id + '" name="pick-item" value="' + data.id + '" ' + (checked ? 'checked' : '') + ' />';
                    var nickname = ''
                    if (data.remark) {
                        nickname = data.remark + ' (' + data.nickname_origin + ' ID:' + data.id + ')';
                    } else {
                        nickname = data.nickname + ' (ID:' + data.id + ')';
                    }
                    return '\
                        <label class="pick-item group-' + data.groupId + '" for="pick-' + data.groupId + '-item-' + data.id + '">\
                            <img class="pick-item-avatar" src="' + FastIm.common.imgUrl(data.avatar) + '">\
                            <div class="pick-item-nickname" title="' + nickname + '">' + nickname + '</div>\
                            ' + inputEl + '\
                        </label>';
                }],
                ['user-card', () => {
                    let userCard = FastIm.common.parseHTMLTemplate(function () {
                        /*<div class="user-card" user-card-${idtype}>
                            <div class="user-card-top">
                                <img
                                data-type="${type}"
                                data-session-user="${id}"
                                data-action="get-info"
                                draggable="false" class="user-card-avatar fastim-click" src="${avatar}" />

                                <div class="user-card-info">
                                    <div class="user-card-info-top">
                                        <div
                                        data-type="${type}"
                                        data-session-user="${id}"
                                        data-action="get-info"
                                        class="user-card-nickname fastim-click">${nickname}</div>
                                        ${button}
                                    </div>
                                    <div class="user-card-row">
                                        <img
                                        draggable="false"
                                        title="${gender_title}"
                                        src="${gender_icon}" />
                                        <span>${status}</span>
                                    </div>
                                    <div
                                    class="user-card-note"
                                    title="${bio}"
                                    >${bio}</div>
                                </div>
                            </div>
                        </div>*/
                    });
                    return userCard(data);
                }],
                ['upload-box-image', () => {
                    let uploadBoxImage = FastIm.common.parseHTMLTemplate(function () {
                        /*<div class="upload-box-image">
                            <a target="_blank" href="${fullurl}">
                                <img draggable="false" src="${fullurl}" />
                            </a>
                            <button type="button"
                                data-action="del-tempimage"
                                data-data="${fullurl}"
                                class="im-button fastim-click im-button-grey"
                            >删除</button>
                        </div>*/
                    });
                    return uploadBoxImage(data);
                }],
                ['confirm', () => {
                    let confirm = FastIm.common.parseHTMLTemplate(function () {
                        /*<div class="fastim-confirm" style="${style}">
                                <button type="button" data-action="close-window" data-data="fastim-confirm" class="fastim-close fastim-confirm-close fastim-icon">
                                    <span aria-hidden="true">×</span>
                                </button>
                                <div>${msg}</div>
                                <div class="fastim-center-buttons">
                                    <button type="button" class="im-button fastim-confirm-ok">确定</button>
                                    <button type="button" data-action="close-window" data-data="fastim-confirm" class="im-button im-button-grey fastim-click">取消</button>
                                </div>
                            </div>*/
                    });
                    return confirm(data);
                }],
                ['fast-reply-item', () => {
                    let replyItem = FastIm.common.parseHTMLTemplate(function () {
                        /*<div class="fastim-titem">
                            <div class="fastim-titem-flex7">${title}</div>
                            <div class="fastim-titem-flex2">${status}</div>
                            <div class="fastim-titem-flex3">
                                <button data-action="edit-reply-item" data-data="${id}" class="fastim-click im-button">
                                    <img src="${cdnurl}/assets/addons/fastim/icon/pen.png" />
                                </button>
                                <button data-action="del-reply-item" data-data="${id}" class="fastim-click im-button fastim-bg-red">
                                    <img src="${cdnurl}/assets/addons/fastim/icon/del.png" />
                                </button>
                            </div>
                        </div>*/
                    });
                    data.cdnurl = FastIm.config.__CDN__ ? FastIm.config.__CDN__ : FastIm.build.url('default')
                    data.status = (parseInt(data.status) == 1) ? '启用' : '关闭';
                    return replyItem(data);
                }],
                ['popup-msg', () => {
                    let msg = FastIm.common.parseHTMLTemplate(function () {
                        /*<div class="fastim-window-msg">
                                <div class="fastim-window-msg-box">
                                    <div aria-hidden="true" data-action="close-fastim-pop-msg" class="fastim-pop-msg-close fastim-click">×</div>
                                    <img draggable="false" src="${icon}">
                                    <div class="fastim-window-msg-text">${msg}</div>
                                </div>
                            </div>*/
                    });
                    return msg(data)
                }],
                ['to-do-item', () => {
                    let todoitem = FastIm.common.parseHTMLTemplate(function () {
                        /*<div class="to-do-item">
                            <input
                            type="checkbox"
                            title="点击完成"
                            class="to-do-checkbox fastim-click"
                            data-action="to-do-checkbox"
                            data-data="${id}"
                            name="to-do-checkbox"
                            ${checked} />
                            <div
                            class="fastim-click"
                            data-data="${id}"
                            data-action="edit-to-do">${title}</div>
                        </div>*/
                    });
                    return todoitem(data);
                }],
            ]);
            let action = widgetFun.get(name) || widgetFun.get('default')
            return action.call(this);
        },
        url: function (type = 'default') {
            var protocol = window.location.protocol + '//';
            var port = window.location.port;
            port = port ? ':' + port : '';

            // 外站调用时,从缓存中读取游客token
            var im_tourists_token = localStorage.getItem('im_tourists_token');
            im_tourists_token = im_tourists_token ? im_tourists_token : ''

            var buildFun = new Map([
                ['initialize', () => {
                    return protocol + FastIm.url + port + '/addons/fastim/index/initialize?referrer=' +
                        document.referrer + '&url=' + document.URL + '&im_tourists_token=' + im_tourists_token;
                }],
                ['ws', () => {
                    protocol = parseInt(FastIm.config.wss_switch) == 1 ? 'wss://' : 'ws://';
                    return protocol + FastIm.url + ':' + FastIm.config.websocket_port;
                }],
                ['upload', () => {
                    var url = ''
                    if (FastIm.config.upload.uploadurl) {
                        url = FastIm.config.upload.uploadurl;
                    } else {
                        url = protocol + FastIm.url + port + '/addons/fastim/index/upload';
                    }
                    return url + '?im_tourists_token=' + im_tourists_token;
                }],
                ['storage-notify', () => {
                    return protocol + FastIm.url + port + '/addons/' + FastIm.config.upload.storage + '/index/notify';
                }],
                ['spark', () => {
                    if (FastIm.config.__CDN__) {
                        return FastIm.config.__CDN__ + '/assets/addons/fastim/js/spark.js';
                    }
                    return protocol + FastIm.url + port + '/assets/addons/fastim/js/spark.js';
                }],
                ['load_message_prompt', () => {
                    var url = ''
                    if (FastIm.config.upload.cdnurl) {
                        url = FastIm.config.upload.cdnurl + FastIm.config.ringing;
                    } else {
                        url = protocol + FastIm.url + port + '/addons/fastim/index/loadMessagePrompt';
                    }
                    return url + '?im_tourists_token=' + im_tourists_token;
                }],
                ['pasteupload.js', () => {
                    if (FastIm.config.__CDN__) {
                        return FastIm.config.__CDN__ + '/assets/addons/fastim/js/jquery.pasteupload.js';
                    }
                    return protocol + FastIm.url + port + '/assets/addons/fastim/js/jquery.pasteupload.js';
                }],
                ['full-screen', () => {
                    return protocol + FastIm.url + port + '/addons/fastim/index/fullScreen';
                }],
                ['default', () => {
                    return protocol + FastIm.url + port
                }]
            ]);

            let action = buildFun.get(type) || buildFun.get('default')
            return action.call(this);
        }
    }
};
